#include <algorithm>
#include <iostream>
#include <wx/tokenzr.h>
#include "chainbox.h"

ChainBox::ChainBox (
    wxWindow *parent,
    int id,
    const std::vector<wxString> &chainVectorParameter ) :
        wxListBox ( parent, id ), chainVector ( chainVectorParameter )
{
    size_t vectorSize = chainVector.size();
    for ( size_t i = 0; i < vectorSize; i++ )
        Insert ( chainVector.at ( i ), i );
    SetSelection ( 0 );
}

void ChainBox::getChainVector ( std::vector<wxString> *ret )
{
    ret->clear();

    size_t count = GetCount();

    for ( size_t i = 0; i < count; i++ )
        ret->push_back ( GetString ( i ) );
}

void ChainBox::clone()
{
    int i = GetSelection();
    if ( i == wxNOT_FOUND )
        return;

    wxString selection = GetStringSelection();
    Freeze();
    Insert ( selection, ++i );
    SetSelection (
        ( (size_t )i < GetCount() ) ? i : GetCount() - 1
    );
    Thaw();
}

void ChainBox::clear()
{
    Clear();
}

void ChainBox::moveDown()
{
    int i = GetSelection();
    if ( i == wxNOT_FOUND || (size_t)i >= GetCount() - 1 )
        return;

    wxString selection = GetStringSelection();
    Freeze();
    Insert ( selection, i + 2 );
    Delete ( i );
    SetSelection( i + 1 );
    Thaw();
}

void ChainBox::moveUp()
{
    int i = GetSelection();
    if ( i == wxNOT_FOUND || !i )
        return;

    SetSelection ( i - 1 );
    moveDown();
    SetSelection ( i - 1 );
    Thaw();
}

void ChainBox::remove()
{
    int i = GetSelection();
    if ( i == wxNOT_FOUND )
        return;

    Delete ( i );

    size_t count = GetCount();

    if ( !count )
        return;

    SetSelection (
        ( (size_t)i < GetCount() ) ? i : GetCount() - 1
    );
}

void ChainBox::restore()
{
    clear();
    Freeze();
    size_t vectorSize = chainVector.size();
    for ( size_t i = 0; i < vectorSize; i++ )
        Insert ( chainVector.at ( i ), i );
    Thaw();
}

wxString ChainBox::saveState ()
{
    size_t count;
    count = GetCount();

    if ( !count )
        return _("");

    wxString ret;
    for ( size_t i = 0; i < count; i++ )
    {
        ret += GetString ( i );
        ret += _T("#");
    }

    return ret;
}

bool ChainBox::getUpdateDelete()
{
    int i = GetSelection();
    if ( i == wxNOT_FOUND )
        return false;
    wxString s = GetStringSelection();
    int limit, count;
    limit = GetCount();
    count = 0;

    for ( int i = 0; i < limit; i++ )
    {
        if ( GetString ( i ) == s )
        {
            count++;
            if ( count > 1 )
                return true;
        }
    }
    return false;
}

void ChainBox::loadState ( const wxString& state )
{
    wxStringTokenizer tokenizer( state, _T("#") );
    std::vector<wxString> nameVector;

    while ( tokenizer.HasMoreTokens() )
    {
        nameVector.push_back ( tokenizer.GetNextToken() );
    }

    size_t nameVectorSize = nameVector.size();
    size_t chainVectorSize = chainVector.size();

    wxString currentName;
    bool found = false;
    for ( size_t i = 0; i < chainVectorSize; i++ )
    {
        currentName = chainVector.at ( i );
        for ( size_t j = 0; j < nameVectorSize; j++ )
        {
            found = currentName.IsSameAs ( nameVector.at ( j ) );
            if ( found )
                break;
        }
        if ( !found )
            nameVector.push_back ( currentName );
    }

    Freeze ();
    clear();
    nameVectorSize = nameVector.size();
    for ( size_t i = 0; i < nameVectorSize; i++ )
    {
        Insert ( nameVector.at ( i ), i );
    }
    Thaw ();
}

bool ChainBox::hasSelection()
{
    return ( GetSelection() != wxNOT_FOUND );
}
