#include <algorithm>

#include <wx/window.h>

#include "effectsgrid.h"


EffectsGrid::EffectsGrid (
    wxWindow *parent,
    int id,
    const std::vector<wxString> &v,
    size_t colNo,
    int cellHeight,
    int cellWidth,
    bool verticalConstraint ) :
        MonochromeGrid ( parent, id, v.size(), colNo, cellHeight, cellWidth, verticalConstraint )
{
    SetColLabelSize ( 0 );
    for ( size_t i = 0; i < v.size(); i++ )
    {
        SetRowLabelValue ( i, v.at ( i ) );
    }
    effectsVector = v;
}

int EffectsGrid::getEffectsRowNumber ( wxString name )
{
    int rowCount = GetNumberRows ( ) ;
    for ( int i = 0; i < rowCount; i++ )
    {
        if ( GetRowLabelValue( i ) == name )
            return i;
    }
    return -1;

}

void EffectsGrid::getNamedPositionVector ( wxString name, std::vector<size_t>& positionVector )
{
    int i = getEffectsRowNumber ( name );
    if ( i == -1 )
        return;
    positionVector.clear();
    int colCount = GetNumberCols ( ) ;

    for ( int j = 0; j < colCount; j++ )
    {
        if ( isSelected ( i, j ) )
            positionVector.push_back ( ( size_t ) j );
    }
}

void EffectsGrid::setNamedPositionVector ( wxString name, std::vector<size_t>& positionVector )
{
    int i = getEffectsRowNumber ( name );
    if ( i == -1 )
        return;

    BeginBatch ();
    clearRow ( i );
    int colCount = GetNumberCols ( ) ;
    std::vector<size_t>::iterator it;
    for ( it = positionVector.begin(); it != positionVector.end(); it++ )
    {
        if ( (*it) < (size_t) colCount )
            setValue ( true, i, (int)(*it) );
    }
    EndBatch ();
}

void EffectsGrid::createRandomPositionVector ( std::vector<size_t>& positionVector, size_t length )
{
    std::vector<bool> boolVector;
    int colCount = GetNumberCols ( ) ;
    if ( length > (size_t)colCount )
        return;
    for ( size_t i = 0; i < (size_t) colCount; i++ )
    {
        boolVector.push_back ( ( i < length ) ? true : false );
    }
    std::random_shuffle ( boolVector.begin (), boolVector.end ());

    positionVector.clear();
    for ( size_t j = 0; j < boolVector.size(); j++ )
    {
        if ( boolVector.at ( j ) )
            positionVector.push_back ( j );
    }
}

void EffectsGrid::getEffectsVector ( std::vector<wxString>& v )
{
    v = effectsVector;
}
