#include <cmath>
#include <stdlib.h>

#include "functor.h"
#include "samplemath.h"
//#include <iostream>

//OK!!!
double heavySideStepFunction  ( double d )
{
    return ( 1.0 + SampleMath::sign ( d ) ) / 2.0;
}

//OK!!!
double rect ( double d )
{
    return heavySideStepFunction ( d + PI) * heavySideStepFunction ( PI - d );
}

//OK!!!
double square ( double d )
{
    double tmp =  SampleMath::sign ( sin ( d ) );
    return ( tmp > 0 ) ? tmp : 0;
}

//OK
double aaSquare ( double d )
{
    double tmp =  SampleMath::sign ( sin ( d ) );
    if ( tmp <= 0 )
        return 0;
    double rem = ( d/PI ) - floor (d/PI);
    double lClipAt, rClipat;
    lClipAt = 0.05;//0.1;
    rClipat = 1 - lClipAt;
    if ( rem <= lClipAt )
        return  lConcaveSoftClipper ( rem, 0.0, lClipAt );

    if ( rem <= rClipat )
        return tmp;

    return rConvexSoftClipper ( rem, rClipat, 1.0 );
}

double passThrough ( double d1, double d2, double d3 )
{
    return 1.0;
}

double passThrough ( double d1, double d2 )
{
    return 1.0;
}

double passThrough ( double d1 )
{
    return 1.0;
}

double sampleAndHold ( double d )
{
    int seed, randomNumber;
    seed = floor ( d / ( PI / 2 ) ) + 1;
    srand ( seed );
    randomNumber = rand() % 10 + 1;

    return ( ( double ) randomNumber ) * 0.1;
}

double sweepIn ( double f, double limit )
{
    if ( !limit || ( f > limit ) )
        return 1.0;
    return lConcaveSoftClipper ( f, 0.0, limit );
}

double sweepOut ( double f, double limit )
{
    return sweepIn ( limit - f, limit );
}

//OK
double rConvexSoftClipper ( double toClip, double start, double end )
{
    if ( toClip <= start )
        return 1.0;

    if ( toClip >= end )
        return 0.0;

    return 1.0 - pow ( ( toClip - start ) / ( end - start ) , 2 );
}

double rConcaveSoftClipper ( double toClip, double start, double end )
{
    if ( toClip <= start )
        return 1.0;

    if ( toClip >= end )
        return 0.0;

    double y = 1 - ( toClip - start ) / ( end - start );
    double z = rConvexSoftClipper ( toClip, start, end ) - y;
    return y - z;
}

double lConcaveSoftClipper ( double toClip, double start, double end )
{
    return ( 1.0 - rConvexSoftClipper ( toClip, start, end ) );
}

double lConvexSoftClipper ( double toClip, double start, double end )
{
    return ( 1.0 - rConcaveSoftClipper ( toClip, start, end ) );
}

//OK!!! 0: peak, range: -1 +1
double aaTriangleFunction ( double d )
{
    double tmp, clipAt;
    tmp = 1 - fabs ( d );

    if ( tmp < 0 )
        return 0;

    clipAt = 0.05 * PI / 2 ;
    if ( SampleMath::sign ( d ) )
    {
        if ( tmp > clipAt )
            return  tmp;
        return tmp * lConcaveSoftClipper ( tmp, 0.0, clipAt );
    }
    if ( tmp > clipAt )
        return  tmp;
    return tmp * ( clipAt - lConcaveSoftClipper ( tmp, 0.0, clipAt ));
}

//OK!!! 0: peak, range: -1 +1
double triangleFunction ( double d )
{
    double tmp = 1 - fabs ( d );
    if ( tmp > 0 )
        return tmp;
    return 0;
}

//OK
double triangle ( double d )
{
    double a, t_0, T;
    a = 1;
    t_0 = PI/2;
    T = PI/2;
    double tmp;
    tmp = fabs (d);
    while ( tmp > 2*T)
        tmp -= 2*T;
    return a * triangleFunction ( ( tmp - t_0 ) / T );
}

//OK
double aaTriangle ( double d )
{
    double a, t_0, T;
    a = 1;
    t_0 = PI/2;
    T = PI/2;
    double tmp;
    tmp = fabs (d);
    while ( tmp > 2*T)
        tmp -= 2*T;
    return a * aaTriangleFunction ( ( tmp - t_0 ) / T );
}

//OK
double sawTooth ( double d )
{
    return 0.5 + ( ( d + PI )/(2*PI) - floor ( ( d + PI )/(2*PI) + 0.5 ) );
}

//OK
double aaSawTooth ( double d )
{
    double tmp, rClipAt;
    tmp = 0.5 + ( ( d + PI )/(2*PI) - floor ( ( d + PI )/(2*PI) + 0.5 ) );

    rClipAt = 0.95;

    if ( tmp <= rClipAt )
        return  tmp;

    return tmp * rConvexSoftClipper ( tmp, rClipAt, 1.0 );
}

double sinPlus ( double d )
{
    return 0.5 + 0.5 * sin ( d );
}

//OK
double sinPow2 ( double d )
{
    return pow ( sin ( d ), 2 );
}

double resonance ( double x )
{
    return 1 / ( 1 + pow ( x, 2 ));
}

double fftLowpassFilter (
    double f,
    double bandLo, double slopeLo,
    double bandHi, double slopeHi)
{
    double dist;
    if ( f <= bandHi )
        return 1.0 ;
    else
        dist = ( f - bandHi ) / slopeHi;

    return exp ( -dist * dist ) ;
}

double fftHighpassFilter (
    double f,
    double bandLo, double slopeLo,
    double bandHi, double slopeHi)
{
    double dist;
    if ( f >= bandLo )
        return 1.0 ;
    else
        dist = ( bandLo - f ) / slopeLo;

    return exp ( -dist * dist ) ;
}

double fftBandpassFilter (
    double f,
    double bandLo, double slopeLo,
    double bandHi, double slopeHi)
{
    double dist = 0.0;
    if (( f >= bandLo ) && ( f <= bandHi ))
        return 1.0 ;

    else if ( f > bandHi )
        dist = (f - bandHi) / slopeHi;

    else if ( f < bandLo )
        dist = (bandLo - f) / slopeLo;

    return exp ( -dist * dist ) ;
}

double fftBandstopFilter (
    double f,
    double bandLo, double slopeLo,
    double bandHi, double slopeHi)
{
    if ( ( f < bandLo ) || ( f > bandHi ) )
        return 1.0;

    if (( f >= bandLo ) && ( f <= bandLo + slopeLo ) )
        return rConcaveSoftClipper ( f, bandLo, bandLo + slopeLo );

    if (( f > bandLo + slopeLo ) && ( f < bandHi - slopeHi ))
        return 0.0;

    if (( f >= bandHi - slopeHi ) && ( f <= bandHi ) && ( f > bandLo + slopeLo ))
        return lConcaveSoftClipper ( f, bandHi - slopeHi, bandHi );

    return 1.0;
}


double b3Overdrive ( double d )
{
    return ( flatResponse(d)  );
}

double flatResponse ( double d )
{
    return d;
}

double seventiesFuzzguitar ( double d )
{
    return ( flatResponse(d) * ( -40 ) * rConvexSoftClipper ( d, -20, 0 ) );
}

double tubeDrive ( double d )
{
    return ( flatResponse(d));
}

/*double sinDistorted ( double d, double cutoff, double distortRange, double steepness )
{
    if( cutoff >= 1  ||
        distortRange >= cutoff ||
        cutoff < 0 ||
        distortRange < 0 )
        throw;

    double result = sin ( steepness * d );
    double absResult = fabs ( result );
	double linearRange = cutoff - distortRange;

	if ( ! ( absResult > linearRange ) )
		return 0.5 + (( 1.0 / (2*cutoff) ) * result);

	double distortFactor = distortRange / ( 1 - linearRange );

	double absResultDistorted = linearRange + distortFactor * ( absResult - linearRange );
	return 0.5 + (( 1.0 /( 2*cutoff) ) * SampleMath::sign ( result ) * absResultDistorted);
}


double sinDistortedNp2Np1 ( double d )
{
    double cutoff =         0.00000002;
    double distortRange =   0.00000001999;
    double steepness = 4.0;
    if( cutoff >= 1  ||
        distortRange >= cutoff ||
        cutoff < 0 ||
        distortRange < 0 )
        throw;

    double result = sin ( steepness * d );
    double absResult = fabs ( result );
	double linearRange = cutoff - distortRange;

	if ( ! ( absResult > linearRange ) )
		return (( 1.0 / (2*cutoff) ) * result);

	double distortFactor = distortRange / ( 1 - linearRange );

	double absResultDistorted = linearRange + distortFactor * ( absResult - linearRange );
	return 0.5 + (( 1.0 /( 2*cutoff) ) * SampleMath::sign ( result ) * absResultDistorted);
}

double sinToSquare ( double d )
{
    double f, result;
    result = 0;
    for ( f = 1.0; f < 126; f +=2 )
    {
        result += ( 1.0 / f ) * sin ( f * d );
    }
    result = ( 1.0 / 2.0 ) + ( ( 1.0 / 2.0 ) * result );
    if ( result < 0.15 )
        return result*0.01;
    if ( result > 1.0 )
        return 1.0;
    return result;
}

//http://ccrma.stanford.edu/~jos/pasp/Cubic_Soft_Clipper.html
double cubicSoftClipper ( double d )
{
    if ( fabs ( d > 1))
        return SampleMath::sign ( d ) * ( 2.0 / 3.0 );
	return d - ( pow ( d, 3 ) / 3.0 );
}

double cubicSoftClipping ( double d)
{
    return 0.5 + ( ( 3.0 / 4.0 ) * cubicSoftClipper ( d - 1.0 + PI ) * cubicSoftClipper ( PI - d + 1.0 ));
}

//http://ccrma.stanford.edu/~jos/pasp/Arctangent_Nonlinearity.html
double arcTangentNonLinearity ( double d )
{
    double a = 1;//10; // a >> 1
    if ( fabs ( d > 1))
        //return
	return ( 2.0 / PI ) * atan ( a * d );
}

double arcTangentNonLinear ( double d)
{
    return 0.5 + ( 0.25 * arcTangentNonLinearity ( d + PI ) * arcTangentNonLinearity ( PI - d ));
}*/

