#include "wx/pen.h"
#include <wx/window.h>

#include "samplemath.h"
#include "fxplotwindow.h"

#include <iostream>

BEGIN_EVENT_TABLE( FxPlotWindow, wxWindow )
    EVT_PAINT( FxPlotWindow::OnPaint )
END_EVENT_TABLE()

FxPlotWindow::FxPlotWindow ( wxWindow *parent, int id, const wxPoint& pos, const wxSize& size) :
        wxWindow ( parent, id, pos, size ), height ( size.GetHeight()), width ( size.GetWidth())
{
    data = NULL;
}

void FxPlotWindow::DoPaint( wxClientDC& dc )
{
    updatePoints();

    dc.DrawLines ( ( int ) points.size(), ( wxPoint* ) points.data() );
}

void FxPlotWindow::OnPaint( wxPaintEvent& event )
{
    if ( !points.empty() )
    {
        wxClientDC dc ( this );
        prepareDC ( dc );
        dc.SetPen ( *wxBLACK_PEN );
        DoPaint( dc );
    }
    event.Skip();
}


void FxPlotWindow::prepareDC ( wxClientDC& dc )
{
    dc.SetAxisOrientation( true, true );
    dc.SetDeviceOrigin( 0, ( int ) ( 0.95 * height ) );
}


void FxPlotWindow::set( std::basic_string<double> *dataParameter )
{
    if ( !dataParameter )
        return;

    data = dataParameter;
    wxClientDC dc ( this );
    prepareDC ( dc );

    if ( !points.empty() )
    {
        wxPen pen ( ( const wxColour ) GetBackgroundColour() );
        dc.SetPen ( pen );
        dc.DrawLines ( ( int ) points.size(), ( wxPoint* ) points.data() );
    }

    dc.SetPen ( *wxBLACK_PEN );
    DoPaint( dc );
}


void FxPlotWindow::updatePoints()
{
    points.clear();
    if ( !data )
        return;

    for ( size_t i = 0; i < data->size(); i ++ )
        points.push_back( wxPoint ( i, ( int ) ( 0.9 * ( double ) height * data->at( i ) ) ) );

}

