#include <algorithm>
#include <wx/window.h>
#include "progressgrid.h"

BEGIN_EVENT_TABLE(ProgressGrid, wxGrid)
    EVT_GRID_CELL_LEFT_CLICK( ProgressGrid::OnLeftClick )
    EVT_GRID_CELL_LEFT_DCLICK( ProgressGrid::OnLeftDClick )
    EVT_GRID_LABEL_LEFT_CLICK( ProgressGrid::OnLabelLeftClick )
    EVT_GRID_LABEL_RIGHT_CLICK( ProgressGrid::OnLabelRightClick )
    EVT_GRID_RANGE_SELECT( ProgressGrid::OnGridRangeSelect )
END_EVENT_TABLE()


ProgressGrid::ProgressGrid (
    wxWindow *parent,
    int id,
    int colNoParameter,
    int cellHeight,
    int cellWidth ) :
        MonochromeGrid ( parent, id, 1, colNoParameter, cellHeight, cellWidth )
{
    int colNo = colNoParameter;
    SetColLabelSize ( 0 );
    SetRowLabelValue ( 0, wxEmptyString );
    SetRowMinimalHeight ( 0, 4 );
    EnableGridLines ( false );

    initColours();
    colourVectorSize = ( int ) colourVector.size();

    for ( int i = 0; i < colNo; i++ )
        SetCellBackgroundColour ( 0, i, *wxWHITE );

    setColumn ( 0 );
    memoryColumn = -1;
    reset();
}

void ProgressGrid::initColours()
{
    int colNo = GetNumberCols();
    colourVector.clear();
    colourVector.push_back ( *wxBLACK );

    wxColour grey1 = wxColour ( 248, 248, 248 );
    wxColour grey2 = wxColour ( 240, 240, 240 );
    wxColour grey3 = wxColour ( 224, 224, 224 );
    wxColour grey4 = wxColour ( 192, 192, 192 );

    if ( colNo > 4 && colNo <= 8 )
    {
        colourVector.push_back ( grey1 );
    }
    else if ( colNo > 8 && colNo <= 16 )
    {
        colourVector.push_back ( grey2 );
        colourVector.push_back ( grey1 );
    }
    else if ( colNo > 16 && colNo <= 32 )
    {
        colourVector.push_back ( grey3 );
        colourVector.push_back ( grey2 );
        colourVector.push_back ( grey1 );
    }
    else if ( colNo > 32 )
    {
        colourVector.push_back ( grey4 );
        colourVector.push_back ( grey3 );
        colourVector.push_back ( grey2 );
        colourVector.push_back ( grey1 );
    }
    colourVectorSize = (int ) colourVector.size();
}

void ProgressGrid::clearGrid()
{
    int colNo = GetNumberCols();

    BeginBatch ();
    for ( int i = 0; i < colNo; i++ )
        SetCellBackgroundColour ( 0, i, *wxWHITE );
    EndBatch ();
}

void ProgressGrid::reset()
{
    initColours();

    memoryColumn = -1;
    setColumn ( 0 );
}

void ProgressGrid::setColumn( int col )
{
    if ( col == memoryColumn )
        return;

    int colNo = GetNumberCols();

    BeginBatch ();

    clearGrid();

    // set current selection
    SetCellBackgroundColour ( 0, col, colourVector.at ( 0 ) );

    // paint gradient
    int right, left, limit;
    right = left = 0;
    limit = ( colNo/2 > colourVectorSize ) ? colourVectorSize : colNo/2;
    for ( int i = 1; i < /*colourVectorSize*/limit; i++ )
    {
        right = col + i;
        if ( ( col + i ) < colNo )
            SetCellBackgroundColour ( 0, right, colourVector.at ( i ) );

        left = col - i;
        if ( ( col - i ) >= 0 )
            SetCellBackgroundColour ( 0, left, colourVector.at ( i ) );
    }

    setLabel( col );

    EndBatch ();
}

void ProgressGrid::setLabel( int col )
{
    int colNo = GetNumberCols();

    BeginBatch ();
    wxString sCol;
    sCol.Printf( _T("%i / %i"), col + 1, colNo );
    SetRowLabelValue ( 0, sCol );
    EndBatch ();
}
