#include "ratedialog.h"
#include <wx/stattext.h>
#include <wx/button.h>
#include <wx/textctrl.h>
#include <wx/sizer.h>
#include <wx/msgdlg.h>
#include <wx/url.h>
#include <wx/utils.h>
#include <wx/filename.h>
#include <wx/combobox.h>
#include "urlprovider.h"
#include "smasherdb.h"
#include "httpresource.h"

BEGIN_EVENT_TABLE ( RateDialog, wxDialog )
    EVT_BUTTON ( ID_REGISTER, RateDialog::OnRegister )
    EVT_BUTTON ( wxID_OK, RateDialog::OnOk )
END_EVENT_TABLE()

RateDialog::RateDialog (
    wxWindow *parent,
    const wxString& title,
    wxString& presetParameter,
    wxString& authorParameter,
    wxString& passcodeParameter,
    wxString& emailParameter
) : wxDialog ( parent, wxID_ANY, title ), preset ( presetParameter ), author ( authorParameter ), passcode ( passcodeParameter ), email ( emailParameter )
{
    wxArrayString ratingChoices;
    for ( size_t rating = 1; rating < 6; rating++ )
        ratingChoices.push_back ( getRatingString ( rating ) );

    rateComboBox = new wxComboBox (
        this,
        wxID_ANY,
        _("rate..."),
        wxDefaultPosition,
        wxSize ( 400, -1 ),
        ratingChoices,
        wxCB_READONLY
    );
#ifdef __WXMSW__
    unicodeFont = new wxFont (
        8,
        wxFONTFAMILY_DEFAULT,
        wxFONTSTYLE_NORMAL,
        wxFONTWEIGHT_NORMAL,
        false,
        _T("Arial Unicode MS"),
        wxFONTENCODING_DEFAULT);
    rateComboBox->SetFont( *unicodeFont );
#else
    rateComboBox->SetFont( *wxSMALL_FONT );
#endif

    authorLabel = new wxStaticText ( this, wxID_ANY, _T("user") );
    authorEdit = new wxTextCtrl ( this, wxID_ANY, author, wxDefaultPosition, wxSize ( 400, -1 ) );

    passcodeLabel = new wxStaticText ( this, wxID_ANY, _T("passcode") );
    passcodeEdit = new wxTextCtrl ( this, wxID_ANY, passcode, wxDefaultPosition, wxSize ( 400, -1 ) );
    registerLabel = new wxStaticText ( this, wxID_ANY, _T("email") );
    registerEdit = new wxTextCtrl ( this, wxID_ANY, email, wxDefaultPosition, wxSize ( 240, -1 ) );

    presetPtr = &presetParameter;
    authorPtr = &authorParameter;
    passcodePtr = &passcodeParameter;
    emailPtr = &emailParameter;

    int width, height;
    registerEdit->GetSize ( &width, &height );
    registerButton = new wxButton ( this, ID_REGISTER, _T("request passcode"), wxDefaultPosition, wxSize ( 150, height ) );

    wxBoxSizer *embeddedSizer = new wxBoxSizer ( wxHORIZONTAL );
    embeddedSizer->Add ( registerEdit );
    embeddedSizer->Add ( registerButton );

    wxButton *cancelButton = new wxButton ( this, wxID_CANCEL, _T("&cancel") );
    wxButton *shareButton = new wxButton ( this, wxID_OK, _T("&rate preset") );
    wxBoxSizer *buttonSizer = new wxBoxSizer ( wxHORIZONTAL );
    buttonSizer->Add ( cancelButton );
    buttonSizer->Add ( shareButton );
    //wxSizer *buttonSizer = CreateButtonSizer ( wxID_OK | wxID_CANCEL );

    wxStaticBoxSizer *passcodeSizer = new wxStaticBoxSizer ( wxVERTICAL, this, _T("new user") );
    passcodeSizer->Add ( registerLabel );
    passcodeSizer->Add ( embeddedSizer );

    wxBoxSizer *sizer = new wxBoxSizer ( wxVERTICAL );
    sizer->AddSpacer ( 5 );
    sizer->Add ( rateComboBox, 0, wxRIGHT|wxLEFT, 5 );
    sizer->Add ( authorLabel, 0, wxRIGHT|wxLEFT, 5 );
    sizer->Add ( authorEdit, 0, wxRIGHT|wxLEFT, 5 );
    sizer->AddSpacer ( 5 );
    sizer->Add ( passcodeLabel, 0, wxRIGHT|wxLEFT, 5 );
    sizer->Add ( passcodeEdit, 0, wxRIGHT|wxLEFT, 5 );
    sizer->AddSpacer ( 5 );
    sizer->Add ( passcodeSizer, 0, wxRIGHT|wxLEFT, 5 );
    sizer->AddSpacer ( 5 );

    if ( buttonSizer )
        sizer->Add ( buttonSizer, 0, wxALIGN_RIGHT | wxRIGHT, 0 );

    SetSizer ( sizer );
    sizer->Fit ( this );
    rateComboBox->SetFocus();
}

RateDialog::~RateDialog()
{
#ifdef __WXMSW__
    delete unicodeFont;
#endif
}

void RateDialog::OnOk ( wxCommandEvent& e )
{
    author = authorEdit->GetValue();
    passcode = passcodeEdit->GetValue();

    if ( author.empty() || passcode.empty() )
    {
        wxMessageBox ( _("please fill in the user and passcode fields"),
                       _("can't rate preset"),
                       wxOK,
                       GetParent() );
        return;
    }

    if ( preset.size() > 32 || author.size() > 32 || passcode.size() > 32 )
    {
        wxMessageBox (
            _("the title, user and passcode fields should be no longer than 32 characters"),
            _("can't rate preset"),
            wxOK,
            GetParent() );
        return;
    }

    *presetPtr = preset;
    *authorPtr = author;
    *passcodePtr = passcode;
    int rating = rateComboBox->GetSelection() + 1;

    if ( rating < 1 || rating > 5 )
    {
        wxMessageBox (
            _("rating required"),
            _("can't rate preset"),
            wxOK,
            GetParent() );
        return;
    }

    wxString message;
    if ( !SmasherDb::ratePreset (
                rating,
                preset,
                author,
                passcode,
                message ) )
    {
        wxMessageBox (
            message,//_("check internet connection"),
            _("can't rate preset"),
            wxOK,
            GetParent() );
        return;
    }

    e.Skip();
}

void RateDialog::OnRegister ( wxCommandEvent& e )
{
    author = authorEdit->GetValue();
    email = registerEdit->GetValue();

    if ( author.empty() )
    {
        wxMessageBox (
            _("user name required"),
            _("can't request passcode"),
            wxOK,
            GetParent() );
        return;
    }

    if (
        email.size() < 3 ||
        email.Find ( _("@") ) == wxNOT_FOUND )
    {
        wxMessageBox (
            _("valid email address required"),
            _("can't request passcode"),
            wxOK,
            GetParent() );
        return;
    }

    *emailPtr = email;

    UrlProvider up;

    // check user's access
    std::string baseRegisterUrl = up.getRegisterUrl();
    wxString wideRegisterUrl = wxString ( baseRegisterUrl.c_str(), wxConvUTF8, baseRegisterUrl.size() );

    wideRegisterUrl += _T("?email=") + email + _T("&user=") + author;

    wxString buffer, message;
    HttpResource hr;
    if ( !hr.load ( wideRegisterUrl, &buffer ) )
    {
        message = hr.getError();
        return;
    }

    if ( buffer.size() > 0 && buffer[0] == L'!' )
    {
        wxMessageBox (
            _("user name already in use"),
            _("can't request passcode"),
            wxOK,
            GetParent() );
        return;
    }

    // trigger email
    std::string baseUrl = up.getPasscodeUrl();
    wxString wideUrl = wxString ( baseUrl.c_str(), wxConvUTF8, baseUrl.size() );

    wideUrl += _T("?email=") + email + _T("&user=") + author;

    wxURL url ( wideUrl );

    if ( url.GetError() != wxURL_NOERR )
    {
        wxMessageBox ( _("can't connect to server"),
                       _T("can't rate preset"),
                       wxOK,
                       GetParent()  );

        return;
    }

    wxYield();
    url.GetProtocol().SetTimeout ( 5 );

    wxInputStream *inStream;

    inStream = url.GetInputStream();

    if ( !inStream )
        return;

    wxSleep ( 1 );

}

wxString RateDialog::getRatingString ( size_t i )
{
    wxString ret;
    switch ( i )
    {
    case 1:
        ret = _T("\u2605\u2606\u2606\u2606\u2606");
        break;
    case 2:
        ret = _T("\u2605\u2605\u2606\u2606\u2606");
        break;
    case 3:
        ret = _T("\u2605\u2605\u2605\u2606\u2606");
        break;
    case 4:
        ret = _T("\u2605\u2605\u2605\u2605\u2606");
        break;
    case 5:
        ret = _T("\u2605\u2605\u2605\u2605\u2605");
        break;
    default:
        ret = _T("\u2605\u2605\u2605\u2606\u2606");
        break;
    }
    return ret;
}


