#include <iostream>
#include <cfloat>
#include <climits>
#include <cmath>
#include <cstdio>

#include "samplemath.h"

void SampleMath::joinChannels ( const std::vector<double> *leftChannel, const std::vector<double> *rightChannel, std::vector<double> *slice )
{
    size_t channelSize = leftChannel->size();

    if ( channelSize != rightChannel->size() )
    {
        std::cout << "rightchannelsize != leftchannelsize" << std::endl;
        return;
    }

    slice->clear();
    for ( size_t i = 0; i < channelSize; i++ )
    {
        slice->push_back( leftChannel->at ( i ));
        slice->push_back( rightChannel->at ( i ));
    }
}


void SampleMath::splitChannels ( const std::vector<double> *slice, std::vector<double> *leftChannel, std::vector<double> *rightChannel)
{
    size_t sliceSize = slice->size();
    leftChannel->clear();
    rightChannel->clear();
    for ( size_t i = 0; ( i + 1 ) < sliceSize; i +=2 )
    {
        leftChannel->push_back ( slice->at ( i ));
        rightChannel->push_back ( slice->at ( i + 1 ));
    }
}


long SampleMath::getNextPower2 ( long l )
{
    return ( long ) pow ( 2, ( 1 + floor ( log ( l )/log( 2 ))));
}

short SampleMath::doubleToShort ( double d )
{
    if ( d <= SHRT_MIN - 0.5 )
    {
        return SHRT_MIN;
    }
    else if ( d >= SHRT_MAX + 0.5 )
    {
        return SHRT_MAX;
    }
    if ( d >= 0 )
        return ( short ) ( d + 0.5 );
    return ( short ) ( d - 0.5 );
}

double SampleMath::shortToDouble ( short s )
{
    return ( double ) s;
}


double SampleMath::sign ( double d )
{
    if ( d < 0 )
        return -1.0;
    if ( d == 0 )
        return 0;
    return 1.0;
}

double SampleMath::decibelToAmplitude ( double d )
{
    return ( pow ( 10, d/20.0 ) * DOUBLE_SHRT_MAX );
}

double SampleMath::amplitudeToDecibel ( double d )
{
    double tmp = 20 * log10 ( d ) - 20 * log10 ( DOUBLE_SHRT_MAX );
    if ( tmp < DBL_MIN )
        tmp = DBL_MIN;
    return tmp;
    /*double tmp = d / DOUBLE_SHRT_MAX;
    if ( tmp < DBL_MIN )
        tmp = DBL_MIN;
    return ( 20.0 * log10 ( tmp ) );*/
}

double SampleMath::crushBits ( double d, double currentMax, double crushedMax )
{
    double crushFactor;
    short gain;
    crushFactor = crushedMax/currentMax;
    gain = SampleMath::doubleToShort( currentMax / crushedMax );
    short tmp1 = SampleMath::doubleToShort ( d * crushFactor );
    return SampleMath::shortToDouble ( gain * tmp1 );
}

double SampleMath::getPeakLevel ( std::vector<double> *leftChannel, std::vector<double> *rightChannel )
{
    size_t leftChannelSize, rightChannelSize;
    leftChannelSize = leftChannel->size();
    double peakLevel = 0.0;

    for ( size_t i = 0; i < leftChannelSize; i++ )
    {
        if ( fabs (leftChannel->at (i)) > peakLevel )
            peakLevel = fabs (leftChannel->at (i));
    }

    if ( !rightChannel )
        return peakLevel;

    rightChannelSize = rightChannel->size();
    for ( size_t i = 0; i < rightChannelSize; i++ )
    {
        if ( fabs (rightChannel->at (i)) > peakLevel )
            peakLevel = fabs (rightChannel->at (i));
    }
    return peakLevel;
}

bool SampleMath::isPeak ( double before, double current, double after )
{
    return ( ( fabs( current ) > fabs ( before ) ) && ( fabs ( current ) > fabs ( after ) ) );
}

double SampleMath::getGradient ( std::vector<std::pair<double,double> >& peakVector )
{
    double meanX, meanY, sumXY, sumXX;
    meanX = meanY = sumXY = sumXX = 0;
    double peakVectorLength = ( double ) peakVector.size();
    std::vector<std::pair<double,double> >::iterator it;

    for ( it = peakVector.begin(); it != peakVector.end(); it++ )
    {
        meanX += it->first;
        meanY += it->second;

        //fprintf ( stderr, "it->first: %f\tit->second: %f\n", it->first, it->second);
        //fprintf ( stderr, "meanX: %f\tmeanY: %f\n", meanX, meanY);
    }

    if ( !peakVectorLength )
    {
        fprintf ( stderr, "SampleMath::getGradient\t peakVectorLength = 0\n" );
        fflush ( stderr );
        throw;
    }
    //fprintf ( stderr, "\nmeanX: %f\tmeanY: %f\n", meanX, meanY);

    meanX /= peakVectorLength;
    meanY /= peakVectorLength;

    //fprintf ( stderr, "peakVectorLength: %f\n", peakVectorLength);
    //fprintf ( stderr, "meanX: %f\tmeanY: %f\n\n", meanX, meanY);
    //fprintf ( stderr, "sumXX: %f\tsumXY: %f\n\n", sumXX, sumXY);

    for ( it = peakVector.begin(); it != peakVector.end(); it++ )
    {
        double temp = it->first - meanX;
        sumXY += temp * ( it->second - meanY );
        sumXX += pow ( temp, 2 );
    }

    //fprintf ( stderr, "sumXX: %f\tsumXY: %f\tsumXY/sumXX: %f\n", sumXX, sumXY, sumXY/sumXX);

    //
    return ( ( peakVector.at ( SampleMath::doubleToShort( peakVectorLength ) - 1).second - peakVector.at ( 0 ).second ) / peakVectorLength );
    //

    if (sumXX != 0)
        return sumXY / sumXX;
    fprintf ( stderr, "SampleMath::getGradient\t sumXX = 0\n" );
    fflush ( stderr );
    return 0.0;//throw;
}
