#include <algorithm>
#include <memory>

#include <wx/checkbox.h>
#include <wx/radiobox.h>
#include <wx/slider.h>
#include <wx/treebook.h>
#include <wx/textctrl.h>
#include <wx/window.h>
#include <wx/button.h>
#include <wx/tglbtn.h>
#include <wx/font.h>
#include <wx/listctrl.h>
#include <wx/combobox.h>
#include <wx/font.h>
#include <wx/filename.h>
#include <wx/tokenzr.h>
#include <wx/hyperlink.h>
#include "effectsgrid.h"
#include "warpgrid.h"
#include "chainbox.h"
#include "enum.h"
#include "fxplotwindow.h"
#include "functor.h"
#include "ratedialog.h"
#include "samplemath.h"
#include "settingsbook.h"
#include "smasherMain.h"
#include "smasherdb.h"
#include "smashercopy.h"
#include "dir.h"
#include "binaryfile.h"

BEGIN_EVENT_TABLE( SettingsBook, wxAuiNotebook )
    EVT_IDLE( SettingsBook::OnIdle )
    EVT_LIST_ITEM_ACTIVATED( ID_FACTORY_LIST, SettingsBook::OnListViewItemActivated )
    EVT_LIST_COL_CLICK ( ID_FACTORY_LIST, SettingsBook::OnColumnClick )
    EVT_LIST_COL_CLICK ( ID_DB_LIST, SettingsBook::OnColumnClick )
    EVT_RADIOBOX( ID_TREMOLO_TYPES_BOX, SettingsBook::OnRadioBoxChanged )
    EVT_LISTBOX( ID_WARP_PATTERNS_BOX, SettingsBook::OnListboxChanged )
    EVT_RADIOBOX( ID_MASTER_FILTER_TYPES_BOX, SettingsBook::OnRadioBoxChanged )
    EVT_RADIOBOX( ID_MASTER_FILTER_MOD_TYPES_BOX, SettingsBook::OnRadioBoxChanged )
    EVT_RADIOBOX( ID_MASTER_FILTER_LFO_TYPES_BOX, SettingsBook::OnRadioBoxChanged )
    EVT_CHECKBOX( ID_PLAYBACK_COMPENSATION_CHECKBOX, SettingsBook::OnCheckboxChanged )
    EVT_CHECKBOX( ID_ADJUST_PITCH_CHECKBOX, SettingsBook::OnCheckboxChanged )
    EVT_CHECKBOX( ID_MATCH_BPM_CHECKBOX, SettingsBook::OnCheckboxChanged )
    EVT_COMMAND_SCROLL( ID_TREMOLO_DEPTH_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_TREMOLO_PPS_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_LOWPASS_BAND_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_LOWPASS_SLOPE_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_HIGHPASS_BAND_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_HIGHPASS_SLOPE_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_BANDPASS_BAND_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_BANDPASS_SLOPE_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_BANDPASS_BAND_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_BANDPASS_SLOPE_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_BANDSTOP_BAND_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_BANDSTOP_SLOPE_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_BANDSTOP_BAND_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_BANDSTOP_SLOPE_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_LOUDNESS_BAND_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_LOUDNESS_SLOPE_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_LOUDNESS_BAND_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_LOUDNESS_SLOPE_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_MASTER_FILTER_BAND_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_MASTER_FILTER_SLOPE_LO_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_MASTER_FILTER_BAND_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_MASTER_FILTER_SLOPE_HI_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_MASTER_FILTER_MOD_DEPTH_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_MASTER_FILTER_LFO_DEPTH_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_MASTER_FILTER_LFO_PPS_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_MASTER_FILTER_RESONANCE_SLIDER, SettingsBook::OnSliderChanged )
    EVT_COMMAND_SCROLL( ID_PLAYBACK_COMPENSATION_SLIDER, SettingsBook::OnSliderChanged )
    EVT_TREEBOOK_PAGE_CHANGED( ID_EFFECTSTREE, SettingsBook::OnTreeChanged )
    EVT_TREEBOOK_PAGE_CHANGED( ID_GENERALTREE, SettingsBook::OnTreeChanged )
    EVT_TREEBOOK_NODE_EXPANDED( ID_EFFECTSTREE, SettingsBook::OnTreeChanged )
    EVT_TREEBOOK_NODE_EXPANDED( ID_GENERALTREE, SettingsBook::OnTreeChanged )
    EVT_AUINOTEBOOK_PAGE_CHANGED( wxID_ANY, SettingsBook::OnBookChanged )
    EVT_BUTTON( ID_DELAY_MULTITAP_PRESET1, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_MULTITAP_PRESET2, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_MULTITAP_PRESET3, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_MULTITAP_PRESET4, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_MULTITAP_PRESET5, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_MULTITAP_PRESET6, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_VARIABLE_PRESET1, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_VARIABLE_PRESET2, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_VARIABLE_PRESET3, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_VARIABLE_PRESET4, SettingsBook::OnButton )
    EVT_BUTTON( ID_DELAY_VARIABLE_PRESET4, SettingsBook::OnButton )
    EVT_BUTTON( ID_SHOW_FX_ALL, SettingsBook::OnButton )
    EVT_BUTTON( ID_SHOW_FX_NONE, SettingsBook::OnButton )
    EVT_BUTTON( ID_EFFECTSCHAIN_DEFAULT, SettingsBook::OnButton )
    EVT_BUTTON( ID_EFFECTSCHAIN_CLONE, SettingsBook::OnButton )
    EVT_BUTTON( ID_EFFECTSCHAIN_UP, SettingsBook::OnButton )
    EVT_BUTTON( ID_EFFECTSCHAIN_DOWN, SettingsBook::OnButton )
    EVT_BUTTON( ID_EFFECTSCHAIN_DELETE, SettingsBook::OnButton )
    EVT_BUTTON( ID_FINISHINGCHAIN_DEFAULT, SettingsBook::OnButton )
    EVT_BUTTON( ID_FINISHINGCHAIN_UP, SettingsBook::OnButton )
    EVT_BUTTON( ID_FINISHINGCHAIN_DOWN, SettingsBook::OnButton )
    EVT_BUTTON( ID_FINISHINGCHAIN_CLONE, SettingsBook::OnButton )
    EVT_BUTTON( ID_FINISHINGCHAIN_DELETE, SettingsBook::OnButton )
    EVT_BUTTON( ID_REFRESH_PRESETS, SettingsBook::OnButton )
    EVT_BUTTON( ID_DOWNLOAD_PRESET, SettingsBook::OnButton )
    EVT_BUTTON( ID_RATE_PRESET, SettingsBook::OnButton )
    EVT_BUTTON( ID_LOAD_PRESET, SettingsBook::OnButton )
    EVT_UPDATE_UI(ID_FINISHINGCHAIN_UP, SettingsBook::OnUpdateChainButton )
    EVT_UPDATE_UI(ID_FINISHINGCHAIN_DOWN, SettingsBook::OnUpdateChainButton )
    EVT_UPDATE_UI(ID_FINISHINGCHAIN_CLONE, SettingsBook::OnUpdateChainButton )
    EVT_UPDATE_UI(ID_FINISHINGCHAIN_DELETE, SettingsBook::OnUpdateChainButton )
    EVT_UPDATE_UI(ID_EFFECTSCHAIN_UP, SettingsBook::OnUpdateChainButton )
    EVT_UPDATE_UI(ID_EFFECTSCHAIN_DOWN, SettingsBook::OnUpdateChainButton )
    EVT_UPDATE_UI(ID_EFFECTSCHAIN_CLONE, SettingsBook::OnUpdateChainButton )
    EVT_UPDATE_UI(ID_EFFECTSCHAIN_DELETE, SettingsBook::OnUpdateChainButton )
    EVT_SIZE( SettingsBook::OnSize )
END_EVENT_TABLE()

SettingsBook::SettingsBook ( wxWindow *parentParameter, int id, EffectsGrid *effectsGridParameter, int bookWidthParameter, const std::vector<wxString> &v ) :
        wxAuiNotebook ( parentParameter, id, wxDefaultPosition, wxDefaultSize, wxAUI_NB_TOP | wxAUI_NB_SCROLL_BUTTONS ),
        parent ( parentParameter ), effectsGrid ( effectsGridParameter ), bookWidth ( bookWidthParameter ), finishingVector ( v ),
        tremoloPlotUpdateRequired ( false ), lowpassPlotUpdateRequired ( false ), highpassPlotUpdateRequired ( false ),
        bandpassPlotUpdateRequired ( false ), bandstopPlotUpdateRequired ( false ), loudnessPlotUpdateRequired ( false ),
        masterFilterPlotUpdateRequired ( false ), masterFilterModPlotUpdateRequired ( false ), masterFilterLfoPlotUpdateRequired ( false )
{
#ifndef __WXMSW__
    SetNormalFont ( *wxSMALL_FONT );
    SetSelectedFont ( *wxSMALL_FONT );
    SetMeasuringFont ( *wxSMALL_FONT );
#else
    int fontSize = wxNORMAL_FONT->GetPointSize();
    wxFont font ( fontSize, wxNORMAL_FONT->GetFamily(),
                  wxNORMAL_FONT->GetStyle(), wxNORMAL_FONT->GetWeight() );
    SetNormalFont ( font );
    SetSelectedFont ( font );
    SetMeasuringFont ( font );
    unicodeFont = new wxFont (
        8,
        wxFONTFAMILY_DEFAULT,
        wxFONTSTYLE_NORMAL,
        wxFONTWEIGHT_NORMAL,
        false,
        _T("Arial Unicode MS"),
        wxFONTENCODING_DEFAULT);
#endif
    effectsTreeWidthLeft =
#ifndef __WXMSW__
        141;
#else
        124;
#endif
    effectsTreeWidthRight = bookWidth - effectsTreeWidthLeft;
    effectsPlotHeight = 100;

    generalTreeWidthLeft =
#ifndef __WXMSW__
        120;
#else
        109;
#endif

    generalTreeWidthRight = bookWidth - generalTreeWidthLeft;
    generalPlotHeight = 100;

    fxControlSize = wxSize ( 0.9 * effectsTreeWidthRight, -1 );
    fxBoxSize = wxSize ( ( int )( 0.957 * effectsTreeWidthRight ), -1 );
    fxPlotSize = wxSize ( ( int )( 0.9 * effectsTreeWidthRight ), ( int ) ( effectsPlotHeight ) );

    generalPlotSize = wxSize ( ( int ) ( 0.875 * generalTreeWidthRight ), ( int ) ( generalPlotHeight ) );
    generalControlSize = wxSize ( ( int ) ( 0.85 * generalTreeWidthRight ), -1 );
    generalBoxSize = wxSize ( ( int ) ( 0.915 * generalTreeWidthRight ), -1 );

    generalTree = new wxTreebook ( this, ID_GENERALTREE, wxDefaultPosition, wxDefaultSize, wxBK_LEFT );

    wxPanel *warpGridPanel;
    warpGridPanel = new wxPanel ( generalTree, wxID_ANY );
    warpGrid = new WarpGrid ( warpGridPanel, wxID_ANY, ( size_t ) 16, ( size_t ) 16, 8, 10 );

    size_t warpPatternsNo = warpGrid->getPresetCount ( );
    wxString warpPatterns [ 100 ];

    for ( size_t i = 0; i < warpPatternsNo; i++ )
    {
        warpPatterns [ i ] = warpGrid->getPatternName ( i );
    }

    warpPatternsBox = new wxListBox ( warpGridPanel, ID_WARP_PATTERNS_BOX, wxDefaultPosition, wxSize ( generalBoxSize.GetWidth(), 250 ), warpPatternsNo, warpPatterns );

    wxStaticBoxSizer *warpGridSizer;
    warpGridSizer = new wxStaticBoxSizer ( wxVERTICAL, warpGridPanel, _("") );
    warpGridSizer->SetMinSize ( generalBoxSize );
    warpGridSizer->Add ( warpGrid );

    wxBoxSizer *warpSizer;
    warpSizer = new wxBoxSizer ( wxVERTICAL );

    warpSizer->Add ( warpGridSizer );
    warpSizer->Add ( warpPatternsBox );
    warpGridPanel->SetSizer ( warpSizer );
    warpSizer->Layout();

    wxPanel *declickPanel;
    declickPanel = new wxPanel ( generalTree, wxID_ANY );
    declickCheckbox = new wxCheckBox ( declickPanel, wxID_ANY, _("enable declick") );
    wxStaticBoxSizer *declickSizer;
    declickSizer = new wxStaticBoxSizer ( wxVERTICAL, declickPanel, _("") );
    declickSizer->SetMinSize ( generalBoxSize ) ;
    declickSizer->AddSpacer ( 5 );
    declickSizer->Add ( declickCheckbox );
    declickPanel->SetSizer ( declickSizer );
    declickSizer->Layout();

    wxPanel *enhancePanel;
    enhancePanel = new wxPanel ( generalTree, wxID_ANY );
    enhanceCheckbox = new wxCheckBox ( enhancePanel, wxID_ANY, _("enable enhance") );
    wxStaticBoxSizer *enhanceSizer;
    enhanceSizer = new wxStaticBoxSizer ( wxVERTICAL, enhancePanel, _("") );
    enhanceSizer->SetMinSize ( generalBoxSize ) ;
    enhanceSizer->AddSpacer ( 5 );
    enhanceSizer->Add ( enhanceCheckbox );
    enhancePanel->SetSizer ( enhanceSizer );
    enhanceSizer->Layout();

    wxPanel *gatePanel;
    gatePanel = new wxPanel ( generalTree, wxID_ANY );
    gateCheckbox = new wxCheckBox ( gatePanel, ID_GATE_BOX, _("enable gate") );
    wxStaticBoxSizer *gateSizer;
    gateSizer = new wxStaticBoxSizer ( wxVERTICAL, gatePanel, _("") );
    gateSizer->SetMinSize (generalBoxSize ) ;
    gateSizer->AddSpacer ( 5 );
    gateSizer->Add ( gateCheckbox );
    gatePanel->SetSizer ( gateSizer );
    gateSizer->Layout();


    wxPanel *eqPanel;
    eqPanel = new wxPanel ( generalTree, wxID_ANY );

    eqLowGainSlider = new wxSlider ( eqPanel, wxID_ANY,
                                     50, -100, 100, wxDefaultPosition, generalControlSize,
                                     wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    eqLowGainSlider->SetTickFreq ( 10, -1 );
    eqLowGainSlider->SetLineSize ( 1 );
    eqLowGainSlider->SetPageSize ( 5 );

    eqMidGainSlider = new wxSlider ( eqPanel, wxID_ANY,
                                     -30, -100, 100, wxDefaultPosition, generalControlSize,
                                     wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    eqMidGainSlider->SetTickFreq ( 10, -1 );
    eqMidGainSlider->SetLineSize ( 1 );
    eqMidGainSlider->SetPageSize ( 5 );

    eqHighGainSlider = new wxSlider ( eqPanel, wxID_ANY,
                                      20, -100, 100, wxDefaultPosition, generalControlSize,
                                      wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    eqHighGainSlider->SetTickFreq ( 10, -1 );
    eqHighGainSlider->SetLineSize ( 1 );
    eqHighGainSlider->SetPageSize ( 5 );

    eqCheckbox = new wxCheckBox ( eqPanel, wxID_ANY, _("enable eq") );

    wxStaticBoxSizer *eqLowGainSliderSizer, *eqMidGainSliderSizer, *eqHighGainSliderSizer;
    eqLowGainSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, eqPanel, _("low gain (%)"));
    eqLowGainSliderSizer->SetMinSize ( generalBoxSize );
    eqLowGainSliderSizer->Add ( eqLowGainSlider );

    eqMidGainSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, eqPanel, _("mid gain (%)"));
    eqMidGainSliderSizer->SetMinSize ( generalBoxSize );
    eqMidGainSliderSizer->Add ( eqMidGainSlider );

    eqHighGainSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, eqPanel, _("high gain (%)"));
    eqHighGainSliderSizer->SetMinSize ( generalBoxSize );
    eqHighGainSliderSizer->Add ( eqHighGainSlider );

    wxBoxSizer *eqSizer;
    eqSizer = new wxBoxSizer ( wxVERTICAL );
    eqSizer->Add ( eqLowGainSliderSizer );
    eqSizer->AddSpacer ( 10 );
    eqSizer->Add ( eqMidGainSliderSizer );
    eqSizer->AddSpacer ( 10 );
    eqSizer->Add ( eqHighGainSliderSizer );
    eqSizer->AddSpacer ( 10 );
    eqSizer->Add ( eqCheckbox );
    eqPanel->SetSizer ( eqSizer );
    eqSizer->Layout();

    wxPanel *pitchAdjustmentPanel;
    pitchAdjustmentPanel = new wxPanel ( generalTree, wxID_ANY );

    pitchAdjustCheckbox = new wxCheckBox ( pitchAdjustmentPanel, ID_ADJUST_PITCH_CHECKBOX, _("adjust pitch") );
    pitchSemitonesSlider = new wxSlider ( pitchAdjustmentPanel, wxID_ANY,
                                          0, -12, 12, wxDefaultPosition, generalControlSize,
                                          wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    pitchSemitonesSlider->SetTickFreq ( 1, -1 );
    pitchSemitonesSlider->SetLineSize ( 1 );
    wxStaticBoxSizer *pitchSemitonesSizer;
    pitchSemitonesSizer = new wxStaticBoxSizer ( wxVERTICAL, pitchAdjustmentPanel, _("semitones") );
    pitchSemitonesSizer->SetMinSize ( generalBoxSize );
    pitchSemitonesSizer->Add ( pitchSemitonesSlider );

    wxBoxSizer *pitchAdjustmentSizer;
    pitchAdjustmentSizer = new wxBoxSizer ( wxVERTICAL );
    pitchAdjustmentSizer->Add ( pitchSemitonesSizer );
    pitchAdjustmentSizer->AddSpacer ( 10 );
    pitchAdjustmentSizer->Add ( pitchAdjustCheckbox );
    pitchAdjustmentPanel->SetSizer ( pitchAdjustmentSizer );
    pitchSemitonesSizer->Layout();


    wxPanel *matchBpmPanel;
    matchBpmPanel = new wxPanel ( generalTree, wxID_ANY );

    matchBpmCheckbox = new wxCheckBox ( matchBpmPanel, ID_MATCH_BPM_CHECKBOX, _("match bpm") );
    matchBpmBeatsSlider = new wxSlider ( matchBpmPanel, wxID_ANY,
                                         1, 4, 32, wxDefaultPosition, generalControlSize,
                                         wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    matchBpmBeatsSlider->SetTickFreq ( 4, -1 );
    matchBpmBeatsSlider->SetLineSize ( 1 );

    matchBpmTargetBpmSlider = new wxSlider ( matchBpmPanel, wxID_ANY,
            120, 20, 200, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    matchBpmTargetBpmSlider->SetTickFreq ( 20, -1 );
    matchBpmTargetBpmSlider->SetLineSize ( 1 );

    wxStaticBoxSizer *matchBpmBeatsSliderSizer, *matchBpmTargetBpmSliderSizer;
    matchBpmBeatsSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, matchBpmPanel, _("beats") );
    matchBpmBeatsSliderSizer->SetMinSize ( generalBoxSize );
    matchBpmBeatsSliderSizer->Add ( matchBpmBeatsSlider );
    matchBpmTargetBpmSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, matchBpmPanel, _("target bpm") );
    matchBpmTargetBpmSliderSizer->SetMinSize ( generalBoxSize );
    matchBpmTargetBpmSliderSizer->Add ( matchBpmTargetBpmSlider );

    wxBoxSizer *matchBpmSizer;
    matchBpmSizer = new wxBoxSizer ( wxVERTICAL );
    matchBpmSizer->Add ( matchBpmBeatsSliderSizer );
    matchBpmSizer->Add ( matchBpmTargetBpmSliderSizer );
    matchBpmSizer->AddSpacer ( 10 );
    matchBpmSizer->Add ( matchBpmCheckbox );

    matchBpmPanel->SetSizer ( matchBpmSizer );
    matchBpmSizer->Layout();


    wxPanel *normalizePanel;
    normalizePanel = new wxPanel ( generalTree, wxID_ANY );
    normalizeSlider = new wxSlider ( normalizePanel, wxID_ANY,
                                     99, 0, 100, wxDefaultPosition, generalControlSize,
                                     wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    normalizeSlider->SetTickFreq ( 10, -1 );
    normalizeSlider->SetLineSize ( 5 );
    normalizeSlider->SetPageSize ( 10 );
    normalizeCheckbox = new wxCheckBox ( normalizePanel, ID_NORMALIZE_BOX, _("enable normalize") );

    wxStaticBoxSizer *normalizeSliderSizer;
    normalizeSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, normalizePanel, _("level (%)"));
    normalizeSliderSizer->SetMinSize ( generalBoxSize );
    normalizeSliderSizer->Add ( normalizeSlider );

    wxBoxSizer *normalizeSizer;
    normalizeSizer = new wxBoxSizer ( wxVERTICAL );
    normalizeSizer->Add ( normalizeSliderSizer );
    normalizeSizer->AddSpacer ( 10 );
    normalizeSizer->Add ( normalizeCheckbox );
    normalizePanel->SetSizer ( normalizeSizer );
    normalizeSizer->Layout();

    wxPanel *compressPanel;
    compressPanel = new wxPanel ( generalTree, wxID_ANY );
    compressCutoffSlider = new wxSlider ( compressPanel, wxID_ANY,
                                          20, 0, 100, wxDefaultPosition, generalControlSize,
                                          wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    compressCutoffSlider->SetTickFreq ( 10, -1 );
    compressCutoffSlider->SetLineSize ( 5 );
    compressCutoffSlider->SetPageSize ( 10 );
    compressRangeSlider = new wxSlider ( compressPanel, wxID_ANY,
                                         2, 0, 100, wxDefaultPosition, generalControlSize,
                                         wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    compressRangeSlider->SetTickFreq ( 10, -1 );
    compressRangeSlider->SetLineSize ( 5 );
    compressRangeSlider->SetPageSize ( 10 );
    compressCheckbox = new wxCheckBox ( compressPanel, ID_COMPRESS_BOX, _("enable compress") );
    wxStaticBoxSizer *compressCutoffSizer, *compressRangeSizer;
    compressCutoffSizer = new wxStaticBoxSizer ( wxVERTICAL, compressPanel, _("cutoff (%)") );
    compressCutoffSizer->SetMinSize ( generalBoxSize );
    compressCutoffSizer->Add ( compressCutoffSlider );
    compressRangeSizer = new wxStaticBoxSizer ( wxVERTICAL, compressPanel, _("range (%)") );
    compressRangeSizer->SetMinSize ( generalBoxSize );
    compressRangeSizer->Add ( compressRangeSlider );
    wxBoxSizer *compressSizer;
    compressSizer = new wxBoxSizer ( wxVERTICAL );
    compressSizer->Add ( compressCutoffSizer );
    compressSizer->Add ( compressRangeSizer );
    compressSizer->AddSpacer ( 10 );
    compressSizer->Add ( compressCheckbox );
    compressPanel->SetSizer ( compressSizer );
    compressSizer->Layout();

    wxPanel *loudnessPanel;
    loudnessPanel = new wxPanel ( generalTree, wxID_ANY );

    loudnessBandLoSlider = new wxSlider ( loudnessPanel, ID_LOUDNESS_BAND_LO_SLIDER,
                                          4000, 0, 22050, wxDefaultPosition, generalControlSize,
                                          wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    loudnessBandLoSlider->SetTickFreq ( 2000, -1 );
    loudnessBandLoSlider->SetLineSize ( 100 );
    loudnessBandLoSlider->SetPageSize ( 1000 );

    loudnessSlopeLoSlider = new wxSlider ( loudnessPanel, ID_LOUDNESS_SLOPE_LO_SLIDER,
                                           1500, 0, 22050, wxDefaultPosition, generalControlSize,
                                           wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    loudnessSlopeLoSlider->SetTickFreq ( 2000, -1 );
    loudnessSlopeLoSlider->SetLineSize ( 100 );
    loudnessSlopeLoSlider->SetPageSize ( 1000 );

    loudnessBandHiSlider = new wxSlider ( loudnessPanel, ID_LOUDNESS_BAND_HI_SLIDER,
                                          8000, 0, 22050, wxDefaultPosition, generalControlSize,
                                          wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    loudnessBandHiSlider->SetTickFreq ( 2000, -1 );
    loudnessBandHiSlider->SetLineSize ( 100 );
    loudnessBandHiSlider->SetPageSize ( 1000 );

    loudnessSlopeHiSlider = new wxSlider ( loudnessPanel, ID_LOUDNESS_SLOPE_LO_SLIDER,
                                           1500, 0, 22050, wxDefaultPosition, generalControlSize,
                                           wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    loudnessSlopeHiSlider->SetTickFreq ( 2000, -1 );
    loudnessSlopeHiSlider->SetLineSize ( 100 );
    loudnessSlopeHiSlider->SetPageSize ( 1000 );

    loudnessPlot = new FxPlotWindow ( loudnessPanel, wxID_ANY, wxPoint ( 0, 0 ), generalPlotSize );

    wxStaticBoxSizer *loudnessBandLoSizer, *loudnessSlopeLoSizer,
    *loudnessBandHiSizer, *loudnessSlopeHiSizer, *loudnessPlotSizer;
    loudnessBandLoSizer = new wxStaticBoxSizer ( wxVERTICAL, loudnessPanel, _("cutoff low (hz)") );
    loudnessSlopeLoSizer = new wxStaticBoxSizer ( wxVERTICAL, loudnessPanel, _("slope low (hz)") );
    loudnessBandHiSizer = new wxStaticBoxSizer ( wxVERTICAL, loudnessPanel, _("cutoff high (hz)") );
    loudnessSlopeHiSizer = new wxStaticBoxSizer ( wxVERTICAL, loudnessPanel, _("slope high (hz)") );
    loudnessPlotSizer = new wxStaticBoxSizer ( wxVERTICAL, loudnessPanel, _("preview") );

    loudnessBandLoSizer->SetMinSize ( generalBoxSize );
    loudnessSlopeLoSizer->SetMinSize ( generalBoxSize );
    loudnessBandHiSizer->SetMinSize ( generalBoxSize );
    loudnessSlopeHiSizer->SetMinSize ( generalBoxSize );
    loudnessPlotSizer->SetMinSize ( generalBoxSize );

    loudnessBandLoSizer->Add ( loudnessBandLoSlider );
    loudnessSlopeLoSizer->Add ( loudnessSlopeLoSlider );
    loudnessBandHiSizer->Add ( loudnessBandHiSlider );
    loudnessSlopeHiSizer->Add ( loudnessSlopeHiSlider );
    loudnessPlotSizer->Add ( loudnessPlot );

    loudnessCheckbox = new wxCheckBox ( loudnessPanel, ID_LOUDNESS_BOX, _( "enable loudness" ) );

    wxBoxSizer *loudnessSizer;
    loudnessSizer = new wxBoxSizer ( wxVERTICAL );
    loudnessSizer->Add ( loudnessBandLoSizer ) ;
    loudnessSizer->Add ( loudnessSlopeLoSizer ) ;
    loudnessSizer->Add ( loudnessBandHiSizer ) ;
    loudnessSizer->Add ( loudnessSlopeHiSizer ) ;
    loudnessSizer->Add ( loudnessPlotSizer );
    loudnessSizer->AddSpacer ( 10 );
    loudnessSizer->Add ( loudnessCheckbox );
    loudnessPanel->SetSizer ( loudnessSizer );
    loudnessSizer->Layout();

    wxPanel *masterFilterPanel, *masterFilterModPanel, *masterFilterLfoPanel, *masterFilterResPanel;
    masterFilterPanel = new wxPanel ( generalTree, wxID_ANY );
    masterFilterModPanel = new wxPanel ( generalTree, wxID_ANY );
    masterFilterLfoPanel = new wxPanel ( generalTree, wxID_ANY );
    masterFilterResPanel = new wxPanel ( generalTree, wxID_ANY );

    wxString masterFilterTypesChoices[] = { _( "low pass" ), _( "high pass" ), _( "band pass" ), _( "band stop" ) };
    masterFilterTypesBox = new wxRadioBox ( masterFilterPanel, ID_MASTER_FILTER_TYPES_BOX, _("type"),
                                            wxDefaultPosition, generalControlSize, 4, masterFilterTypesChoices, 4, wxRA_SPECIFY_ROWS );
    masterFilterTypesBox->SetSelection ( 0 );
    masterFilterTypesBox->SetMinSize ( generalBoxSize );

    masterFilterBandLoSlider = new wxSlider ( masterFilterPanel, ID_MASTER_FILTER_BAND_LO_SLIDER,
            6000, 0, 22050, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    masterFilterBandLoSlider->SetTickFreq ( 2000, -1 );
    masterFilterBandLoSlider->SetLineSize ( 100 );
    masterFilterBandLoSlider->SetPageSize ( 1000 );

    masterFilterSlopeLoSlider = new wxSlider ( masterFilterPanel, ID_MASTER_FILTER_SLOPE_LO_SLIDER,
            1000, 0, 22050, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    masterFilterSlopeLoSlider->SetTickFreq ( 2000, -1 );
    masterFilterSlopeLoSlider->SetLineSize ( 100 );
    masterFilterSlopeLoSlider->SetPageSize ( 1000 );

    masterFilterBandHiSlider = new wxSlider ( masterFilterPanel, ID_MASTER_FILTER_BAND_HI_SLIDER,
            6000, 0, 22050, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    masterFilterBandHiSlider->SetTickFreq ( 2000, -1 );
    masterFilterBandHiSlider->SetLineSize ( 100 );
    masterFilterBandHiSlider->SetPageSize ( 1000 );

    masterFilterSlopeHiSlider = new wxSlider ( masterFilterPanel, ID_MASTER_FILTER_SLOPE_HI_SLIDER,
            1000, 0, 22050, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    masterFilterSlopeHiSlider->SetTickFreq ( 2000, -1 );
    masterFilterSlopeHiSlider->SetLineSize ( 100 );
    masterFilterSlopeHiSlider->SetPageSize ( 1000 );

    masterFilterCheckbox = new wxCheckBox ( masterFilterPanel, ID_MASTER_FILTER_BOX, _( "enable master filter" ) );

    masterFilterPlot = new FxPlotWindow ( masterFilterPanel, wxID_ANY, wxPoint ( 0, 0 ), generalPlotSize );

    wxStaticBoxSizer *masterFilterBandLoSizer, *masterFilterSlopeLoSizer, *masterFilterBandHiSizer, *masterFilterSlopeHiSizer, *masterFilterPlotSizer;
    masterFilterBandLoSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterPanel, _("cutoff low (hz)") );
    masterFilterBandLoSizer->SetMinSize ( generalBoxSize );
    masterFilterBandLoSizer->Add ( masterFilterBandLoSlider );
    masterFilterSlopeLoSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterPanel, _("slope low (hz)") );
    masterFilterSlopeLoSizer->SetMinSize ( generalBoxSize );
    masterFilterSlopeLoSizer->Add ( masterFilterSlopeLoSlider );
    masterFilterBandHiSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterPanel, _("cutoff high (hz)") );
    masterFilterBandHiSizer->SetMinSize ( generalBoxSize );
    masterFilterBandHiSizer->Add ( masterFilterBandHiSlider );
    masterFilterSlopeHiSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterPanel, _("slope high (hz)") );
    masterFilterSlopeHiSizer->SetMinSize ( generalBoxSize );
    masterFilterSlopeHiSizer->Add ( masterFilterSlopeHiSlider );
    masterFilterPlotSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterPanel, _("preview") );
    masterFilterPlotSizer->SetMinSize ( generalBoxSize );
    masterFilterPlotSizer->Add( masterFilterPlot );

    wxBoxSizer *masterFilterSizer;
    masterFilterSizer = new wxBoxSizer ( wxVERTICAL );
    masterFilterSizer->Add ( masterFilterTypesBox ) ;
    masterFilterSizer->Add ( masterFilterBandLoSizer ) ;
    masterFilterSizer->Add ( masterFilterSlopeLoSizer ) ;
    masterFilterSizer->Add ( masterFilterBandHiSizer ) ;
    masterFilterSizer->Add ( masterFilterSlopeHiSizer ) ;
    masterFilterSizer->Add ( masterFilterPlotSizer );
    masterFilterSizer->AddSpacer ( 10 );
    masterFilterSizer->Add ( masterFilterCheckbox );
    masterFilterSizer->Layout();

    masterFilterResonanceSlider = new wxSlider ( masterFilterResPanel, ID_MASTER_FILTER_RESONANCE_SLIDER,
            2, 1, 20, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    masterFilterResonanceSlider->SetTickFreq ( 5, -1 );
    masterFilterResonanceSlider->SetLineSize ( 1 );
    masterFilterResonanceSlider->SetPageSize ( 2 );
    masterFilterResonanceSlider->SetMinSize ( generalControlSize );

    masterFilterResonanceCheckbox = new wxCheckBox ( masterFilterResPanel, ID_MASTER_FILTER_RESONANCE_BOX, _("enable resonance") );

    wxStaticBoxSizer *masterFilterResonanceSliderSizer;
    masterFilterResonanceSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterResPanel, _("gain") );
    masterFilterResonanceSliderSizer->SetMinSize ( generalBoxSize  );
    masterFilterResonanceSliderSizer->Add ( masterFilterResonanceSlider );

    wxBoxSizer *masterFilterResSizer;
    masterFilterResSizer = new wxBoxSizer ( wxVERTICAL );
    masterFilterResSizer->Add ( masterFilterResonanceSliderSizer );
    masterFilterResSizer->AddSpacer ( 10 );
    masterFilterResSizer->Add ( masterFilterResonanceCheckbox );
    masterFilterResPanel->SetSizer ( masterFilterResSizer );
    masterFilterResSizer->Layout();


    wxString masterFilterModulationTypesChoices[] =
        { _( "in" ), _( "out" ) };
    masterFilterModulationTypesBox = new wxRadioBox ( masterFilterModPanel, ID_MASTER_FILTER_MOD_TYPES_BOX, _("types"),
            wxDefaultPosition, generalControlSize, 2, masterFilterModulationTypesChoices, 2, wxRA_SPECIFY_ROWS );
    masterFilterModulationTypesBox->SetSelection ( 0 );
    masterFilterModulationTypesBox->SetMinSize ( generalBoxSize );

    masterFilterModulationDepthSlider = new wxSlider ( masterFilterModPanel, ID_MASTER_FILTER_MOD_DEPTH_SLIDER,
            100, 0, 100, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    masterFilterModulationDepthSlider->SetTickFreq ( 10, -1 );
    masterFilterModulationDepthSlider->SetLineSize ( 5 );
    masterFilterModulationDepthSlider->SetPageSize ( 10 );
    masterFilterModulationDepthSlider->SetMinSize ( generalControlSize );

    masterFilterModPlot = new FxPlotWindow ( masterFilterModPanel, wxID_ANY, wxPoint ( 0, 0 ), generalPlotSize );

    masterFilterModulationCheckbox = new wxCheckBox ( masterFilterModPanel, ID_MASTER_FILTER_MOD_BOX, _("enable sweep") );

    wxStaticBoxSizer *masterFilterModulationDepthSizer, *masterFilterModPlotSizer;
    masterFilterModulationDepthSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterModPanel, _("depth (%)") );
    masterFilterModulationDepthSizer->SetMinSize ( generalBoxSize );
    masterFilterModulationDepthSizer->Add ( masterFilterModulationDepthSlider );

    masterFilterModPlotSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterModPanel, _("preview") );
    masterFilterModPlotSizer->SetMinSize ( generalBoxSize );
    masterFilterModPlotSizer->Add( masterFilterModPlot );

    wxBoxSizer *masterFilterModSizer;
    masterFilterModSizer = new wxBoxSizer ( wxVERTICAL );
    masterFilterModSizer->Add ( masterFilterModulationTypesBox ) ;
    masterFilterModSizer->Add ( masterFilterModulationDepthSizer );
    masterFilterModSizer->Add ( masterFilterModPlotSizer );
    masterFilterModSizer->AddSpacer ( 10 );
    masterFilterModSizer->Add ( masterFilterModulationCheckbox );
    masterFilterModPanel->SetSizer ( masterFilterModSizer );
    masterFilterModSizer->Layout();


    wxString masterFilterLfoTypesChoices[] =
        { _("sine squared"), _("sine"), _("sawtooth"), _("square"), _("triangle"), _("sample and hold") };
    masterFilterLfoTypesBox = new wxRadioBox ( masterFilterLfoPanel, ID_MASTER_FILTER_LFO_TYPES_BOX, _("types"),
            wxDefaultPosition, generalControlSize, 6, masterFilterLfoTypesChoices, 6, wxRA_SPECIFY_ROWS );
    masterFilterLfoTypesBox->SetSelection ( 0 );
    masterFilterLfoTypesBox->SetMinSize ( generalBoxSize );

    masterFilterLfoDepthSlider = new wxSlider ( masterFilterLfoPanel, ID_MASTER_FILTER_LFO_DEPTH_SLIDER,
            100, 0, 100, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    masterFilterLfoDepthSlider->SetTickFreq ( 10, -1 );
    masterFilterLfoDepthSlider->SetLineSize ( 1 );
    masterFilterLfoDepthSlider->SetPageSize ( 5 );
    masterFilterLfoDepthSlider->SetMinSize ( generalControlSize );

    masterFilterLfoPeriodsPerSliceSlider = new wxSlider ( masterFilterLfoPanel, ID_MASTER_FILTER_LFO_PPS_SLIDER,
            2, 1, 40, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    masterFilterLfoPeriodsPerSliceSlider->SetTickFreq ( 10, -1 );
    masterFilterLfoPeriodsPerSliceSlider->SetLineSize ( 1 );
    masterFilterLfoPeriodsPerSliceSlider->SetPageSize ( 1 );
    masterFilterLfoPeriodsPerSliceSlider->SetMinSize ( generalControlSize );

    masterFilterLfoPlot = new FxPlotWindow ( masterFilterLfoPanel, wxID_ANY, wxPoint ( 0, 0 ), generalPlotSize );

    masterFilterLfoCheckbox = new wxCheckBox ( masterFilterLfoPanel, ID_MASTER_FILTER_LFO_BOX, _( "enable lfo" ) );

    wxStaticBoxSizer *masterFilterLfoDepthSizer, *masterFilterLfoPeriodsPerSliceSliderSizer, *masterFilterLfoPlotSizer;

    masterFilterLfoDepthSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterLfoPanel, _("depth (%)"));
    masterFilterLfoDepthSizer->Add( masterFilterLfoDepthSlider );
    masterFilterLfoDepthSizer->SetMinSize ( generalBoxSize );

    masterFilterLfoPeriodsPerSliceSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterLfoPanel, _("cycles"));
    masterFilterLfoPeriodsPerSliceSliderSizer->SetMinSize ( generalBoxSize  );
    masterFilterLfoPeriodsPerSliceSliderSizer->Add( masterFilterLfoPeriodsPerSliceSlider );

    masterFilterLfoPlotSizer = new wxStaticBoxSizer ( wxVERTICAL, masterFilterLfoPanel, _("preview"));
    masterFilterLfoPlotSizer->SetMinSize ( generalBoxSize );
    masterFilterLfoPlotSizer->Add( masterFilterLfoPlot );

    wxBoxSizer *masterFilterLfoSizer;
    masterFilterLfoSizer = new wxBoxSizer ( wxVERTICAL );
    masterFilterLfoSizer->Add ( masterFilterLfoTypesBox ) ;
    masterFilterLfoSizer->Add ( masterFilterLfoDepthSizer );
    masterFilterLfoSizer->Add ( masterFilterLfoPeriodsPerSliceSliderSizer );
    masterFilterLfoSizer->Add ( masterFilterLfoPlotSizer );
    masterFilterLfoSizer->AddSpacer ( 10 );
    masterFilterLfoSizer->Add ( masterFilterLfoCheckbox );
    masterFilterLfoPanel->SetSizer ( masterFilterLfoSizer );
    masterFilterLfoSizer->Layout();

    wxPanel *reverbPanel;
    reverbPanel = new wxPanel ( generalTree, wxID_ANY );
    reverbRoomsizeSlider = new wxSlider ( reverbPanel, wxID_ANY,
                                          50, 0, 100, wxDefaultPosition, generalControlSize,
                                          wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    reverbRoomsizeSlider->SetTickFreq ( 10, -1 );
    reverbRoomsizeSlider->SetLineSize ( 1 );
    reverbRoomsizeSlider->SetPageSize ( 10 );
    reverbDampSlider = new wxSlider ( reverbPanel, wxID_ANY,
                                      50, 0, 100, wxDefaultPosition, generalControlSize,
                                      wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    reverbDampSlider->SetTickFreq ( 10, -1 );
    reverbDampSlider->SetLineSize ( 1 );
    reverbDampSlider->SetPageSize ( 10 );
    reverbDryMixSlider = new wxSlider ( reverbPanel, wxID_ANY,
                                        75, 0, 100, wxDefaultPosition, generalControlSize,
                                        wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    reverbDryMixSlider->SetTickFreq ( 10, -1 );
    reverbDryMixSlider->SetLineSize ( 1 );
    reverbDryMixSlider->SetPageSize ( 10 );
    reverbCheckbox = new wxCheckBox ( reverbPanel, wxID_ANY, _("enable reverb") );
    wxStaticBoxSizer *reverbRoomsizeSizer, *reverbDampSizer, *reverbDryMixSizer;
    reverbRoomsizeSizer = new wxStaticBoxSizer ( wxVERTICAL, reverbPanel, _("room size (%)") );
    reverbRoomsizeSizer->SetMinSize ( generalBoxSize );
    reverbRoomsizeSizer->Add ( reverbRoomsizeSlider );
    reverbDampSizer = new wxStaticBoxSizer ( wxVERTICAL, reverbPanel, _("dampen (%)") );
    reverbDampSizer->SetMinSize ( generalBoxSize );
    reverbDampSizer->Add ( reverbDampSlider );
    reverbDryMixSizer = new wxStaticBoxSizer ( wxVERTICAL, reverbPanel, _("dry (%)") );
    reverbDryMixSizer->SetMinSize ( generalBoxSize );
    reverbDryMixSizer->Add ( reverbDryMixSlider );
    wxBoxSizer *reverbSizer;
    reverbSizer = new wxBoxSizer ( wxVERTICAL );
    reverbSizer->Add ( reverbRoomsizeSizer );
    reverbSizer->Add ( reverbDampSizer );
    reverbSizer->Add ( reverbDryMixSizer );
    reverbSizer->AddSpacer ( 10 );
    reverbSizer->Add ( reverbCheckbox );
    reverbPanel->SetSizer ( reverbSizer );
    reverbSizer->Layout();

    wxPanel *auxPanel;
    auxPanel = new wxPanel ( generalTree, wxID_ANY );

    auxCheckbox = new wxCheckBox ( auxPanel, wxID_ANY, _("enable aux") );
    wxBoxSizer *auxSizer;
    auxSizer = new wxBoxSizer ( wxVERTICAL );
    auxSizer->AddSpacer ( 10 );
    auxSizer->Add ( auxCheckbox );
    auxPanel->SetSizer ( auxSizer );
    auxSizer->Layout();

    effectsTree = new wxTreebook ( this, ID_EFFECTSTREE, wxDefaultPosition, wxDefaultSize,
                                   wxBK_LEFT );
    wxPanel *showFxPanel = new wxPanel ( effectsTree, wxID_ANY );
    wxString fxName;
    wxToggleButton *showFxButton;
    wxStaticBoxSizer *showFxSizer;
    showFxSizer = new wxStaticBoxSizer ( wxVERTICAL, showFxPanel, _(""));
    showFxSizer->SetMinSize( fxBoxSize );
    size_t rowNo = effectsGrid->GetNumberRows();
    int showFxId = ID_EFFECTS_START;
    size_t i;
    for ( i = 0; i < rowNo; i++ )
    {
        fxName = effectsGrid->GetRowLabelValue( i );
        showFxId = ID_EFFECTS_START + i;
        showFxButton = new wxToggleButton ( showFxPanel, showFxId, fxName, wxDefaultPosition, wxDefaultSize, 2 );

        bool b = ( fxName != _T("fade in") && fxName != _T("fade out") && fxName != _T("bandpass") && fxName != _T("bandstop") );

        showFxButton->SetValue ( b );
        showFxButton->SetMinSize (
#ifdef __WXMSW__
            fxBoxSize
#else
            fxControlSize
#endif
        );
        showFxSizer->Add ( showFxButton );
    }

    Connect ( ID_EFFECTS_START, showFxId, wxEVT_COMMAND_TOGGLEBUTTON_CLICKED, wxCommandEventHandler( SettingsBook::OnToggleButtonChanged ) );

    showFxAllButton = new wxButton ( showFxPanel, ID_SHOW_FX_ALL, _T ( "show all"), wxDefaultPosition, fxControlSize );
    showFxNoneButton = new wxButton ( showFxPanel, ID_SHOW_FX_NONE, _T ( "hide all"), wxDefaultPosition, fxControlSize );

    showFxSizer->AddSpacer ( 10 );
    showFxSizer->Add ( showFxAllButton );
    showFxSizer->Add ( showFxNoneButton );

    showFxPanel->SetSizer ( showFxSizer );
    showFxSizer->Layout();

    wxPanel *delayMultiTapPanel;
    delayMultiTapPanel = new wxPanel ( effectsTree, wxID_ANY );

    delayMultitapPreset1Button = new wxButton ( delayMultiTapPanel, ID_DELAY_MULTITAP_PRESET1, _T ( "1/100"), wxDefaultPosition, fxControlSize );
    delayMultitapPreset2Button = new wxButton ( delayMultiTapPanel, ID_DELAY_MULTITAP_PRESET2, _T ( "1/50"), wxDefaultPosition, fxControlSize );
    delayMultitapPreset3Button = new wxButton ( delayMultiTapPanel, ID_DELAY_MULTITAP_PRESET3, _T ( "1/8"), wxDefaultPosition, fxControlSize );
    delayMultitapPreset4Button = new wxButton ( delayMultiTapPanel, ID_DELAY_MULTITAP_PRESET4, _T ( "1/4"), wxDefaultPosition, fxControlSize );
    delayMultitapPreset5Button = new wxButton ( delayMultiTapPanel, ID_DELAY_MULTITAP_PRESET5, _T ( "1/3"), wxDefaultPosition, fxControlSize );
    delayMultitapPreset6Button = new wxButton ( delayMultiTapPanel, ID_DELAY_MULTITAP_PRESET6, _T ( "1/2"), wxDefaultPosition, fxControlSize );

    delayMultiTapDistanceSlider = new wxSlider ( delayMultiTapPanel, wxID_ANY,
            1, 1, 100, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayMultiTapDistanceSlider->SetTickFreq ( 10, -1 );
    delayMultiTapDistanceSlider->SetLineSize ( 1 );
    delayMultiTapDistanceSlider->SetPageSize ( 10 );

    delayMultiTapFeedbackSlider = new wxSlider ( delayMultiTapPanel, wxID_ANY,
            0, 0, 10, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayMultiTapFeedbackSlider->SetTickFreq ( 1, -1 );
    delayMultiTapFeedbackSlider->SetLineSize ( 1 );
    delayMultiTapFeedbackSlider->SetPageSize ( 2 );

    delayMultiTapGainSlider = new wxSlider ( delayMultiTapPanel, wxID_ANY,
            1, 0, 10, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayMultiTapGainSlider->SetTickFreq ( 1, -1 );
    delayMultiTapGainSlider->SetLineSize ( 1 );
    delayMultiTapGainSlider->SetPageSize ( 2 );

    delayMultiTapStepNoSlider = new wxSlider ( delayMultiTapPanel, wxID_ANY,
            1, 1, 4, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayMultiTapStepNoSlider->SetTickFreq ( 1, -1 );
    delayMultiTapStepNoSlider->SetLineSize ( 1 );
    delayMultiTapStepNoSlider->SetPageSize ( 1 );

    wxStaticBoxSizer *delayMultiTapFeedbackSliderSizer, *delayMultiTapStepNoSliderSizer, *delayMultiTapDistanceSliderSizer, *delayMultiTapGainSliderSizer;

    delayMultiTapFeedbackSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayMultiTapPanel, _("feedback"));
    delayMultiTapFeedbackSliderSizer->SetMinSize( fxBoxSize );
    delayMultiTapFeedbackSliderSizer->Add ( delayMultiTapFeedbackSlider );

    delayMultiTapStepNoSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayMultiTapPanel, _("taps"));
    delayMultiTapStepNoSliderSizer->SetMinSize( fxBoxSize );
    delayMultiTapStepNoSliderSizer->Add ( delayMultiTapStepNoSlider );

    delayMultiTapDistanceSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayMultiTapPanel, _("distance (%)"));
    delayMultiTapDistanceSliderSizer->SetMinSize( fxBoxSize );
    delayMultiTapDistanceSliderSizer->AddSpacer ( 5 );
    delayMultiTapDistanceSliderSizer->Add ( delayMultiTapDistanceSlider );
    delayMultiTapDistanceSliderSizer->Add ( delayMultitapPreset1Button );
    delayMultiTapDistanceSliderSizer->Add ( delayMultitapPreset2Button );
    delayMultiTapDistanceSliderSizer->Add ( delayMultitapPreset3Button );
    delayMultiTapDistanceSliderSizer->Add ( delayMultitapPreset4Button );
    delayMultiTapDistanceSliderSizer->Add ( delayMultitapPreset5Button );
    delayMultiTapDistanceSliderSizer->Add ( delayMultitapPreset6Button );


    delayMultiTapGainSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayMultiTapPanel, _("gain"));
    delayMultiTapGainSliderSizer->SetMinSize( fxBoxSize );
    delayMultiTapGainSliderSizer->Add ( delayMultiTapGainSlider );

    wxBoxSizer *delayMultiTapSizer;

    delayMultiTapSizer = new wxBoxSizer ( wxVERTICAL );
    delayMultiTapSizer->Add ( delayMultiTapDistanceSliderSizer );
    delayMultiTapSizer->Add ( delayMultiTapGainSliderSizer );
    delayMultiTapSizer->Add( delayMultiTapFeedbackSliderSizer );
    delayMultiTapSizer->Add( delayMultiTapStepNoSliderSizer );
    delayMultiTapPanel->SetSizer ( delayMultiTapSizer );
    delayMultiTapSizer->Layout();


    wxPanel *delayVariablePanel;
    delayVariablePanel = new wxPanel ( generalTree, wxID_ANY );

    delayVariablePreset1Button = new wxButton ( delayVariablePanel, ID_DELAY_VARIABLE_PRESET1, _T ( "32 samples"), wxDefaultPosition, generalControlSize );
    delayVariablePreset2Button = new wxButton ( delayVariablePanel, ID_DELAY_VARIABLE_PRESET2, _T ( "64 samples"), wxDefaultPosition, generalControlSize );
    delayVariablePreset3Button = new wxButton ( delayVariablePanel, ID_DELAY_VARIABLE_PRESET3, _T ( "128 samples"), wxDefaultPosition, generalControlSize );
    delayVariablePreset4Button = new wxButton ( delayVariablePanel, ID_DELAY_VARIABLE_PRESET4, _T ( "256 samples"), wxDefaultPosition, generalControlSize );

    delayVariableDistanceSlider = new wxSlider ( delayVariablePanel, wxID_ANY,
            1, 1, 256, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayVariableDistanceSlider->SetTickFreq ( 64, -1 );
    delayVariableDistanceSlider->SetLineSize ( 1 );
    delayVariableDistanceSlider->SetPageSize ( 1 );

    delayVariableFeedbackSlider = new wxSlider ( delayVariablePanel, wxID_ANY,
            0, 0, 10, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayVariableFeedbackSlider->SetTickFreq ( 1, -1 );
    delayVariableFeedbackSlider->SetLineSize ( 1 );
    delayVariableFeedbackSlider->SetPageSize ( 2 );

    delayVariableGainSlider = new wxSlider ( delayVariablePanel, wxID_ANY,
            1, 0, 10, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayVariableGainSlider->SetTickFreq ( 1, -1 );
    delayVariableGainSlider->SetLineSize ( 1 );
    delayVariableGainSlider->SetPageSize ( 2 );

    wxString delayLfoTypesChoices[] = {_("sine squared"), _("triangle")};
    delayVariableLfoTypesBox = new wxRadioBox ( delayVariablePanel, wxID_ANY, _("lfo type"),
            wxDefaultPosition, generalBoxSize, 2, delayLfoTypesChoices, 2, wxRA_SPECIFY_ROWS);
    delayVariableLfoTypesBox->SetSelection( 0 );
    delayVariableLfoTypesBox->SetMinSize( generalBoxSize );

    delayVariableLfoDepthSlider = new wxSlider ( delayVariablePanel, wxID_ANY,
            1, 0, 100, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayVariableLfoDepthSlider->SetTickFreq ( 10, -1 );
    delayVariableLfoDepthSlider->SetLineSize ( 1 );
    delayVariableLfoDepthSlider->SetPageSize ( 10 );

    delayVariableLfoPeriodsPerSliceSlider = new wxSlider ( delayVariablePanel, wxID_ANY,
            25, 1, 100, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayVariableLfoPeriodsPerSliceSlider->SetTickFreq ( 10, -1 );
    delayVariableLfoPeriodsPerSliceSlider->SetLineSize ( 1 );
    delayVariableLfoPeriodsPerSliceSlider->SetPageSize ( 1 );

    delayVariableCheckbox = new wxCheckBox ( delayVariablePanel, wxID_ANY, _("enable master flanger") );

    wxStaticBoxSizer *delayVariableFeedbackSliderSizer, *delayVariableLfoDepthSliderSizer, *delayVariableDistanceSliderSizer,
    *delayVariableGainSliderSizer, *delayVariableLfoPeriodsPerSliceSliderSizer;

    delayVariableDistanceSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayVariablePanel, _("distance"));
    delayVariableDistanceSliderSizer->SetMinSize( generalBoxSize );
    delayVariableDistanceSliderSizer->Add ( delayVariableDistanceSlider );
    delayVariableDistanceSliderSizer->AddSpacer ( 5 );
    delayVariableDistanceSliderSizer->Add ( delayVariablePreset1Button );
    delayVariableDistanceSliderSizer->Add ( delayVariablePreset2Button );
    delayVariableDistanceSliderSizer->Add ( delayVariablePreset3Button );
    delayVariableDistanceSliderSizer->Add ( delayVariablePreset4Button );

    delayVariableGainSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayVariablePanel, _("gain"));
    delayVariableGainSliderSizer->SetMinSize( generalBoxSize );
    delayVariableGainSliderSizer->Add ( delayVariableGainSlider );

    delayVariableFeedbackSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayVariablePanel, _("feedback"));
    delayVariableFeedbackSliderSizer->SetMinSize( generalBoxSize );
    delayVariableFeedbackSliderSizer->Add ( delayVariableFeedbackSlider );

    delayVariableLfoDepthSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayVariablePanel, _("lfo depth (%)"));
    delayVariableLfoDepthSliderSizer->SetMinSize( generalBoxSize );
    delayVariableLfoDepthSliderSizer->Add ( delayVariableLfoDepthSlider );

    delayVariableLfoPeriodsPerSliceSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayVariablePanel, _("lfo cycle (%)"));
    delayVariableLfoPeriodsPerSliceSliderSizer->SetMinSize( generalBoxSize );
    delayVariableLfoPeriodsPerSliceSliderSizer->Add ( delayVariableLfoPeriodsPerSliceSlider );

    wxBoxSizer *delayVariableSizer;

    delayVariableSizer = new wxBoxSizer ( wxVERTICAL );
    delayVariableSizer->Add ( delayVariableDistanceSliderSizer );
    delayVariableSizer->Add ( delayVariableGainSliderSizer );
    delayVariableSizer->Add( delayVariableFeedbackSliderSizer );
    delayVariableSizer->Add( delayVariableLfoTypesBox );
    delayVariableSizer->Add( delayVariableLfoDepthSliderSizer );
    delayVariableSizer->Add( delayVariableLfoPeriodsPerSliceSliderSizer );
    delayVariableSizer->AddSpacer ( 5 );
    delayVariableSizer->Add( delayVariableCheckbox );
    delayVariablePanel->SetSizer ( delayVariableSizer );
    delayVariableSizer->Layout();

    wxPanel *delayPhasingPanel;
    delayPhasingPanel = new wxPanel ( generalTree, wxID_ANY );

    delayPhasingDistanceOffsetSlider = new wxSlider ( delayPhasingPanel, wxID_ANY,
            440, 1, 500, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayPhasingDistanceOffsetSlider->SetTickFreq ( 50, -1 );
    delayPhasingDistanceOffsetSlider->SetLineSize ( 1 );
    delayPhasingDistanceOffsetSlider->SetPageSize ( 1 );

    delayPhasingDistanceRangeSlider = new wxSlider ( delayPhasingPanel, wxID_ANY,
            800, 1, 1000, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayPhasingDistanceRangeSlider->SetTickFreq ( 100, -1 );
    delayPhasingDistanceRangeSlider->SetLineSize ( 1 );
    delayPhasingDistanceRangeSlider->SetPageSize ( 1 );

    delayPhasingGainSlider = new wxSlider ( delayPhasingPanel, wxID_ANY,
                                            7, 0, 10, wxDefaultPosition, generalControlSize,
                                            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayPhasingGainSlider->SetTickFreq ( 1, -1 );
    delayPhasingGainSlider->SetLineSize ( 1 );
    delayPhasingGainSlider->SetPageSize ( 2 );

    wxString delayPhasingLfoTypesChoices[] = {_("sine squared"), _("triangle")};
    delayPhasingLfoTypesBox = new wxRadioBox ( delayPhasingPanel, wxID_ANY, _("lfo type"),
            wxDefaultPosition, generalBoxSize, 2, delayPhasingLfoTypesChoices, 2, wxRA_SPECIFY_ROWS);
    delayPhasingLfoTypesBox->SetSelection( 0 );
    delayPhasingLfoTypesBox->SetMinSize( generalBoxSize );

    delayPhasingLfoDepthSlider = new wxSlider ( delayPhasingPanel, wxID_ANY,
            1, 0, 100, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayPhasingLfoDepthSlider->SetTickFreq ( 10, -1 );
    delayPhasingLfoDepthSlider->SetLineSize ( 1 );
    delayPhasingLfoDepthSlider->SetPageSize ( 10 );

    delayPhasingLfoPeriodsPerSliceSlider = new wxSlider ( delayPhasingPanel, wxID_ANY,
            25, 1, 100, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    delayPhasingLfoPeriodsPerSliceSlider->SetTickFreq ( 10, -1 );
    delayPhasingLfoPeriodsPerSliceSlider->SetLineSize ( 1 );
    delayPhasingLfoPeriodsPerSliceSlider->SetPageSize ( 1 );

    delayPhasingCheckbox = new wxCheckBox ( delayPhasingPanel, wxID_ANY, _("enable master phaser") );

    wxStaticBoxSizer *delayPhasingDistanceOffsetSliderSizer, *delayPhasingDistanceRangeSliderSizer, *delayPhasingLfoDepthSliderSizer,
    *delayPhasingGainSliderSizer, *delayPhasingLfoPeriodsPerSliceSliderSizer;

    delayPhasingDistanceOffsetSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayPhasingPanel, _("tone"));
    delayPhasingDistanceOffsetSliderSizer->SetMinSize( generalBoxSize );
    delayPhasingDistanceOffsetSliderSizer->Add ( delayPhasingDistanceOffsetSlider );

    delayPhasingDistanceRangeSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayPhasingPanel, _("width"));
    delayPhasingDistanceRangeSliderSizer->SetMinSize( generalBoxSize );
    delayPhasingDistanceRangeSliderSizer->Add ( delayPhasingDistanceRangeSlider );

    delayPhasingGainSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayPhasingPanel, _("gain"));
    delayPhasingGainSliderSizer->SetMinSize( generalBoxSize );
    delayPhasingGainSliderSizer->Add ( delayPhasingGainSlider );

    delayPhasingLfoDepthSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayPhasingPanel, _("lfo depth (%)"));
    delayPhasingLfoDepthSliderSizer->SetMinSize( generalBoxSize );
    delayPhasingLfoDepthSliderSizer->Add ( delayPhasingLfoDepthSlider );

    delayPhasingLfoPeriodsPerSliceSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, delayPhasingPanel, _("lfo cycle (%)"));
    delayPhasingLfoPeriodsPerSliceSliderSizer->SetMinSize( generalBoxSize );
    delayPhasingLfoPeriodsPerSliceSliderSizer->Add ( delayPhasingLfoPeriodsPerSliceSlider );

    wxBoxSizer *delayPhasingSizer;

    delayPhasingSizer = new wxBoxSizer ( wxVERTICAL );
    delayPhasingSizer->Add ( delayPhasingDistanceOffsetSliderSizer );
    delayPhasingSizer->Add ( delayPhasingDistanceRangeSliderSizer );
    delayPhasingSizer->Add ( delayPhasingGainSliderSizer );
    delayPhasingSizer->Add ( delayPhasingLfoTypesBox );
    delayPhasingSizer->Add ( delayPhasingLfoDepthSliderSizer );
    delayPhasingSizer->Add ( delayPhasingLfoPeriodsPerSliceSliderSizer );
    delayPhasingSizer->AddSpacer ( 5 );
    delayPhasingSizer->Add ( delayPhasingCheckbox );
    delayPhasingPanel->SetSizer ( delayPhasingSizer );
    delayPhasingSizer->Layout();



    wxPanel *fftLowPassPanel, *fftHighPassPanel,
    *fftBandPassPanel, *fftBandStopPanel;
    fftLowPassPanel = new wxPanel ( effectsTree, wxID_ANY );
    fftHighPassPanel = new wxPanel ( effectsTree, wxID_ANY );
    fftBandPassPanel = new wxPanel ( effectsTree, wxID_ANY );
    fftBandStopPanel = new wxPanel ( effectsTree, wxID_ANY );

    fftLowPassBandHiSlider = new wxSlider ( fftLowPassPanel, ID_LOWPASS_BAND_HI_SLIDER,
                                            1000, 0, 22050, wxDefaultPosition, fxControlSize,
                                            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftLowPassBandHiSlider->SetTickFreq ( 2000, -1 );
    fftLowPassBandHiSlider->SetLineSize ( 100 );
    fftLowPassBandHiSlider->SetPageSize ( 1000 );

    fftLowPassSlopeHiSlider = new wxSlider ( fftLowPassPanel, ID_LOWPASS_SLOPE_HI_SLIDER,
            1000, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftLowPassSlopeHiSlider->SetTickFreq ( 2000, -1 );
    fftLowPassSlopeHiSlider->SetLineSize ( 100 );
    fftLowPassSlopeHiSlider->SetPageSize ( 1000 );

    fftHighPassBandLoSlider = new wxSlider ( fftHighPassPanel, ID_HIGHPASS_BAND_LO_SLIDER,
            5500, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftHighPassBandLoSlider->SetTickFreq ( 2000, -1 );
    fftHighPassBandLoSlider->SetLineSize ( 100 );
    fftHighPassBandLoSlider->SetPageSize ( 1000 );

    fftHighPassSlopeLoSlider = new wxSlider ( fftHighPassPanel, ID_HIGHPASS_SLOPE_LO_SLIDER,
            2500, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftHighPassSlopeLoSlider->SetTickFreq ( 2000, -1 );
    fftHighPassSlopeLoSlider->SetLineSize ( 100 );
    fftHighPassSlopeLoSlider->SetPageSize ( 1000 );

    fftBandPassBandLoSlider = new wxSlider ( fftBandPassPanel, ID_BANDPASS_BAND_LO_SLIDER,
            4500, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftBandPassBandLoSlider->SetTickFreq ( 2000, -1 );
    fftBandPassBandLoSlider->SetLineSize ( 100 );
    fftBandPassBandLoSlider->SetPageSize ( 1000 );

    fftBandPassSlopeLoSlider = new wxSlider ( fftBandPassPanel, ID_BANDPASS_SLOPE_LO_SLIDER,
            1500, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftBandPassSlopeLoSlider->SetTickFreq ( 2000, -1 );
    fftBandPassSlopeLoSlider->SetLineSize ( 100 );
    fftBandPassSlopeLoSlider->SetPageSize ( 1000 );

    fftBandPassBandHiSlider = new wxSlider ( fftBandPassPanel, ID_BANDPASS_BAND_HI_SLIDER,
            5500, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftBandPassBandHiSlider->SetTickFreq ( 2000, -1 );
    fftBandPassBandHiSlider->SetLineSize ( 100 );
    fftBandPassBandHiSlider->SetPageSize ( 1000 );

    fftBandPassSlopeHiSlider = new wxSlider ( fftBandPassPanel, ID_BANDPASS_SLOPE_HI_SLIDER,
            1500, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftBandPassSlopeHiSlider->SetTickFreq ( 2000, -1 );
    fftBandPassSlopeHiSlider->SetLineSize ( 100 );
    fftBandPassSlopeHiSlider->SetPageSize ( 1000 );

    fftBandStopBandLoSlider = new wxSlider ( fftBandStopPanel, ID_BANDSTOP_BAND_LO_SLIDER,
            2000, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftBandStopBandLoSlider->SetTickFreq ( 2000, -1 );
    fftBandStopBandLoSlider->SetLineSize ( 100 );
    fftBandStopBandLoSlider->SetPageSize ( 1000 );

    fftBandStopSlopeLoSlider = new wxSlider ( fftBandStopPanel, ID_BANDSTOP_SLOPE_LO_SLIDER,
            1500, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftBandStopSlopeLoSlider->SetTickFreq ( 2000, -1 );
    fftBandStopSlopeLoSlider->SetLineSize ( 100 );
    fftBandStopSlopeLoSlider->SetPageSize ( 1000 );

    fftBandStopBandHiSlider = new wxSlider ( fftBandStopPanel, ID_BANDSTOP_BAND_HI_SLIDER,
            8000, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftBandStopBandHiSlider->SetTickFreq ( 2000, -1 );
    fftBandStopBandHiSlider->SetLineSize ( 100 );
    fftBandStopBandHiSlider->SetPageSize ( 1000 );

    fftBandStopSlopeHiSlider = new wxSlider ( fftBandStopPanel, ID_BANDSTOP_SLOPE_HI_SLIDER,
            1500, 0, 22050, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fftBandStopSlopeHiSlider->SetTickFreq ( 2000, -1 );
    fftBandStopSlopeHiSlider->SetLineSize ( 100 );
    fftBandStopSlopeHiSlider->SetPageSize ( 1000 );

    lowpassPlot = new FxPlotWindow ( fftLowPassPanel, wxID_ANY, wxPoint ( 0, 0 ), fxPlotSize );
    highpassPlot = new FxPlotWindow ( fftHighPassPanel, wxID_ANY, wxPoint ( 0, 0 ), fxPlotSize );
    bandpassPlot = new FxPlotWindow ( fftBandPassPanel, wxID_ANY, wxPoint ( 0, 0 ), fxPlotSize );
    bandstopPlot = new FxPlotWindow ( fftBandStopPanel, wxID_ANY, wxPoint ( 0, 0 ), fxPlotSize );

    wxStaticBoxSizer
    *fftLowPassBandHiSizer, *fftLowPassSlopeHiSizer, *fftLowPassPlotSizer,
    *fftHighPassBandLoSizer, *fftHighPassSlopeLoSizer, *fftHighPassPlotSizer,
    *fftBandPassBandLoSizer, *fftBandPassSlopeLoSizer, *fftBandPassBandHiSizer, *fftBandPassSlopeHiSizer, *fftBandPassPlotSizer,
    *fftBandStopBandLoSizer, *fftBandStopSlopeLoSizer, *fftBandStopBandHiSizer, *fftBandStopSlopeHiSizer, *fftBandStopPlotSizer;

    fftLowPassPlotSizer = new wxStaticBoxSizer ( wxVERTICAL, fftLowPassPanel, _("preview"));
    fftLowPassPlotSizer->SetMinSize ( fxBoxSize );
    fftLowPassPlotSizer->Add( lowpassPlot );
    fftHighPassPlotSizer = new wxStaticBoxSizer ( wxVERTICAL, fftHighPassPanel, _("preview"));
    fftHighPassPlotSizer->SetMinSize ( fxBoxSize );
    fftHighPassPlotSizer->Add ( highpassPlot );
    fftBandPassPlotSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandPassPanel, _("preview"));
    fftBandPassPlotSizer->SetMinSize ( fxBoxSize );
    fftBandPassPlotSizer->Add ( bandpassPlot );
    fftBandStopPlotSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandStopPanel, _("preview"));
    fftBandStopPlotSizer->SetMinSize ( fxBoxSize );
    fftBandStopPlotSizer->Add ( bandstopPlot );

    fftLowPassBandHiSizer = new wxStaticBoxSizer ( wxVERTICAL, fftLowPassPanel, _("cutoff high (hz)"));
    fftLowPassBandHiSizer->SetMinSize( fxBoxSize );
    fftLowPassSlopeHiSizer = new wxStaticBoxSizer ( wxVERTICAL, fftLowPassPanel, _("slope high"));
    fftLowPassSlopeHiSizer->SetMinSize( fxBoxSize );

    fftHighPassBandLoSizer = new wxStaticBoxSizer ( wxVERTICAL, fftHighPassPanel, _("cutoff low (hz)"));
    fftHighPassBandLoSizer->SetMinSize( fxBoxSize );
    fftHighPassSlopeLoSizer = new wxStaticBoxSizer ( wxVERTICAL, fftHighPassPanel, _("slope low"));
    fftHighPassSlopeLoSizer->SetMinSize( fxBoxSize );

    fftBandPassBandLoSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandPassPanel, _("cutoff low (hz)"));
    fftBandPassBandLoSizer->SetMinSize( fxBoxSize );
    fftBandPassSlopeLoSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandPassPanel, _("slope low"));
    fftBandPassSlopeLoSizer->SetMinSize( fxBoxSize );
    fftBandPassBandHiSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandPassPanel, _("cutoff high (hz)"));
    fftBandPassBandHiSizer->SetMinSize( fxBoxSize );
    fftBandPassSlopeHiSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandPassPanel, _("slope high"));
    fftBandPassSlopeHiSizer->SetMinSize( fxBoxSize );

    fftBandStopBandLoSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandStopPanel, _("cutoff low (hz)"));
    fftBandStopBandLoSizer->SetMinSize( fxBoxSize );
    fftBandStopSlopeLoSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandStopPanel, _("slope low"));
    fftBandStopSlopeLoSizer->SetMinSize( fxBoxSize );
    fftBandStopBandHiSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandStopPanel, _("cutoff high (hz)"));
    fftBandStopBandHiSizer->SetMinSize( fxBoxSize );
    fftBandStopSlopeHiSizer = new wxStaticBoxSizer ( wxVERTICAL, fftBandStopPanel, _("slope high"));
    fftBandStopSlopeHiSizer->SetMinSize( fxBoxSize );

    fftLowPassBandHiSizer->Add ( fftLowPassBandHiSlider );
    fftLowPassSlopeHiSizer->Add ( fftLowPassSlopeHiSlider );

    fftHighPassBandLoSizer->Add ( fftHighPassBandLoSlider );
    fftHighPassSlopeLoSizer->Add ( fftHighPassSlopeLoSlider );

    fftBandPassBandLoSizer->Add ( fftBandPassBandLoSlider );
    fftBandPassSlopeLoSizer->Add ( fftBandPassSlopeLoSlider );
    fftBandPassBandHiSizer->Add ( fftBandPassBandHiSlider );
    fftBandPassSlopeHiSizer->Add ( fftBandPassSlopeHiSlider );

    fftBandStopBandLoSizer->Add ( fftBandStopBandLoSlider );
    fftBandStopSlopeLoSizer->Add ( fftBandStopSlopeLoSlider );
    fftBandStopBandHiSizer->Add ( fftBandStopBandHiSlider );
    fftBandStopSlopeHiSizer->Add ( fftBandStopSlopeHiSlider );

    wxBoxSizer *fftLowPassSizer, *fftHighPassSizer, *fftBandPassSizer, *fftBandStopSizer;

    fftLowPassSizer = new wxBoxSizer ( wxVERTICAL );
    fftLowPassSizer->Add( fftLowPassBandHiSizer );
    fftLowPassSizer->Add ( fftLowPassSlopeHiSizer );
    fftLowPassSizer->Add ( fftLowPassPlotSizer );
    fftLowPassPanel->SetSizer ( fftLowPassSizer );
    fftLowPassSizer->Layout();

    fftHighPassSizer = new wxBoxSizer ( wxVERTICAL );
    fftHighPassSizer->Add ( fftHighPassBandLoSizer );
    fftHighPassSizer->Add ( fftHighPassSlopeLoSizer );
    fftHighPassSizer->Add ( fftHighPassPlotSizer );
    fftHighPassPanel->SetSizer ( fftHighPassSizer );
    fftHighPassSizer->Layout();

    fftBandPassSizer = new wxBoxSizer ( wxVERTICAL );
    fftBandPassSizer->Add ( fftBandPassBandLoSizer );
    fftBandPassSizer->Add ( fftBandPassSlopeLoSizer );
    fftBandPassSizer->Add ( fftBandPassBandHiSizer );
    fftBandPassSizer->Add ( fftBandPassSlopeHiSizer );
    fftBandPassSizer->Add ( fftBandPassPlotSizer );
    fftBandPassPanel->SetSizer ( fftBandPassSizer );
    fftBandPassSizer->Layout();

    fftBandStopSizer = new wxBoxSizer ( wxVERTICAL );
    fftBandStopSizer->Add ( fftBandStopBandLoSizer );
    fftBandStopSizer->Add ( fftBandStopSlopeLoSizer );
    fftBandStopSizer->Add ( fftBandStopBandHiSizer );
    fftBandStopSizer->Add ( fftBandStopSlopeHiSizer );
    fftBandStopSizer->Add ( fftBandStopPlotSizer );
    fftBandStopPanel->SetSizer ( fftBandStopSizer );
    fftBandStopSizer->Layout();

    wxPanel *bitCrushPanel;
    bitCrushPanel = new wxPanel ( effectsTree, wxID_ANY );

    wxString bitCrushTypesChoices[] = { _("4 bit"), _("6 bit"), _("8 bit"), _("10 bit") };
    bitCrushTypesBox = new wxRadioBox ( bitCrushPanel, wxID_ANY, _("type"),
                                        wxDefaultPosition, fxBoxSize, 4, bitCrushTypesChoices, 4, wxRA_SPECIFY_ROWS);
    wxBoxSizer *bitCrushSizer;
    bitCrushSizer = new wxBoxSizer ( wxVERTICAL );
    bitCrushSizer->Add ( bitCrushTypesBox );
    bitCrushSizer->Layout();

    wxPanel *downSamplePanel;
    downSamplePanel = new wxPanel ( effectsTree, wxID_ANY );

    wxString downSampleTypesChoices[] = { _("44,100 Hz"), _("22,050 Hz"), _("11,025 Hz"), _("5,513 Hz"), _("2,756 Hz"), _("1,378 Hz") };
    downSampleTypesBox = new wxRadioBox ( downSamplePanel, wxID_ANY, _("type"),
                                          wxDefaultPosition, fxBoxSize, 6, downSampleTypesChoices, 6, wxRA_SPECIFY_ROWS);
    downSampleTypesBox->SetSelection ( 2 );
    wxBoxSizer *downSampleSizer;
    downSampleSizer = new wxBoxSizer ( wxVERTICAL );
    downSampleSizer->Add ( downSampleTypesBox );
    downSampleSizer->Layout();

    wxPanel *distortPanel;
    distortPanel = new wxPanel ( generalTree, wxID_ANY );


    distortCutoffSlider = new wxSlider ( distortPanel, wxID_ANY,
                                         3000, 0, SHRT_MAX, wxDefaultPosition, generalControlSize,
                                         wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    distortCutoffSlider->SetTickFreq ( 5000, -1 );
    distortCutoffSlider->SetLineSize ( 1 );
    distortCutoffSlider->SetPageSize ( 1 );

    distortPregainSlider = new wxSlider ( distortPanel, wxID_ANY,
                                          100, 0, 300, wxDefaultPosition, generalControlSize,
                                          wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    distortPregainSlider->SetTickFreq ( 50, -1 );
    distortPregainSlider->SetLineSize ( 1 );
    distortPregainSlider->SetPageSize ( 1 );

    distortPostgainSlider = new wxSlider ( distortPanel, wxID_ANY,
                                           200, 0, 500, wxDefaultPosition, generalControlSize,
                                           wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    distortPostgainSlider->SetTickFreq ( 50, -1 );
    distortPostgainSlider->SetLineSize ( 1 );
    distortPostgainSlider->SetPageSize ( 1 );

    distortToneShaping1Slider = new wxSlider ( distortPanel, wxID_ANY,
            5500, 0, 22050, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    distortToneShaping1Slider->SetTickFreq ( 2000, -1 );
    distortToneShaping1Slider->SetLineSize ( 10 );
    distortToneShaping1Slider->SetPageSize ( 100 );

    distortToneShaping2Slider = new wxSlider ( distortPanel, wxID_ANY,
            1000, 0, 22050, wxDefaultPosition, generalControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    distortToneShaping2Slider->SetTickFreq ( 2000, -1 );
    distortToneShaping2Slider->SetLineSize ( 10 );
    distortToneShaping2Slider->SetPageSize ( 100 );

    distortFeedbackSlider = new wxSlider ( distortPanel, wxID_ANY,
                                           1, 0, 5, wxDefaultPosition, generalControlSize,
                                           wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    distortFeedbackSlider->SetTickFreq ( 1, -1 );
    distortFeedbackSlider->SetLineSize ( 1 );
    distortFeedbackSlider->SetPageSize ( 1 );

    distortCheckbox = new wxCheckBox ( distortPanel, wxID_ANY, _("enable master distort") );

    wxBoxSizer *distortSizer;
    distortSizer = new wxBoxSizer ( wxVERTICAL );

    wxStaticBoxSizer *distortPregainSizer, *distortPostgainSizer, *distortCutoffSizer,
    *distortFeedbackSizer, *distortToneShaping1Sizer, *distortToneShaping2Sizer;
    distortCutoffSizer = new wxStaticBoxSizer ( wxVERTICAL, distortPanel, _("cutoff"));
    distortPregainSizer = new wxStaticBoxSizer ( wxVERTICAL, distortPanel, _("pregain (%)"));
    distortToneShaping1Sizer = new wxStaticBoxSizer ( wxVERTICAL, distortPanel, _("filter cutoff (hz)"));
    distortToneShaping2Sizer = new wxStaticBoxSizer ( wxVERTICAL, distortPanel, _("filter slope (hz)"));
    distortFeedbackSizer = new wxStaticBoxSizer ( wxVERTICAL, distortPanel, _("cycles"));
    distortPostgainSizer = new wxStaticBoxSizer ( wxVERTICAL, distortPanel, _("postgain (%)"));

    distortCutoffSizer->Add ( distortCutoffSlider );
    distortCutoffSizer->SetMinSize ( fxBoxSize );
    distortSizer->Add ( distortCutoffSizer );

    distortFeedbackSizer->Add ( distortFeedbackSlider );
    distortFeedbackSizer->SetMinSize ( fxBoxSize );
    distortSizer->Add ( distortFeedbackSizer );

    distortToneShaping1Sizer->Add ( distortToneShaping1Slider );
    distortToneShaping1Sizer->SetMinSize ( fxBoxSize );
    distortSizer->Add ( distortToneShaping1Sizer );

    distortToneShaping2Sizer->Add ( distortToneShaping2Slider );
    distortToneShaping2Sizer->SetMinSize ( fxBoxSize );
    distortSizer->Add ( distortToneShaping2Sizer );

    distortPregainSizer->Add ( distortPregainSlider );
    distortPregainSizer->SetMinSize ( fxBoxSize );
    distortSizer->Add ( distortPregainSizer );

    distortPostgainSizer->Add ( distortPostgainSlider );
    distortPostgainSizer->SetMinSize ( fxBoxSize );
    distortSizer->Add ( distortPostgainSizer );

    distortSizer->AddSpacer ( 5 );
    distortSizer->Add ( distortCheckbox );

    distortPanel->SetSizer ( distortSizer );
    distortSizer->Layout();

    wxPanel *fadeInPanel, *fadeOutPanel;
    fadeInPanel = new wxPanel ( effectsTree, wxID_ANY );
    fadeOutPanel = new wxPanel ( effectsTree, wxID_ANY );

    wxString fadeTypesChoices[] = { _("log10"), _("log5"), _("log3"), _("log1.5") };
    fadeOutTypesBox = new wxRadioBox ( fadeOutPanel, wxID_ANY, _("type"),
                                       wxDefaultPosition, fxBoxSize, 4, fadeTypesChoices, 4, wxRA_SPECIFY_ROWS);
    fadeInTypesBox = new wxRadioBox ( fadeInPanel, wxID_ANY, _("type"),
                                      wxDefaultPosition, fxBoxSize, 4, fadeTypesChoices, 4, wxRA_SPECIFY_ROWS);

    fadeInRangeSlider = new wxSlider ( fadeInPanel, wxID_ANY,
                                       3000, 0, 8000, wxDefaultPosition, fxControlSize,
                                       wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fadeInRangeSlider->SetTickFreq ( 1000, -1 );
    fadeInRangeSlider->SetLineSize ( 50 );
    fadeInRangeSlider->SetPageSize ( 100 );

    fadeOutRangeSlider = new wxSlider ( fadeOutPanel, wxID_ANY,
                                        3000, 0, 8000, wxDefaultPosition, fxControlSize,
                                        wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    fadeOutRangeSlider->SetTickFreq ( 1000, -1 );
    fadeOutRangeSlider->SetLineSize ( 50 );
    fadeOutRangeSlider->SetPageSize ( 100 );

    wxBoxSizer *fadeInSizer, *fadeOutSizer;
    fadeInSizer = new wxBoxSizer ( wxVERTICAL );
    fadeOutSizer = new wxBoxSizer ( wxVERTICAL );

    wxStaticBoxSizer *fadeInRangeSizer, *fadeOutRangeSizer;
    fadeInRangeSizer = new wxStaticBoxSizer ( wxVERTICAL, fadeInPanel, _("range"));
    fadeOutRangeSizer = new wxStaticBoxSizer ( wxVERTICAL, fadeOutPanel, _("range"));

    fadeInRangeSizer->SetMinSize ( fxBoxSize );
    fadeInRangeSizer->Add ( fadeInRangeSlider );
    fadeInTypesBox->SetMinSize ( fxBoxSize );
    fadeOutRangeSizer->SetMinSize ( fxBoxSize );
    fadeOutRangeSizer->Add ( fadeOutRangeSlider );
    fadeOutTypesBox->SetMinSize ( fxBoxSize );

    fadeInSizer->Add ( fadeInTypesBox );
    fadeInSizer->Add ( fadeInRangeSizer );
    fadeInPanel->SetSizer ( fadeInSizer ) ;
    fadeOutSizer->Add ( fadeOutTypesBox );
    fadeOutSizer->Add ( fadeOutRangeSizer );
    fadeOutPanel->SetSizer ( fadeOutSizer ) ;

    fadeInSizer->Layout();
    fadeOutSizer->Layout();

    wxPanel *retriggerPanel;
    retriggerPanel = new wxPanel ( effectsTree, wxID_ANY );

    retriggerCountSlider = new wxSlider ( retriggerPanel, wxID_ANY,
                                          2, 1, 20,
                                          wxDefaultPosition, fxControlSize,
                                          wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    retriggerCountSlider->SetTickFreq ( 1, -1 );
    retriggerCountSlider->SetLineSize ( 1 );
    retriggerCountSlider->SetPageSize ( 2 );

    wxBoxSizer *retriggerSizer;
    retriggerSizer = new wxBoxSizer ( wxVERTICAL );

    wxStaticBoxSizer *retriggerSliderSizer;
    retriggerSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, retriggerPanel, _("trigger"));

    retriggerSliderSizer->Add ( retriggerCountSlider );
    retriggerSliderSizer->SetMinSize ( fxBoxSize );
    retriggerSizer->Add ( retriggerSliderSizer );

    retriggerPanel->SetSizer ( retriggerSizer );
    retriggerSizer->Layout();


    wxPanel *tremoloPanel;
    tremoloPanel = new wxPanel ( effectsTree, wxID_ANY );

    wxString tremoloTypesChoices[] = {_("sine squared"), _("sine"), _("sawtooth"), _("square"), _("triangle")};
    tremoloTypesBox = new wxRadioBox ( tremoloPanel, ID_TREMOLO_TYPES_BOX, _("type"),
                                       wxDefaultPosition, fxBoxSize, 5, tremoloTypesChoices, 5, wxRA_SPECIFY_ROWS);
    tremoloTypesBox->SetSelection( 3 );
    tremoloTypesBox->SetMinSize( fxBoxSize );

    wxString tremoloStereoModeChoices[] = {_("synchronous"), _("ping pong"), _("left delayed"), _("right delayed")};
    tremoloStereoModeBox = new wxRadioBox ( tremoloPanel, ID_TREMOLO_STEREO_MODE_BOX, _("pan"),
                                            wxDefaultPosition, fxBoxSize, 4, tremoloStereoModeChoices, 4, wxRA_SPECIFY_ROWS);
    tremoloStereoModeBox->SetSelection( 0 );
    tremoloStereoModeBox->SetMinSize( fxBoxSize );

    tremoloDepthSlider = new wxSlider ( tremoloPanel, ID_TREMOLO_DEPTH_SLIDER,
                                        100, 0, 100, wxDefaultPosition, fxControlSize,
                                        wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    tremoloDepthSlider->SetTickFreq ( 10, -1 );
    tremoloDepthSlider->SetLineSize ( 5 );
    tremoloDepthSlider->SetPageSize ( 10 );

    tremoloPeriodsPerSliceSlider = new wxSlider ( tremoloPanel, ID_TREMOLO_PPS_SLIDER,
            2, 1, 40, wxDefaultPosition, fxControlSize,
            wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    tremoloPeriodsPerSliceSlider->SetTickFreq ( 10, -1 );
    tremoloPeriodsPerSliceSlider->SetLineSize ( 2 );
    tremoloPeriodsPerSliceSlider->SetPageSize ( 5 );

    wxBoxSizer *tremoloSizer;
    tremoloSizer = new wxBoxSizer ( wxVERTICAL );

    tremoloPlot = new FxPlotWindow ( tremoloPanel, wxID_ANY, wxPoint ( 0, 0 ), fxPlotSize );
    wxStaticBoxSizer *tremoloDepthSizer, *tremoloPeriodsPerSliceSizer, *tremoloPlotSizer;
    tremoloDepthSizer = new wxStaticBoxSizer ( wxVERTICAL, tremoloPanel, _("depth (%)"));
    tremoloPeriodsPerSliceSizer = new wxStaticBoxSizer ( wxVERTICAL, tremoloPanel, _("cycles"));
    tremoloPlotSizer = new wxStaticBoxSizer ( wxVERTICAL, tremoloPanel, _("preview"));

    tremoloDepthSizer->Add ( tremoloDepthSlider );
    tremoloDepthSizer->SetMinSize( fxBoxSize );
    tremoloPeriodsPerSliceSizer->Add ( tremoloPeriodsPerSliceSlider );
    tremoloPeriodsPerSliceSizer->SetMinSize( fxBoxSize );

    tremoloPlotSizer->Add ( tremoloPlot );
    tremoloPlotSizer->SetMinSize ( fxBoxSize );

    tremoloSizer->Add ( tremoloTypesBox );
    tremoloSizer->Add ( tremoloStereoModeBox );
    tremoloSizer->Add ( tremoloDepthSizer );
    tremoloSizer->Add ( tremoloPeriodsPerSliceSizer );
    tremoloSizer->Add ( tremoloPlotSizer );
    tremoloSizer->Layout();

    effectsTree->AddPage( new wxPanel ( effectsTree, wxID_ANY ), _("all") );
    effectsTree->AddSubPage( showFxPanel, _("show") );
    effectsTree->AddPage( new wxPanel ( effectsTree, wxID_ANY ), _("special"));
    effectsTree->AddSubPage( bitCrushPanel, _("bitcrusher"));
    effectsTree->AddSubPage( downSamplePanel, _("downsample"));
    effectsTree->AddSubPage( retriggerPanel, _("retrigger"));
    effectsTree->AddPage( new wxPanel ( effectsTree, wxID_ANY ), _("filters"));
    effectsTree->AddSubPage( fftLowPassPanel, _("lowpass"));
    effectsTree->AddSubPage( fftHighPassPanel, _("highpass"));
    effectsTree->AddSubPage( fftBandPassPanel, _("bandpass"));
    effectsTree->AddSubPage( fftBandStopPanel, _("bandstop"));
    effectsTree->AddPage( new wxPanel ( effectsTree, wxID_ANY ), _("delay"));
    effectsTree->AddSubPage( delayMultiTapPanel, _("multitap"));
    //effectsTree->AddSubPage( new wxPanel ( effectsTree, wxID_ANY ), _("pan"));
    effectsTree->AddPage( new wxPanel ( effectsTree, wxID_ANY ), _("amplitude"));
    effectsTree->AddSubPage( tremoloPanel, _("tremolo"));
    effectsTree->AddSubPage( fadeInPanel, _("fade in"));
    effectsTree->AddSubPage( fadeOutPanel, _("fade out"));
    effectsTree->AddPage( new wxPanel ( effectsTree, wxID_ANY ), _("time/pitch"));
    effectsTree->AddSubPage( new wxPanel ( effectsTree, wxID_ANY ), _("tapestop"));
    effectsTree->AddSubPage( new wxPanel ( effectsTree, wxID_ANY ), _("reverse"));
    effectsTree->GetTreeCtrl()->ExpandAll();
    effectsTree->Layout();

    generalTree->AddPage( distortPanel, _("master distort"));
    generalTree->AddPage ( new wxPanel ( generalTree, wxID_ANY ), _("master filter"));
    generalTree->AddSubPage ( masterFilterPanel, _("filter") );
    generalTree->AddSubPage ( masterFilterResPanel, _("q") );
    generalTree->AddSubPage ( masterFilterLfoPanel, _("lfo") );
    generalTree->AddSubPage ( masterFilterModPanel, _("sweep") );
    generalTree->AddPage( delayVariablePanel, _("master flanger"));
    generalTree->AddPage( delayPhasingPanel, _("master phaser"));
    generalTree->AddPage ( reverbPanel, _("master reverb"));
    generalTree->AddPage ( auxPanel, _("master aux"));
    generalTree->AddPage ( new wxPanel ( generalTree, wxID_ANY ), _("finishing"));
    generalTree->AddSubPage ( compressPanel, _("compress") );
    generalTree->AddSubPage ( declickPanel, _("declick") );
    generalTree->AddSubPage ( enhancePanel, _("enhance") );
    generalTree->AddSubPage ( eqPanel, _("eq") );
    generalTree->AddSubPage ( gatePanel, _("gate") );
    generalTree->AddSubPage ( loudnessPanel, _("loudness") );
    generalTree->AddSubPage ( matchBpmPanel, _("match bpm") );
    generalTree->AddSubPage ( normalizePanel, _("normalize") );
    generalTree->AddSubPage ( pitchAdjustmentPanel, _("pitch") );
    generalTree->AddPage ( new wxPanel ( generalTree, wxID_ANY ), _("warp"));
    generalTree->AddSubPage ( warpGridPanel, _("patterns") );
    generalTree->GetTreeCtrl()->ExpandAll();
    generalTree->Layout();

    int width = 0.98 * bookWidth;
    int widthHalf = 0.42 * bookWidth;

    wxPanel *chainPanel = new wxPanel ( this, wxID_ANY);
    std::vector<wxString> ve;

    effectsGrid->getEffectsVector ( ve );
    effectsChainBox = new ChainBox (
        chainPanel,
        wxID_ANY,
        ve );

    effectsChainBox->SetMinSize ( wxSize ( ( int ) ( 0.44 * width ), 22 * 16 ) );

    wxBoxSizer *effectsChainBoxSizer = new wxBoxSizer ( wxVERTICAL );
    effectsChainBoxSizer->AddSpacer ( 10 );
    effectsChainBoxSizer->Add ( effectsChainBox );

    effectsChainUpButton = new wxButton ( chainPanel, ID_EFFECTSCHAIN_UP, _T ( "up"), wxDefaultPosition, wxSize ( widthHalf, -1 ) );
    effectsChainDownButton = new wxButton ( chainPanel, ID_EFFECTSCHAIN_DOWN, _T ( "down"), wxDefaultPosition, wxSize ( widthHalf, -1 ) );
    effectsChainCloneButton = new wxButton ( chainPanel, ID_EFFECTSCHAIN_CLONE, _T ( "clone"), wxDefaultPosition, wxSize ( widthHalf, -1 ) );
    effectsChainDeleteButton = new wxButton ( chainPanel, ID_EFFECTSCHAIN_DELETE, _T ( "delete"), wxDefaultPosition, wxSize ( widthHalf, -1 ) );
    effectsChainDefaultButton = new wxButton ( chainPanel, ID_EFFECTSCHAIN_DEFAULT, _T ( "default"), wxDefaultPosition, wxSize ( widthHalf, -1 ) );

    wxBoxSizer *effectsChainButtonsSizer = new wxBoxSizer ( wxVERTICAL );
    effectsChainButtonsSizer->AddSpacer ( 10 );
    effectsChainButtonsSizer->Add ( effectsChainUpButton );
    effectsChainButtonsSizer->Add ( effectsChainDownButton );
    effectsChainButtonsSizer->Add ( effectsChainCloneButton );
    effectsChainButtonsSizer->Add ( effectsChainDeleteButton );
    effectsChainButtonsSizer->Add ( effectsChainDefaultButton );

    wxStaticBoxSizer *effectsChainSizer = new wxStaticBoxSizer ( wxHORIZONTAL, chainPanel, _("effects chain"));
    effectsChainSizer->SetMinSize ( wxSize ( ( int ) ( 0.98 * width ), -1 ) );
    effectsChainSizer->AddSpacer ( 5 );
    effectsChainSizer->Add ( effectsChainBoxSizer );
    effectsChainSizer->AddSpacer ( 10 );
    effectsChainSizer->Add ( effectsChainButtonsSizer );

    finishingChainBox = new ChainBox (
        chainPanel,
        wxID_ANY,
        finishingVector );
    finishingChainBox->SetMinSize ( wxSize ( ( int ) ( 0.44 * width ), 12 * 16 ) );

    wxBoxSizer *finishingChainBoxSizer = new wxBoxSizer ( wxVERTICAL );
    finishingChainBoxSizer->AddSpacer ( 10 );
    finishingChainBoxSizer->Add ( finishingChainBox );

    finishingChainUpButton = new wxButton ( chainPanel, ID_FINISHINGCHAIN_UP, _T ( "up"), wxDefaultPosition, wxSize ( widthHalf, -1 ) );
    finishingChainDownButton = new wxButton ( chainPanel, ID_FINISHINGCHAIN_DOWN, _T ( "down"), wxDefaultPosition, wxSize ( widthHalf, -1 ) );
    finishingChainCloneButton = new wxButton ( chainPanel, ID_FINISHINGCHAIN_CLONE, _T ( "clone"), wxDefaultPosition, wxSize ( widthHalf, -1 ) );
    finishingChainDeleteButton = new wxButton ( chainPanel, ID_FINISHINGCHAIN_DELETE, _T ( "delete"), wxDefaultPosition, wxSize ( widthHalf, -1 ) );
    finishingChainDefaultButton = new wxButton ( chainPanel, ID_FINISHINGCHAIN_DEFAULT, _T ( "default"), wxDefaultPosition, wxSize ( widthHalf, -1 ) );

    wxBoxSizer *finishingChainButtonsSizer = new wxBoxSizer ( wxVERTICAL );
    finishingChainButtonsSizer->AddSpacer ( 10 );
    finishingChainButtonsSizer->Add ( finishingChainUpButton );
    finishingChainButtonsSizer->Add ( finishingChainDownButton );
    finishingChainButtonsSizer->Add ( finishingChainCloneButton );
    finishingChainButtonsSizer->Add ( finishingChainDeleteButton );
    finishingChainButtonsSizer->Add ( finishingChainDefaultButton );

    wxStaticBoxSizer *finishingChainSizer = new wxStaticBoxSizer ( wxHORIZONTAL, chainPanel, _("finishing chain"));
    finishingChainSizer->SetMinSize ( wxSize ( ( int ) ( 0.98 * width ), -1 ) );
    finishingChainSizer->AddSpacer ( 5 );
    finishingChainSizer->Add ( finishingChainBoxSizer );
    finishingChainSizer->AddSpacer ( 10 );
    finishingChainSizer->Add ( finishingChainButtonsSizer );

    wxBoxSizer *chainSizer = new wxBoxSizer ( wxVERTICAL );
    chainSizer->AddSpacer ( 10 );
    chainSizer->Add ( effectsChainSizer );
    chainSizer->AddSpacer ( 10 );
    chainSizer->Add ( finishingChainSizer );
    chainPanel->SetSizer ( chainSizer );
    chainSizer->Layout();

    wxPanel *presetPanel;
    presetPanel = new wxPanel ( this, wxID_ANY );

    int listHeight = 190;//225;

    int presetButtonWidth = ( int ) ( 0.97 * width );

    //factoryListView = new wxListView ( presetPanel, ID_FACTORY_LIST, wxDefaultPosition, wxSize ( ( int ) ( width - 5 ), listHeight ), wxLC_REPORT );
    factoryListView = new wxListView ( presetPanel, ID_FACTORY_LIST, wxDefaultPosition, wxSize ( presetButtonWidth, listHeight ), wxLC_REPORT );
    //dbListView = new wxListView ( presetPanel, ID_DB_LIST, wxDefaultPosition, wxSize ( ( int ) ( width - 5 ), listHeight ), wxLC_REPORT ) ;
    dbListView = new wxListView ( presetPanel, ID_DB_LIST, wxDefaultPosition, wxSize ( presetButtonWidth, listHeight ), wxLC_REPORT ) ;

#ifdef __WXMSW__
    dbListView->SetFont( *unicodeFont );
#endif

    factoryListView->InsertColumn ( 0, _T("preset") );
    factoryListView->InsertColumn ( 1, _T("sample") );
    factoryListView->SetColumnWidth ( 0, 120 );
    //factoryListView->SetColumnWidth ( 1, ( int ) ( width - ( 5 + 120 ) ) );
    factoryListView->SetColumnWidth ( 1, presetButtonWidth - ( 5 + 120 ) );

    wxString presetsDir = Dir::getPresetsDir();

    wxString file;
    std::vector<wxString> files;
    if ( wxFileName::DirExists ( presetsDir ) )
    {
        wxString filter;
        filter = presetsDir + _T("*.smash");
        file = wxFindFirstFile ( filter, wxFILE );

        if ( !file.empty() )
        {
            files.push_back ( file );
            for ( ;; )
            {
                file = wxFindNextFile();
                if ( file.empty() )
                    break;
                files.push_back ( file );
            }
        }
    }

    std::sort( files.begin(), files.end() );

    // display preset names
    size_t vectorSize = files.size();
    wxString shortFileName;
    for ( size_t i = 0; i < vectorSize; i++ )
    {
        file = files.at ( i );
        wxFileName fn ( file );
        shortFileName = fn.GetName();
        shortFileName.Replace ( _T("_"), _T(" ") );
        factoryListView->InsertItem ( i, shortFileName, 0 );
        presetMap.insert ( std::make_pair (  shortFileName, file ) );
    }

    // extract fileIn field from files
    wxString fileBuffer, fileIn;
    std::string stdFile;
    for ( size_t i = 0; i < vectorSize; i++ )
    {
        file = files.at ( i );
        stdFile = file.mb_str ( wxConvLocal );
        BinaryFile bf ( ( const char *) stdFile.c_str() );
        fileBuffer = wxString ( bf.getData(), wxConvLocal, bf.getDataLen() );
        wxStringTokenizer st ( fileBuffer, _T("|") );
        if ( st.HasMoreTokens() )
        {
            fileIn = st.GetNextToken();
            fileIn.Replace ( _T("fileIn="), _T("") );
            wxFileName fn ( fileIn );
            fileIn = fn.GetName();
            fileIn.Replace ( _T("_"), _T(" ") );
            factoryListView->SetItem ( i, 1, fileIn );
        }
    }

    dbListView->InsertColumn ( 0, _T("preset") );
    dbListView->InsertColumn ( 1, _T("user") );
    dbListView->InsertColumn ( 2, _T("sample") );
    dbListView->InsertColumn ( 3, _T("rating") );
    dbListView->SetColumnWidth ( 0, 70 );
    dbListView->SetColumnWidth ( 1, 50 );
    dbListView->SetColumnWidth ( 2, 50 );
    dbListView->SetColumnWidth ( 3, ( int ) ( width - ( 5 + 70 + 50 + 50 ) )  );

    wxButton *loadButton = new wxButton ( presetPanel, ID_LOAD_PRESET, _T("load"),
                                          wxDefaultPosition, wxSize ( presetButtonWidth, -1 ) );

    wxButton *refreshButton = new wxButton ( presetPanel, ID_REFRESH_PRESETS, _T("refresh"),
            wxDefaultPosition, wxSize ( presetButtonWidth, -1 ) );
    wxButton *downloadButton = new wxButton ( presetPanel, ID_DOWNLOAD_PRESET, _T("download"),
            wxDefaultPosition, wxSize ( presetButtonWidth, -1 ) );
    wxButton *rateButton = new wxButton ( presetPanel, ID_RATE_PRESET, _T("rate"),
                                          wxDefaultPosition, wxSize ( presetButtonWidth, -1 ) );

    wxStaticBoxSizer *factoryListViewBoxSizer;
    factoryListViewBoxSizer = new wxStaticBoxSizer ( wxVERTICAL, presetPanel, _("factory"));
    factoryListViewBoxSizer->SetMinSize ( wxSize ( ( int ) ( width ), -1 ) );
    factoryListViewBoxSizer->Add ( factoryListView );
    factoryListViewBoxSizer->Add ( loadButton );

    wxStaticBoxSizer *dbListViewBoxSizer;
    dbListViewBoxSizer = new wxStaticBoxSizer ( wxVERTICAL, presetPanel, _("community") );
    dbListViewBoxSizer->SetMinSize ( wxSize ( ( int ) ( width ), -1 ) );
    dbListViewBoxSizer->Add ( dbListView );
    dbListViewBoxSizer->Add ( refreshButton );
    dbListViewBoxSizer->Add ( downloadButton );
    dbListViewBoxSizer->Add ( rateButton );

    wxStaticBoxSizer *samplepacksBoxSizer = new wxStaticBoxSizer ( wxVERTICAL, presetPanel, _("sample packs") );
    samplepacksBoxSizer->SetMinSize ( wxSize ( ( int ) ( width ), -1 ) );
    wxHyperlinkCtrl *browseSamplepacksHyperlink = new wxHyperlinkCtrl (
        presetPanel,
        wxID_ANY,
        _T("browse sample packs online"),
        _T("http://sourceforge.net/projects/smasher/files/smasher-samplepacks/") );
    samplepacksBoxSizer->Add ( browseSamplepacksHyperlink );

    wxBoxSizer *presetSizer = new wxBoxSizer ( wxVERTICAL );

    presetSizer->Add ( factoryListViewBoxSizer );
    presetSizer->AddSpacer ( 5 );
    presetSizer->Add ( dbListViewBoxSizer );
    presetSizer->AddSpacer ( 5 );
    presetSizer->Add ( samplepacksBoxSizer );

    presetPanel->SetSizer ( presetSizer );

    wxPanel *playbackPanel;
    playbackPanel = new wxPanel ( this, wxID_ANY );
    playbackSlider = new wxSlider ( playbackPanel, ID_PLAYBACK_COMPENSATION_SLIDER,
                                    50, 0, 200, wxDefaultPosition, wxSize ( ( int ) (
#ifndef __WXMSW__
                                                                                0.975
#else
                                                                                0.95
#endif
                                                                                * width ), -1 ),
                                    wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    playbackSlider->SetTickFreq ( 20, -1 );
    playbackSlider->SetLineSize ( 10 );
    playbackSlider->SetPageSize ( 20 );
    playbackCheckbox = new wxCheckBox ( playbackPanel, ID_PLAYBACK_COMPENSATION_CHECKBOX, _("enable compensation") );
    wxStaticBoxSizer *playbackSliderSizer;
    playbackSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, playbackPanel, _("gap compensation (ms)"));
    playbackSliderSizer->SetMinSize ( wxSize ( ( int ) ( width ), -1 ) );
    playbackSliderSizer->Add ( playbackSlider );
    wxBoxSizer *playbackSizer;
    playbackSizer = new wxBoxSizer ( wxVERTICAL );
    playbackSizer->Add ( playbackSliderSizer );
    playbackSizer->AddSpacer ( 10 );
    playbackSizer->Add ( playbackCheckbox );
    playbackSizer->AddSpacer ( 10 );
    playbackPanel->SetSizer ( playbackSizer );
    playbackSizer->Layout();

    wxPanel *songPanel;
    songPanel = new wxPanel ( this, wxID_ANY );
    songLengthSlider = new wxSlider ( songPanel, ID_SONG_LENGTH_SLIDER,
                                      4, 2, 8, wxDefaultPosition, wxSize ( ( int ) (
#ifndef __WXMSW__
                                                                               0.975
#else
                                                                               0.95
#endif
                                                                               * width ), -1 ),
                                      wxSL_LABELS | wxSL_HORIZONTAL | wxSL_AUTOTICKS );
    songLengthSlider->SetTickFreq ( 1, -1 );
    songLengthSlider->SetLineSize ( 1 );
    songLengthSlider->SetPageSize ( 1 );
    songShuffleVersesCheckbox = new wxCheckBox ( songPanel, ID_SONG_SHUFFLEVERSES_CHECKBOX, _("apply warp slices") );
    wxStaticBoxSizer *songLengthSliderSizer;
    songLengthSliderSizer = new wxStaticBoxSizer ( wxVERTICAL, songPanel, _("song length"));
    songLengthSliderSizer->SetMinSize ( wxSize ( ( int ) ( width ), -1 ) );
    songLengthSliderSizer->Add ( songLengthSlider );
    wxBoxSizer *songSizer;
    songSizer = new wxBoxSizer ( wxVERTICAL );
    songSizer->Add ( songLengthSliderSizer );
    songSizer->AddSpacer ( 10 );
    songSizer->Add ( songShuffleVersesCheckbox );
    songSizer->AddSpacer ( 10 );
    songPanel->SetSizer ( songSizer );
    songSizer->Layout();


    AddPage ( generalTree, _("master"), false );
    AddPage ( effectsTree, _("fx"), false );
    AddPage ( chainPanel, _("chain"), false );
    AddPage ( presetPanel, _("presets"), false );
    AddPage ( songPanel, _("song"), false );
    AddPage ( playbackPanel, _("audio"), false );

    updateShowFxRows ();

    initCheckboxes();

    Layout();
}

SettingsBook::~SettingsBook()
{
#ifdef __WXMSW__
    delete unicodeFont;
#endif
}

void SettingsBook::OnButton( wxCommandEvent& event )
{
    size_t rowNo, i;
    int showFxId;

    switch ( event.GetId() )
    {
    case ID_DELAY_MULTITAP_PRESET1:
        delayMultiTapDistanceSlider->SetValue ( 1 );
        break;
    case ID_DELAY_MULTITAP_PRESET2:
        delayMultiTapDistanceSlider->SetValue ( 2 );
        break;
    case ID_DELAY_MULTITAP_PRESET3:
        delayMultiTapDistanceSlider->SetValue ( 13 );
        break;
    case ID_DELAY_MULTITAP_PRESET4:
        delayMultiTapDistanceSlider->SetValue ( 25 );
        break;
    case ID_DELAY_MULTITAP_PRESET5:
        delayMultiTapDistanceSlider->SetValue ( 33 );
        break;
    case ID_DELAY_MULTITAP_PRESET6:
        delayMultiTapDistanceSlider->SetValue ( 50 );
        break;
    case ID_DELAY_VARIABLE_PRESET1:
        delayVariableDistanceSlider->SetValue ( 32 );
        break;
    case ID_DELAY_VARIABLE_PRESET2:
        delayVariableDistanceSlider->SetValue ( 64 );
        break;
    case ID_DELAY_VARIABLE_PRESET3:
        delayVariableDistanceSlider->SetValue ( 128 );
        break;
    case ID_DELAY_VARIABLE_PRESET4:
        delayVariableDistanceSlider->SetValue ( 256 );
        break;

    case ID_SHOW_FX_ALL:
        rowNo = effectsGrid->GetNumberRows();
        showFxId = ID_EFFECTS_START;
        for ( i = 0; i < rowNo; i++ )
        {
            showFxId = ID_EFFECTS_START + i;
            wxToggleButton *currentToggleButton = ( wxToggleButton* ) FindWindowById ( showFxId );
            currentToggleButton->SetValue ( 1 );
        }
        updateShowFxRows ();
        break;
    case ID_SHOW_FX_NONE:
        rowNo = effectsGrid->GetNumberRows();
        showFxId = ID_EFFECTS_START;
        for ( i = 0; i < rowNo; i++ )
        {
            showFxId = ID_EFFECTS_START + i;
            wxToggleButton *currentToggleButton = ( wxToggleButton* ) FindWindowById ( showFxId );
            currentToggleButton->SetValue ( 0 );
        }
        updateShowFxRows ();
        break;

    case ID_EFFECTSCHAIN_DEFAULT:
        effectsChainBox->restore();
        break;
    case ID_EFFECTSCHAIN_UP:
        effectsChainBox->moveUp();
        break;
    case ID_EFFECTSCHAIN_DOWN:
        effectsChainBox->moveDown();
        break;
    case ID_EFFECTSCHAIN_CLONE:
        effectsChainBox->clone();
        break;
    case ID_EFFECTSCHAIN_DELETE:
        effectsChainBox->remove();
        break;

    case ID_FINISHINGCHAIN_DEFAULT:
        finishingChainBox->restore();
        break;
    case ID_FINISHINGCHAIN_UP:
        finishingChainBox->moveUp();
        break;
    case ID_FINISHINGCHAIN_DOWN:
        finishingChainBox->moveDown();
        break;
    case ID_FINISHINGCHAIN_CLONE:
        finishingChainBox->clone();
        break;
    case ID_FINISHINGCHAIN_DELETE:
        finishingChainBox->remove();
        break;
    case ID_LOAD_PRESET:
        DoListViewItemActivated( factoryListView->GetFirstSelected() );
        break;
    case ID_REFRESH_PRESETS:
        refreshPresets();
        break;
    case ID_DOWNLOAD_PRESET:
        downloadPreset();
        break;
    case ID_RATE_PRESET:
        ratePreset();
    default:
        break;
    }
    event.Skip();
}

void SettingsBook::updateShowFxRows ()
{
    int rowNo, showFxId, i;
    rowNo = effectsGrid->GetNumberRows();
    showFxId = ID_EFFECTS_START;
    for ( i = 0; i < rowNo; i++ )
    {
        showFxId = ID_EFFECTS_START + i;
        wxToggleButton *currentToggleButton = ( wxToggleButton * ) FindWindowById ( showFxId );
        effectsGrid->showRow ( effectsGrid->GetRowLabelValue( i ), currentToggleButton->GetValue() );
    }
}

void SettingsBook::refreshPlotAreas()
{
    tremoloPlot->Refresh();
    lowpassPlot->Refresh();
    highpassPlot->Refresh();
    bandpassPlot->Refresh();
    bandstopPlot->Refresh();
    loudnessPlot->Refresh();
    masterFilterPlot->Refresh();
    masterFilterLfoPlot->Refresh();
    masterFilterModPlot->Refresh();
}

void SettingsBook::initPlotAreas()
{
    refreshPlotAreas();
    tremoloPlotUpdateRequired = true;
    lowpassPlotUpdateRequired = true;
    highpassPlotUpdateRequired = true;
    bandpassPlotUpdateRequired = true;
    bandstopPlotUpdateRequired = true;
    loudnessPlotUpdateRequired = true;
    masterFilterPlotUpdateRequired = true;
    masterFilterModPlotUpdateRequired = true;
    masterFilterLfoPlotUpdateRequired = true;
}

void SettingsBook::initControls()
{
    initSliders();
    initCheckboxes();
}

void SettingsBook::initSliders()
{
    eqLowGainSlider->SetValue ( 50 );
    eqMidGainSlider->SetValue ( -30 );
    eqHighGainSlider->SetValue ( 20 );
    /*eqCutoffLowSlider->SetValue ( 2000 );
    eqCutoffHighSlider->SetValue ( 4000 );*/
    pitchSemitonesSlider->SetValue ( 0 );
    matchBpmBeatsSlider->SetValue ( 4 );
    matchBpmTargetBpmSlider->SetValue ( 120 );
    normalizeSlider->SetValue ( 99 );
    compressCutoffSlider->SetValue ( 20 );
    compressRangeSlider->SetValue ( 2 );
    loudnessBandLoSlider->SetValue ( 4000 );
    loudnessSlopeLoSlider->SetValue ( 1500 );
    loudnessBandHiSlider->SetValue ( 8000 );
    loudnessSlopeHiSlider->SetValue ( 1500 );
    masterFilterBandLoSlider->SetValue ( 60 );
    masterFilterSlopeLoSlider->SetValue ( 1000 );
    masterFilterBandHiSlider->SetValue ( 6000 );
    masterFilterSlopeHiSlider->SetValue ( 1000 );
    masterFilterResonanceSlider->SetValue ( 2 );
    masterFilterModulationDepthSlider->SetValue ( 100 );
    masterFilterLfoDepthSlider->SetValue ( 100 );
    masterFilterLfoPeriodsPerSliceSlider->SetValue ( 2 );
    reverbRoomsizeSlider->SetValue ( 50 );
    reverbDampSlider->SetValue ( 50 );
    reverbDryMixSlider->SetValue ( 75 );
    delayMultiTapDistanceSlider->SetValue ( 1 );
    delayMultiTapFeedbackSlider->SetValue ( 0 );
    delayMultiTapGainSlider->SetValue ( 1 );
    delayMultiTapStepNoSlider->SetValue ( 1 );
    delayVariableDistanceSlider->SetValue ( 1 );
    delayVariableFeedbackSlider->SetValue ( 0 );
    delayVariableGainSlider->SetValue ( 1 );
    delayVariableLfoDepthSlider->SetValue ( 1 );
    delayVariableLfoPeriodsPerSliceSlider->SetValue ( 25 );
    delayPhasingDistanceOffsetSlider->SetValue ( 440 );
    delayPhasingDistanceRangeSlider->SetValue ( 800 );
    delayPhasingGainSlider->SetValue ( 7 );
    delayPhasingLfoDepthSlider->SetValue ( 1 );
    delayPhasingLfoPeriodsPerSliceSlider->SetValue ( 25 );
    fftLowPassBandHiSlider->SetValue ( 1000 );
    fftLowPassSlopeHiSlider->SetValue ( 1000 );
    fftHighPassBandLoSlider->SetValue ( 5500 );
    fftHighPassSlopeLoSlider->SetValue ( 2500 );
    fftBandPassBandLoSlider->SetValue ( 4500 );
    fftBandPassSlopeLoSlider->SetValue ( 1500 );
    fftBandPassBandHiSlider->SetValue ( 5500 );
    fftBandPassSlopeHiSlider->SetValue ( 1500 );
    fftBandStopBandLoSlider->SetValue ( 2000 );
    fftBandStopSlopeLoSlider->SetValue ( 1500 );
    fftBandStopBandHiSlider->SetValue ( 8000 );
    fftBandStopSlopeHiSlider->SetValue ( 1500 );
    distortCutoffSlider->SetValue ( 3000 );
    distortPregainSlider->SetValue ( 100 );
    distortPostgainSlider->SetValue ( 200 );
    distortToneShaping1Slider->SetValue ( 5500 );
    distortToneShaping2Slider->SetValue ( 1000 );
    distortFeedbackSlider->SetValue ( 1 );
    fadeOutRangeSlider->SetValue ( 3000 );
    retriggerCountSlider->SetValue ( 2 );
    tremoloPeriodsPerSliceSlider->SetValue ( 2 );
    playbackSlider->SetValue ( 50 );
    songLengthSlider->SetValue ( 4 );
}

void SettingsBook::initCheckboxes()
{
    distortCheckbox->SetValue ( false );
    compressCheckbox->SetValue ( false );
    gateCheckbox->SetValue ( false );
    normalizeCheckbox->SetValue ( false );
    loudnessCheckbox->SetValue ( false );
    masterFilterCheckbox->SetValue ( false );
    masterFilterModulationCheckbox->SetValue ( false );
    masterFilterLfoCheckbox->SetValue ( false );
    masterFilterResonanceCheckbox->SetValue ( false );
    reverbCheckbox->SetValue ( false );
    auxCheckbox->SetValue ( false );
    pitchAdjustCheckbox->SetValue ( false );
    playbackCheckbox->SetValue ( false );
    matchBpmCheckbox->SetValue ( false );
    songShuffleVersesCheckbox->SetValue( false );
    declickCheckbox->SetValue ( false );
    enhanceCheckbox->SetValue ( false );
    eqCheckbox->SetValue ( false );
    delayVariableCheckbox->SetValue ( false );
    delayPhasingCheckbox->SetValue ( false );
}

void SettingsBook::OnTreeChanged( wxTreebookEvent& event )
{
    initPlotAreas();
    event.Skip();
}

void SettingsBook::OnBookChanged( wxAuiNotebookEvent& event )
{
    event.Skip();
}

void SettingsBook::OnSize ( wxSizeEvent& event )
{
    initPlotAreas();
    event.Skip();
}

void SettingsBook::OnIdle ( wxIdleEvent& event )
{
    smasherFrame *frame = ( smasherFrame * )GetParent();
    if ( !frame->getViewSettingsState() )
        return;

    if ( tremoloPlotUpdateRequired )
    {
        updateTremoloPlotArea();
        tremoloPlotUpdateRequired = false;
    }
    if ( lowpassPlotUpdateRequired )
    {
        updateLowpassPlotArea();
        lowpassPlotUpdateRequired = false;
    }
    if ( highpassPlotUpdateRequired )
    {
        updateHighpassPlotArea();
        highpassPlotUpdateRequired = false;
    }
    if ( bandpassPlotUpdateRequired )
    {
        updateBandpassPlotArea();
        bandpassPlotUpdateRequired = false;
    }
    if ( bandstopPlotUpdateRequired )
    {
        updateBandstopPlotArea();
        bandstopPlotUpdateRequired = false;
    }
    if ( loudnessPlotUpdateRequired )
    {
        updateLoudnessPlotArea();
        loudnessPlotUpdateRequired = false;
    }
    if ( masterFilterPlotUpdateRequired )
    {
        updateMasterFilterPlotArea();
        masterFilterPlotUpdateRequired = false;
    }
    if ( masterFilterModPlotUpdateRequired )
    {
        updateMasterFilterModPlotArea();
        masterFilterModPlotUpdateRequired = false;
    }
    if ( masterFilterLfoPlotUpdateRequired )
    {
        updateMasterFilterLfoPlotArea();
        masterFilterLfoPlotUpdateRequired = false;
    }
}

void SettingsBook::OnRadioBoxChanged ( wxCommandEvent& event )
{
    switch ( event.GetId() )
    {
    case ID_TREMOLO_TYPES_BOX:
        tremoloPlotUpdateRequired = true;
        tremoloPlot->Refresh();
        break;
    case ID_TREMOLO_STEREO_MODE_BOX:
        tremoloPlotUpdateRequired = true;
        tremoloPlot->Refresh();
        break;
    case ID_MASTER_FILTER_TYPES_BOX:
        masterFilterPlotUpdateRequired = true;
        masterFilterPlot->Refresh();
        break;
    case ID_MASTER_FILTER_MOD_TYPES_BOX:
        masterFilterModPlotUpdateRequired = true;
        masterFilterModPlot->Refresh();
        break;
    case ID_MASTER_FILTER_LFO_TYPES_BOX:
        masterFilterLfoPlotUpdateRequired = true;
        masterFilterLfoPlot->Refresh();
        break;
    default:
        break;
    }
}

void SettingsBook::OnListboxChanged ( wxCommandEvent& event )
{
    switch ( event.GetId() )
    {
    case ID_WARP_PATTERNS_BOX:
        warpGrid->preset ( event.GetSelection() );
        break;
    default:
        break;
    }
    event.Skip ();
}


void SettingsBook::DoListViewItemActivated( int index )
{
    smasherFrame *frame = (smasherFrame *) parent;
    if ( index == -1 )
    {
        wxMessageBox ( _T("please select a preset"),
                       _T("can't load"),
                       wxOK,
                       parent  );
        return;
    }
    frame->activateFactoryPreset ( index, true );
}

void SettingsBook::OnListViewItemActivated ( wxListEvent& event )
{
    DoListViewItemActivated( event.GetIndex() );
    event.Skip ();
}

void SettingsBook::OnCheckboxChanged ( wxCommandEvent& event )
{
    smasherFrame *frame = (smasherFrame *) parent;
    int eventId = event.GetId();

    switch ( eventId )
    {
    case ID_PLAYBACK_COMPENSATION_CHECKBOX:
        frame->setLoopLatency ( ( playbackCheckbox->IsChecked() ) ? playbackSlider->GetValue() : 0);
        break;
    case ID_ADJUST_PITCH_CHECKBOX:
        if ( pitchAdjustCheckbox->IsChecked() )
            matchBpmCheckbox->SetValue( false );
        break;
    case ID_MATCH_BPM_CHECKBOX:
        if ( matchBpmCheckbox->IsChecked() )
            pitchAdjustCheckbox->SetValue( false );
        break;
    default:
        break;
    }

    event.Skip ();
}

void SettingsBook::OnToggleButtonChanged ( wxCommandEvent& event )
{
    int eventId = event.GetId();
    effectsGrid->showRow ( effectsGrid->GetRowLabelValue( eventId - ID_EFFECTS_START ), event.IsChecked() );
    event.Skip();
}

void SettingsBook::OnSliderChanged ( wxScrollEvent& event )
{
    smasherFrame *frame = (smasherFrame *) parent;
    switch ( event.GetId() )
    {
    case ID_TREMOLO_DEPTH_SLIDER:
        tremoloPlotUpdateRequired = true;
        tremoloPlot->Refresh();
        break;

    case ID_TREMOLO_PPS_SLIDER:
        tremoloPlotUpdateRequired = true;
        tremoloPlot->Refresh();
        break;

    case ID_LOWPASS_BAND_HI_SLIDER:
        lowpassPlotUpdateRequired = true;
        lowpassPlot->Refresh();
        break;
    case ID_LOWPASS_SLOPE_HI_SLIDER:
        lowpassPlotUpdateRequired = true;
        lowpassPlot->Refresh();
        break;

    case ID_HIGHPASS_BAND_LO_SLIDER:
        highpassPlotUpdateRequired = true;
        highpassPlot->Refresh();
        break;
    case ID_HIGHPASS_SLOPE_LO_SLIDER:
        highpassPlotUpdateRequired = true;
        highpassPlot->Refresh();
        break;

    case ID_BANDPASS_BAND_LO_SLIDER:
        bandpassPlotUpdateRequired = true;
        bandpassPlot->Refresh();
        break;
    case ID_BANDPASS_SLOPE_LO_SLIDER:
        bandpassPlotUpdateRequired = true;
        bandpassPlot->Refresh();
        break;
    case ID_BANDPASS_BAND_HI_SLIDER:
        bandpassPlotUpdateRequired = true;
        bandpassPlot->Refresh();
        break;
    case ID_BANDPASS_SLOPE_HI_SLIDER:
        bandpassPlotUpdateRequired = true;
        bandpassPlot->Refresh();
        break;

    case ID_BANDSTOP_BAND_LO_SLIDER:
        bandstopPlotUpdateRequired = true;
        bandstopPlot->Refresh();
        break;
    case ID_BANDSTOP_SLOPE_LO_SLIDER:
        bandstopPlotUpdateRequired = true;
        bandstopPlot->Refresh();
        break;
    case ID_BANDSTOP_BAND_HI_SLIDER:
        bandstopPlotUpdateRequired = true;
        bandstopPlot->Refresh();
        break;
    case ID_BANDSTOP_SLOPE_HI_SLIDER:
        bandstopPlotUpdateRequired = true;
        bandstopPlot->Refresh();
        break;
    case ID_LOUDNESS_BAND_LO_SLIDER:
        loudnessPlotUpdateRequired = true;
        loudnessPlot->Refresh();
        break;
    case ID_LOUDNESS_SLOPE_LO_SLIDER:
        loudnessPlotUpdateRequired = true;
        loudnessPlot->Refresh();
        break;
    case ID_LOUDNESS_BAND_HI_SLIDER:
        loudnessPlotUpdateRequired = true;
        loudnessPlot->Refresh();
        break;
    case ID_LOUDNESS_SLOPE_HI_SLIDER:
        loudnessPlotUpdateRequired = true;
        loudnessPlot->Refresh();
        break;
    case ID_MASTER_FILTER_BAND_LO_SLIDER:
        masterFilterPlotUpdateRequired = true;
        masterFilterPlot->Refresh();
        break;
    case ID_MASTER_FILTER_SLOPE_LO_SLIDER:
        masterFilterPlotUpdateRequired = true;
        masterFilterPlot->Refresh();
        break;
    case ID_MASTER_FILTER_BAND_HI_SLIDER:
        masterFilterPlotUpdateRequired = true;
        masterFilterPlot->Refresh();
        break;
    case ID_MASTER_FILTER_SLOPE_HI_SLIDER:
        masterFilterPlotUpdateRequired = true;
        masterFilterPlot->Refresh();
        break;
    case ID_MASTER_FILTER_MOD_DEPTH_SLIDER:
        masterFilterModPlotUpdateRequired = true;
        masterFilterModPlot->Refresh();
        break;
    case ID_MASTER_FILTER_LFO_DEPTH_SLIDER:
        masterFilterLfoPlotUpdateRequired = true;
        masterFilterLfoPlot->Refresh();
        break;
    case ID_MASTER_FILTER_LFO_PPS_SLIDER:
        masterFilterLfoPlotUpdateRequired = true;
        masterFilterLfoPlot->Refresh();
        break;
    case ID_PLAYBACK_COMPENSATION_SLIDER:
        if ( playbackCheckbox->IsChecked() )
            frame->setLoopLatency ( playbackSlider->GetValue() );
        break;
    default:
        break;
    }
    event.Skip();
}

void SettingsBook::updateTremoloPlotArea ()
{
    if ( !tremoloPlot )
    {
        //std::cout << "!tremoloplot" << std::endl;
        return;
    }

    int tremoloType = tremoloTypesBox->GetSelection();
    double tremoloDepth = 0.01 * ( double ) ( tremoloDepthSlider->GetValue() );
    double tremoloPps = ( double ) ( tremoloPeriodsPerSliceSlider->GetValue() );
    double ( *lfo ) ( double );
    switch ( tremoloType )
    {
    case TREMOLO_SAWTOOTH:
        lfo = aaSawTooth;
        break;
    case TREMOLO_SQUARE:
        lfo = aaSquare;
        break;
    case TREMOLO_TRIANGLE:
        lfo = aaTriangle;
        break;
    case TREMOLO_SIN:
        lfo = sinPlus;
        break;
    case TREMOLO_SIN_POW2:
    default:
        lfo = sinPow2;
    }

    tremoloData.clear();

    double increment = ( 2.0 * PI ) / ( double ) fxPlotSize.GetWidth();
    for ( double d = 0.0; d < 2.0 * PI; d += increment )
    {
        tremoloData.push_back (
            ( 1.0 - tremoloDepth ) + tremoloDepth * lfo ( tremoloPps * d ));
    }

    tremoloPlot->set ( &tremoloData );

    if ( tremoloData.empty() )
    {
        //std::cout << "tremoloData.empty()" << std::endl;
        return;
    }

}

void SettingsBook::updateLowpassPlotArea ()
{
    if ( !lowpassPlot )
    {
        //std::cout << "!lowpassPlot" << std::endl;
        return;
    }

    double bh =  ( double ) fftLowPassBandHiSlider->GetValue() * 0.5 / 22050;
    double sh = ( double ) fftLowPassSlopeHiSlider->GetValue() * 0.5 / 22050;

    lowpassData.clear();

    double increment = 0.5 / (double) fxPlotSize.GetWidth();
    for ( double d = 0.0; d < 0.5; d += increment )
    {
        lowpassData.push_back (
            fftLowpassFilter (
                d, 0.0, 0.0,
                bh, sh) );
    }

    lowpassPlot->set ( &lowpassData );

    if ( lowpassData.empty() )
    {
        //std::cout << "lowpassData.empty()" << std::endl;
        return;
    }

}

void SettingsBook::updateHighpassPlotArea ()
{
    if ( !highpassPlot )
    {
        //std::cout << "!highpassPlot" << std::endl;
        return;
    }

    double bl =  ( double ) fftHighPassBandLoSlider->GetValue() * 0.5 / 22050;
    double sl = ( double ) fftHighPassSlopeLoSlider->GetValue() * 0.5 / 22050;

    highpassData.clear();

    double increment = 0.5 / ( double ) fxPlotSize.GetWidth();
    for ( double d = 0.0; d < 0.5; d += increment )
    {
        highpassData.push_back (
            fftHighpassFilter (
                d, bl, sl,
                0.0, 0.0 ) );
    }

    highpassPlot->set ( &highpassData );

    if ( highpassData.empty() )
    {
        //std::cout << "highpassData.empty()" << std::endl;
        return;
    }

}


void SettingsBook::updateBandpassPlotArea ()
{
    if ( !bandpassPlot )
    {
        //std::cout << "!bandpassPlot" << std::endl;
        return;
    }

    double bl =  ( double ) fftBandPassBandLoSlider->GetValue() * 0.5 / 22050;
    double sl = ( double ) fftBandPassSlopeLoSlider->GetValue() * 0.5 / 22050;
    double bh =  ( double ) fftBandPassBandHiSlider->GetValue() * 0.5 / 22050;
    double sh = ( double ) fftBandPassSlopeHiSlider->GetValue() * 0.5 / 22050;

    bandpassData.clear();

    double increment = 0.5 / ( double ) fxPlotSize.GetWidth();
    for ( double d = 0.0; d < 0.5; d += increment )
    {
        bandpassData.push_back (
            fftBandpassFilter (
                d, bl, sl,
                bh, sh) );
    }

    bandpassPlot->set ( &bandpassData );

    if ( bandpassData.empty() )
    {
        //std::cout << "bandpassData.empty()" << std::endl;
        return;
    }

}

void SettingsBook::updateBandstopPlotArea ()
{
    if ( !bandstopPlot )
    {
        //std::cout << "!bandstopPlot" << std::endl;
        return;
    }

    double bl =  ( double ) fftBandStopBandLoSlider->GetValue() * 0.5 / 22050;
    double sl = ( double ) fftBandStopSlopeLoSlider->GetValue() * 0.5 / 22050;
    double bh =  ( double ) fftBandStopBandHiSlider->GetValue() * 0.5 / 22050;
    double sh = ( double ) fftBandStopSlopeHiSlider->GetValue() * 0.5 / 22050;

    bandstopData.clear();

    double increment = 0.5 / ( double ) fxPlotSize.GetWidth();
    for ( double d = 0.0; d < 0.5; d += increment )
    {
        bandstopData.push_back (
            fftBandstopFilter (
                d, bl, sl,
                bh, sh) );
    }

    bandstopPlot->set ( &bandstopData );

    if ( bandstopData.empty() )
    {
        //std::cout << "bandstopData.empty()" << std::endl;
        return;
    }

}

void SettingsBook::updateLoudnessPlotArea ()
{
    if ( !loudnessPlot )
    {
        //std::cout << "!loudnessPlot" << std::endl;
        return;
    }
    double bl =  ( double ) loudnessBandLoSlider->GetValue() * 0.5 / 22050;
    double sl = ( double ) loudnessSlopeLoSlider->GetValue() * 0.5 / 22050;
    double bh =  ( double ) loudnessBandHiSlider->GetValue() * 0.5 / 22050;
    double sh = ( double ) loudnessSlopeHiSlider->GetValue() * 0.5 / 22050;

    loudnessData.clear();

    double increment = 0.5 / ( double ) generalPlotSize.GetWidth();
    for ( double d = 0.0; d < 0.5; d += increment )
    {
        loudnessData.push_back (
            fftBandstopFilter (
                d, bl, sl,
                bh, sh) );
    }

    loudnessPlot->set ( &loudnessData );

    if ( loudnessData.empty() )
    {
        //std::cout << "loudnessData.empty()" << std::endl;
        return;
    }

}


void SettingsBook::updateMasterFilterPlotArea()
{
    if ( !masterFilterPlot )
    {
        //std::cout << "!masterFilterPlot" << std::endl;
        return;
    }

    double ( *filter ) ( double frequency,
                         double bandLo, double slopeLo,
                         double bandHi, double slopeHi);
    double bl, sl, bh, sh;

    switch ( masterFilterTypesBox->GetSelection() )
    {
    case ( FFT_LOWPASS ):
        filter = fftLowpassFilter;
        bl = sl = 0.0;
        bh = ( double ) masterFilterBandHiSlider->GetValue() * 0.5 / 22050;
        sh = ( double ) masterFilterSlopeHiSlider->GetValue() * 0.5 / 22050;
        break;
    case ( FFT_HIGHPASS ):
        filter = fftHighpassFilter;
        bl = ( double ) masterFilterBandLoSlider->GetValue() * 0.5 / 22050;
        sl = ( double ) masterFilterSlopeLoSlider->GetValue() * 0.5 / 22050;
        bh = sh = 0.0;
        break;
    case ( FFT_BANDPASS ):
        filter = fftBandpassFilter;
        bl = ( double ) masterFilterBandLoSlider->GetValue() * 0.5 / 22050;
        sl = ( double ) masterFilterSlopeLoSlider->GetValue() * 0.5 / 22050;
        bh = ( double ) masterFilterBandHiSlider->GetValue() * 0.5 / 22050;
        sh = ( double ) masterFilterSlopeHiSlider->GetValue() * 0.5 / 22050;
        break;
    case ( FFT_BANDSTOP ):
        filter = fftBandstopFilter;
        bl = ( double ) masterFilterBandLoSlider->GetValue() * 0.5 / 22050;
        sl = ( double ) masterFilterSlopeLoSlider->GetValue() * 0.5 / 22050;
        bh = ( double ) masterFilterBandHiSlider->GetValue() * 0.5 / 22050;
        sh = ( double ) masterFilterSlopeHiSlider->GetValue() * 0.5 / 22050;
        break;
    default:
        filter = fftLowpassFilter;
        bl = sl = 0.0;
        bh = ( double ) masterFilterBandHiSlider->GetValue() * 0.5 / 22050;
        sh = ( double ) masterFilterSlopeHiSlider->GetValue() * 0.5 / 22050;
        break;
    }

    masterFilterData.clear();

    double increment = 0.5 / ( double ) generalPlotSize.GetWidth();
    for ( double d = 0.0; d < 0.5; d += increment )
    {
        masterFilterData.push_back (
            filter (
                d, bl, sl,
                bh, sh) );
    }

    masterFilterPlot->set ( &masterFilterData );

    if ( masterFilterData.empty() )
    {
        //std::cout << "masterFilterData.empty()" << std::endl;
        return;
    }

}

void SettingsBook::updateMasterFilterModPlotArea()
{
    if ( !masterFilterModPlot )
    {
        //std::cout << "!masterFilterModPlot" << std::endl;
        return;
    }

    double ( *sweep ) ( double frequency, double limit );
    double depth = 0.01 * masterFilterModulationDepthSlider->GetValue();

    switch ( masterFilterModulationTypesBox->GetSelection() )
    {
    case ( FFT_MOD_SWEEP_IN ):
        sweep = sweepIn;
        break;
    case ( FFT_MOD_SWEEP_OUT ):
        sweep = sweepOut;
        break;
    default:
        sweep = sweepIn;
        break;
    }

    masterFilterModData.clear();

    double increment = 0.5 / ( double ) generalPlotSize.GetWidth();
    for ( double d = 0.0; d < 0.5; d += increment )
    {
        masterFilterModData.push_back (
            ( 1.0 - depth ) + depth * ( sweep ( d, 0.5 ) ) );
    }

    masterFilterModPlot->set ( &masterFilterModData );

    if ( masterFilterModData.empty() )
    {
        //std::cout << "masterFilterModData.empty()" << std::endl;
        return;
    }

}


void SettingsBook::updateMasterFilterLfoPlotArea()
{
    if ( !masterFilterLfoPlot )
    {
        //std::cout << "!masterFilterLfoPlot" << std::endl;
        return;
    }

    int lfoType = masterFilterLfoTypesBox->GetSelection();
    double masterFilterLfoDepth = 0.01 * ( double ) ( masterFilterLfoDepthSlider->GetValue() );
    double masterFilterLfoPps = ( double ) ( masterFilterLfoPeriodsPerSliceSlider->GetValue() );
    double ( *lfo ) ( double );
    switch ( lfoType )
    {
    case FFT_LFO_SINPLUS:
        lfo = sinPlus;
        break;
    case FFT_LFO_SINPOW2:
        lfo = sinPow2;
        break;
    case FFT_LFO_SAWTOOTH:
        lfo = aaSawTooth;
        break;
    case FFT_LFO_SQUARE:
        lfo = aaSquare;
        break;
    case FFT_LFO_TRIANGLE:
        lfo = aaTriangle;
        break;
    case FFT_LFO_SAMPLEANDHOLD:
        lfo = sampleAndHold;
        break;
    default:
        lfo = passThrough;
        break;
    }

    masterFilterLfoData.clear();

    double increment = ( 2.0 * PI ) / generalPlotSize.GetWidth();
    for ( double d = 0.0; d < 2.0 * PI; d += increment )
    {
        masterFilterLfoData.push_back (
            ( 1.0 - masterFilterLfoDepth ) + masterFilterLfoDepth * lfo ( masterFilterLfoPps * d ));
    }

    masterFilterLfoPlot->set ( &masterFilterLfoData );

    if ( masterFilterLfoData.empty() )
    {
        //std::cout << "masterFilterLfoData.empty()" << std::endl;
        return;
    }

}

void SettingsBook::OnUpdateChainButton ( wxUpdateUIEvent& event )
{
    int id = event.GetId();

    bool b = true;
    switch ( id )
    {
    case ID_EFFECTSCHAIN_UP:
    case ID_EFFECTSCHAIN_DOWN:
    case ID_EFFECTSCHAIN_CLONE:
        b = effectsChainBox->hasSelection();
        break;
    case ID_EFFECTSCHAIN_DELETE:
        b = effectsChainBox->getUpdateDelete();
        break;
    case ID_FINISHINGCHAIN_UP:
    case ID_FINISHINGCHAIN_DOWN:
    case ID_FINISHINGCHAIN_CLONE:
        b = finishingChainBox->hasSelection();
        break;
    case ID_FINISHINGCHAIN_DELETE:
        b = finishingChainBox->getUpdateDelete();
        break;
    }
    event.Enable ( b );
}

void SettingsBook::initPresets()
{
    wxString s = SMASHER_DEFAULT_COMMUNITY_PRESETS;
    refreshPresets ( s );
}

void SettingsBook::refreshPresets()
{
    wxString s;
    if ( !SmasherDb::getTitles ( s ) )
    {
        wxMessageBox ( _T("check internet connection"),
                       _T("can't refresh"),
                       wxOK,
                       parent  );
        return;
    }

    refreshPresets ( s );
}

void SettingsBook::refreshPresets ( const wxString& s )
{

    communityPresets = s;

    wxStringTokenizer tokenizer( s, _T("%") );

    std::vector< std::vector<wxString> > v;
    std::vector<wxString> tokenVector;

    while ( tokenizer.HasMoreTokens() )
    {
        tokenVector.push_back ( tokenizer.GetNextToken() );
    }

    std::vector<wxString>::iterator it;
    std::vector<wxString> subVector;

    for ( it = tokenVector.begin(); it != tokenVector.end(); it++ )
    {
        subVector.clear();
        wxStringTokenizer subTokenizer ( *it, _T(",") );
        while ( subTokenizer.HasMoreTokens() )
        {
            subVector.push_back ( subTokenizer.GetNextToken() );
        }

        if ( subVector.size() < 3 )
        {
            continue;
        }

        v.push_back ( subVector );
    }

    dbListView->DeleteAllItems();
    if ( v.empty() )
        return;

    size_t vectorSize = v.size();
    wxString title, author, sample, starCount, ratingCount, ratingString;
    long starCountLong, ratingCountLong;

    for ( size_t i = 0; i < vectorSize; i++ )
    {
        title = v[i].at ( 0 );
        author = v[i].at ( 1 );
        sample = v[i].at ( 2 );
        starCount = v[i].at ( 3 );
        ratingCount = v[i].at ( 4 );

        starCount.ToLong ( &starCountLong );
        ratingCount.ToLong ( &ratingCountLong );

        if ( !ratingCountLong )
        {
            ratingString = _T("\u2606\u2606\u2606\u2606\u2606 (0 votes)");
            dbListView->SetItemTextColour( (long) i, wxColour ( 128, 128, 128 ) );
        }
        else
        {
            ratingString = getRatingString ( (size_t) SampleMath::doubleToShort( ( 1.0 * starCountLong ) / ( 1.0 * ratingCountLong ) ) );
            ratingString += _T( " (" );
            ratingString << ratingCountLong;
            ratingString += ( ratingCountLong == 1 )? _T( " vote)" ) : _T( " votes)" );
        }

        dbListView->InsertItem ( i, title, 0 );
        dbListView->SetItem ( i, 1, author );
        dbListView->SetItem ( i, 2, sample );
        dbListView->SetItem ( i, 3, ratingString );
    }
}

void SettingsBook::downloadPreset()
{
    long item = dbListView->GetFirstSelected();
    if ( item == -1 )
    {
        wxMessageBox ( _T("please select a preset"),
                       _T("can't download"),
                       wxOK,
                       parent  );
        return;
    }
    wxString title, author, preset;
    title = dbListView->GetItemText ( item );

    // obtain text from 2nd column
    wxListItem Item;
    Item.SetId ( item );
    Item.SetColumn ( 1 );
    Item.SetMask ( wxLIST_MASK_TEXT );
    dbListView->GetItem ( Item );
    author = Item.GetText();

    if ( !SmasherDb::getPreset ( title, author, preset ) )
    {
        wxMessageBox ( _T("check internet connection"),
                       _T("can't download"),
                       wxOK,
                       parent  );
        return;
    }

    smasherFrame *frame = ( smasherFrame * ) parent;

    initControls();
    frame->loadState ( preset, true );
}


void SettingsBook::ratePreset ()
{
    wxString title;
    title = _T( "rate preset");

    smasherFrame *frame = ( smasherFrame * ) GetParent();

    long focus = dbListView->GetFocusedItem();
    if ( focus == -1 )
    {
        wxMessageBox ( _T("please select a preset"),
                       _T("can't rate"),
                       wxOK,
                       parent  );
        return;
    }
    wxString preset = dbListView->GetItemText ( focus );

    RateDialog rd ( this, preset, preset, frame->getShareAuthor(), frame->getSharePasscode(), frame->getShareEmail() );
    if ( rd.ShowModal() == wxID_OK )
        refreshPresets();
}


wxString SettingsBook::getRatingString ( size_t i )
{
    wxString ret;
    switch ( i )
    {
    case 1:
        ret = _T("\u2605\u2606\u2606\u2606\u2606");
        break;
    case 2:
        ret = _T("\u2605\u2605\u2606\u2606\u2606");
        break;
    case 3:
        ret = _T("\u2605\u2605\u2605\u2606\u2606");
        break;
    case 4:
        ret = _T("\u2605\u2605\u2605\u2605\u2606");
        break;
    case 5:
        ret = _T("\u2605\u2605\u2605\u2605\u2605");
        break;
    default:
        ret = _T("\u2605\u2605\u2605\u2606\u2606");
        break;
    }
    return ret;
}

int wxCALLBACK SettingsBook::compareFunction (
    long item1,
    long item2,
    long sortData )
{
    SortData *data = ( SortData * ) sortData;
    int column;
    wxListView *table;
    column = data->column;
    table = data->table;

    wxString string1 = getTextByColumn ( table, item1, column );
    wxString string2 = getTextByColumn ( table, item2, column );

    if ( string1.CmpNoCase ( string2 ) < 0 )
        return -1;
    else if ( string1.CmpNoCase ( string2 ) > 0 )
        return 1;
    else
        return 0;
}

wxString SettingsBook::getTextByColumn ( wxListView *table, long index, int col )
{
    wxListItem Item;
    Item.SetId ( index );
    Item.SetColumn ( col );
    Item.SetMask ( wxLIST_MASK_TEXT );
    table->GetItem ( Item );
    return Item.GetText();
}

void SettingsBook::OnColumnClick ( wxListEvent& event )
{
    std::auto_ptr<SortData> data ( new SortData );
    data->column = event.GetColumn();

    wxListView *list = NULL;

    switch ( event.GetId() )
    {
    case ID_FACTORY_LIST:
        list = factoryListView;
        break;
    case ID_DB_LIST:
        list = dbListView;
        break;
    }
    data->table = list;

    long itemCount = list->GetItemCount();

    for ( long i = 0; i < itemCount; i++ )
    {
        list->SortItems ( compareFunction, ( long ) data.get() );
        for ( int i = 0; i < itemCount; i++ )
            list->SetItemData ( i, i );
    }
}
