#ifndef SETTINGSBOOK_H
#define SETTINGSBOOK_H

#include <vector>
#include <map>

#include <wx/aui/aui.h>
#include <wx/gdicmn.h>
#include <wx/listctrl.h>

enum
{
    ID_TREMOLO_TYPES_BOX = 1000,
    ID_TREMOLO_STEREO_MODE_BOX,
    ID_TREMOLO_DEPTH_SLIDER,
    ID_TREMOLO_PPS_SLIDER,
    ID_EFFECTSTREE,
    ID_GENERALTREE,
    ID_LOWPASS_BAND_HI_SLIDER,
    ID_LOWPASS_SLOPE_HI_SLIDER,
    ID_HIGHPASS_BAND_LO_SLIDER,
    ID_HIGHPASS_SLOPE_LO_SLIDER,
    ID_BANDPASS_BAND_LO_SLIDER,
    ID_BANDPASS_SLOPE_LO_SLIDER,
    ID_BANDPASS_BAND_HI_SLIDER,
    ID_BANDPASS_SLOPE_HI_SLIDER,
    ID_BANDSTOP_BAND_LO_SLIDER,
    ID_BANDSTOP_SLOPE_LO_SLIDER,
    ID_BANDSTOP_BAND_HI_SLIDER,
    ID_BANDSTOP_SLOPE_HI_SLIDER,
    ID_DELAY_MULTITAP_PRESET1,
    ID_DELAY_MULTITAP_PRESET2,
    ID_DELAY_MULTITAP_PRESET3,
    ID_DELAY_MULTITAP_PRESET4,
    ID_DELAY_MULTITAP_PRESET5,
    ID_DELAY_MULTITAP_PRESET6,
    ID_DELAY_VARIABLE_PRESET1,
    ID_DELAY_VARIABLE_PRESET2,
    ID_DELAY_VARIABLE_PRESET3,
    ID_DELAY_VARIABLE_PRESET4,
    ID_COMPRESS_BOX,
    ID_GATE_BOX,
    ID_NORMALIZE_BOX,
    ID_LOUDNESS_BAND_LO_SLIDER,
    ID_LOUDNESS_SLOPE_LO_SLIDER,
    ID_LOUDNESS_BAND_HI_SLIDER,
    ID_LOUDNESS_SLOPE_HI_SLIDER,
    ID_LOUDNESS_BOX,
    ID_DEFAULT_SAMPLE_BOX,
    ID_MASTER_FILTER_BOX,
    ID_MASTER_FILTER_TYPES_BOX,
    ID_MASTER_FILTER_BAND_LO_SLIDER,
    ID_MASTER_FILTER_SLOPE_LO_SLIDER,
    ID_MASTER_FILTER_BAND_HI_SLIDER,
    ID_MASTER_FILTER_SLOPE_HI_SLIDER,
    ID_MASTER_FILTER_MOD_BOX,
    ID_MASTER_FILTER_MOD_TYPES_BOX,
    ID_MASTER_FILTER_MOD_DEPTH_SLIDER,
    ID_MASTER_FILTER_LFO_TYPES_BOX,
    ID_MASTER_FILTER_LFO_DEPTH_SLIDER,
    ID_MASTER_FILTER_LFO_PPS_SLIDER,
    ID_MASTER_FILTER_LFO_BOX,
    ID_MASTER_FILTER_RESONANCE_SLIDER,
    ID_MASTER_FILTER_RESONANCE_BOX,
    ID_DELAY_LFO_TYPES_BOX,
    ID_DELAY_VARIABLE_LFO_DEPTH_SLIDER,
    ID_DELAY_VARIABLE_PPS_SLIDER,
    ID_PLAYBACK_COMPENSATION_SLIDER,
    ID_PLAYBACK_COMPENSATION_CHECKBOX,
    ID_ADJUST_PITCH_CHECKBOX,
    ID_MATCH_BPM_CHECKBOX,
    ID_SHOW_FX_ALL,
    ID_SHOW_FX_NONE,
    ID_EFFECTSCHAIN_UP,
    ID_EFFECTSCHAIN_DOWN,
    ID_EFFECTSCHAIN_DELETE,
    ID_EFFECTSCHAIN_CLONE,
    ID_EFFECTSCHAIN_DEFAULT,
    ID_FINISHINGCHAIN_UP,
    ID_FINISHINGCHAIN_DOWN,
    ID_FINISHINGCHAIN_DELETE,
    ID_FINISHINGCHAIN_CLONE,
    ID_FINISHINGCHAIN_DEFAULT,
    ID_SONG_LENGTH_SLIDER,
    ID_SONG_SHUFFLEVERSES_CHECKBOX,
    ID_WARP_PATTERNS_BOX,
    ID_DOWNLOAD_PRESET,
    ID_RATE_PRESET,
    ID_REFRESH_PRESETS,
    ID_LOAD_PRESET,
    ID_FACTORY_LIST,
    ID_DB_LIST,
    ID_EFFECTS_START = 2000
};

class wxButton;
class wxCheckBox;
class wxRadioBox;
class wxListBox;
class wxSlider;
class wxTreebook;
class wxTextCtrl;
class wxWindow;
class wxListView;

class wxCommandEvent;
class wxIdleEvent;
class wxSizeEvent;
class wxScrollEvent;
class wxTreebookEvent;

class EffectsGrid;
class WarpGrid;
class ChainBox;
class FxPlotWindow;
class wxFont;

struct SortData
{
    int column;
    wxListView *table;
};


class SettingsBook : public wxAuiNotebook
{
public:
    SettingsBook ( wxWindow *parent, int id, EffectsGrid *effectsGrid, int bookWidth, const std::vector<wxString> &v );
    ~SettingsBook();
    wxRadioBox *getFadeInTypesBox () {
        return fadeInTypesBox;
    }
    wxRadioBox *getFadeOutTypesBox () {
        return fadeOutTypesBox;
    }
    wxRadioBox *getFftFilterTypesBox () {
        return fftFilterTypesBox;
    }
    wxRadioBox *getMasterFilterTypesBox () {
        return masterFilterTypesBox;
    }
    wxRadioBox *getMasterFilterModulationTypesBox () {
        return masterFilterModulationTypesBox;
    }
    wxRadioBox *getMasterFilterLfoTypesBox () {
        return masterFilterLfoTypesBox;
    }
    wxRadioBox *getTremoloTypesBox () {
        return tremoloTypesBox;
    }
    wxRadioBox *getDelayVariableLfoTypesBox () {
        return delayVariableLfoTypesBox;
    }
    wxRadioBox *getDelayPhasingLfoTypesBox () {
        return delayPhasingLfoTypesBox;
    }
    wxRadioBox *getTremoloStereoModeBox () {
        return tremoloStereoModeBox;
    }
    wxRadioBox *getBitCrushTypesBox () {
        return bitCrushTypesBox;
    }
    wxRadioBox *getDownSampleTypesBox () {
        return downSampleTypesBox;
    }
    wxListBox *getDefaultTypesBox () {
        return defaultTypesBox;
    }
    wxSlider *getRetriggerCountSlider () {
        return retriggerCountSlider;
    }
    wxSlider *getTremoloDepthSlider () {
        return tremoloDepthSlider;
    }
    wxSlider *getTremoloPeriodsPerSliceSlider () {
        return tremoloPeriodsPerSliceSlider;
    }
    wxSlider *getCompressCutoffSlider () {
        return compressCutoffSlider;
    }
    wxSlider *getCompressRangeSlider () {
        return compressRangeSlider;
    }
    wxSlider *getDelayMultiTapDistanceSlider () {
        return delayMultiTapDistanceSlider;
    }
    wxSlider *getDelayMultiTapFeedbackSlider () {
        return delayMultiTapFeedbackSlider;
    }
    wxSlider *getDelayMultiTapGainSlider () {
        return delayMultiTapGainSlider;
    }
    wxSlider *getDelayMultiTapStepNoSlider () {
        return delayMultiTapStepNoSlider;
    }
    wxSlider *getDelayVariableDistanceSlider () {
        return delayVariableDistanceSlider;
    }
    wxSlider *getDelayVariableFeedbackSlider () {
        return delayVariableFeedbackSlider;
    }
    wxSlider *getDelayVariableGainSlider () {
        return delayVariableGainSlider;
    }
    wxSlider *getDelayVariableLfoDepthSlider () {
        return delayVariableLfoDepthSlider;
    }
    wxSlider *getDelayVariableLfoPeriodsPerSliceSlider () {
        return delayVariableLfoPeriodsPerSliceSlider;
    }
    wxSlider *getDelayPhasingDistanceOffsetSlider()
    {
        return delayPhasingDistanceOffsetSlider;
    }
    wxSlider *getDelayPhasingDistanceRangeSlider()
    {
        return delayPhasingDistanceRangeSlider;
    }
    wxSlider *getDelayPhasingGainSlider()
    {
        return delayPhasingGainSlider;
    }
    wxSlider *getDelayPhasingLfoDepthSlider()
    {
        return delayPhasingLfoDepthSlider;
    }
    wxSlider *getDelayPhasingLfoPeriodsPerSliceSlider()
    {
        return delayPhasingLfoPeriodsPerSliceSlider;
    }
    wxSlider *getDistortPregainSlider () {
        return distortPregainSlider;
    }
    wxSlider *getDistortPostgainSlider () {
        return distortPostgainSlider;
    }
    wxSlider *getDistortCutoffSlider () {
        return distortCutoffSlider;
    }
    wxSlider *getDistortToneShaping1Slider () {
        return distortToneShaping1Slider;
    }
    wxSlider *getDistortToneShaping2Slider () {
        return distortToneShaping2Slider;
    }
    wxSlider *getDistortFeedbackSlider () {
        return distortFeedbackSlider;
    }
    wxSlider *getFadeOutRangeSlider () {
        return fadeOutRangeSlider;
    }
    wxSlider *getFadeInRangeSlider () {
        return fadeInRangeSlider;
    }
    wxSlider *getFftLowPassBandHiSlider () {
        return fftLowPassBandHiSlider;
    }
    wxSlider *getFftLowPassSlopeHiSlider () {
        return fftLowPassSlopeHiSlider;
    }
    wxSlider *getFftHighPassBandLoSlider () {
        return fftHighPassBandLoSlider;
    }
    wxSlider *getFftHighPassSlopeLoSlider () {
        return fftHighPassSlopeLoSlider;
    }
    wxSlider *getFftBandPassBandLoSlider () {
        return fftBandPassBandLoSlider;
    }
    wxSlider *getFftBandPassSlopeLoSlider () {
        return fftBandPassSlopeLoSlider;
    }
    wxSlider *getFftBandPassBandHiSlider () {
        return fftBandPassBandHiSlider;
    }
    wxSlider *getFftBandPassSlopeHiSlider () {
        return fftBandPassSlopeHiSlider;
    }
    wxSlider *getFftBandStopBandLoSlider () {
        return fftBandStopBandLoSlider;
    }
    wxSlider *getFftBandStopSlopeLoSlider () {
        return fftBandStopSlopeLoSlider;
    }
    wxSlider *getFftBandStopBandHiSlider () {
        return fftBandStopBandHiSlider;
    }
    wxSlider *getFftBandStopSlopeHiSlider () {
        return fftBandStopSlopeHiSlider;
    }
    wxSlider *getLoudnessBandLoSlider () {
        return loudnessBandLoSlider;
    }
    wxSlider *getLoudnessSlopeLoSlider () {
        return loudnessSlopeLoSlider;
    }
    wxSlider *getLoudnessBandHiSlider () {
        return loudnessBandHiSlider;
    }
    wxSlider *getLoudnessSlopeHiSlider () {
        return loudnessSlopeHiSlider;
    }
    wxSlider *getMasterFilterBandLoSlider () {
        return masterFilterBandLoSlider;
    }
    wxSlider *getMasterFilterSlopeLoSlider () {
        return masterFilterSlopeLoSlider;
    }
    wxSlider *getMasterFilterBandHiSlider () {
        return masterFilterBandHiSlider;
    }
    wxSlider *getMasterFilterSlopeHiSlider () {
        return masterFilterSlopeHiSlider;
    }
    wxSlider *getMasterFilterModulationDepthSlider() {
        return masterFilterModulationDepthSlider;
    }
    wxSlider *getMasterFilterLfoDepthSlider () {
        return masterFilterLfoDepthSlider;
    }
    wxSlider *getMasterFilterLfoPeriodsPerSliceSlider () {
        return masterFilterLfoPeriodsPerSliceSlider;
    }
    wxSlider *getMasterFilterResonanceSlider () {
        return masterFilterResonanceSlider;
    }
    wxSlider *getPitchSemitonesSlider () {
        return pitchSemitonesSlider;
    }
    wxSlider *getPlaybackSlider () {
        return playbackSlider;
    }
    wxSlider *getNormalizeSlider () {
        return normalizeSlider;
    }
    wxSlider *getSongLengthSlider () {
        return songLengthSlider;
    }
    wxSlider *getReverbRoomsizeSlider () {
        return reverbRoomsizeSlider;
    }
    wxSlider *getReverbDampSlider () {
        return reverbDampSlider;
    }
    wxSlider *getMatchBpmBeatsSlider () {
        return matchBpmBeatsSlider;
    }
    wxSlider *getMatchBpmTargetBpmSlider () {
        return matchBpmTargetBpmSlider;
    }
    wxSlider *getReverbDryMixSlider () {
        return reverbDryMixSlider;
    }
    wxSlider *getEqLowGainSlider () {
        return eqLowGainSlider;
    }
    wxSlider *getEqMidGainSlider () {
        return eqMidGainSlider;
    }
    wxSlider *getEqHighGainSlider () {
        return eqHighGainSlider;
    }
    WarpGrid *getWarpGrid () {
        return warpGrid;
    }
    wxCheckBox *getEqCheckbox () {
        return eqCheckbox;
    }
    wxCheckBox *getDistortCheckbox () {
        return distortCheckbox;
    }
    wxCheckBox *getCompressCheckbox () {
        return compressCheckbox;
    }
    wxCheckBox *getGateCheckbox () {
        return gateCheckbox;
    }
    wxCheckBox *getNormalizeCheckbox () {
        return normalizeCheckbox;
    }
    wxCheckBox *getLoudnessCheckbox () {
        return loudnessCheckbox;
    }
    wxCheckBox *getMasterFilterCheckbox () {
        return masterFilterCheckbox;
    }
    wxCheckBox *getMasterFilterModulationCheckbox () {
        return masterFilterModulationCheckbox;
    }
    wxCheckBox *getMasterFilterLfoCheckbox () {
        return masterFilterLfoCheckbox;
    }
    wxCheckBox *getMasterFilterResonanceCheckbox () {
        return masterFilterResonanceCheckbox;
    }
    wxCheckBox *getReverbCheckbox () {
        return reverbCheckbox;
    }
    wxCheckBox *getAuxCheckbox () {
        return auxCheckbox;
    }
    wxCheckBox *getPitchAdjustCheckbox () {
        return pitchAdjustCheckbox;
    }
    wxCheckBox *getPlaybackCheckbox () {
        return playbackCheckbox;
    }
    wxCheckBox *getMatchBpmCheckbox () {
        return matchBpmCheckbox;
    }
    wxCheckBox *getSongShuffleVersesCheckbox () {
        return songShuffleVersesCheckbox;
    }
    wxCheckBox *getDeclickCheckbox() {
        return declickCheckbox;
    }
    wxCheckBox *getEnhanceCheckbox() {
        return enhanceCheckbox;
    }
    wxCheckBox *getDelayVariableCheckbox() {
        return delayVariableCheckbox;
    }
    wxCheckBox *getDelayPhasingCheckbox() {
        return delayPhasingCheckbox;
    }
    ChainBox *getEffectsChainBox() {
        return effectsChainBox;
    }
    ChainBox *getFinishingChainBox() {
        return finishingChainBox;
    }
    wxListView *getFactoryListView() {
        return factoryListView;
    }
    void getFilesMap ( std::map<wxString, wxString>& map ) {
        map = presetMap;
    }
    wxString getCommunityPresets() {
        return communityPresets;
    }
    void initPlotAreas();
    void refreshPlotAreas();
    void updateShowFxRows ();
    void initPresets();
    void refreshPresets();
    void refreshPresets ( const wxString& s );
    void DoListViewItemActivated ( int index );
    void initControls();
    void initCheckboxes();
    void initSliders();
private:
#ifdef __WXMSW__
    wxFont *unicodeFont;
#endif
    wxWindow *parent;
    EffectsGrid *effectsGrid;
    WarpGrid *warpGrid;
    ChainBox *effectsChainBox, *finishingChainBox;
    wxTreebook *generalTree, *effectsTree;
    wxRadioBox *bitCrushTypesBox, *delayVariableLfoTypesBox, *delayPhasingLfoTypesBox,
    *downSampleTypesBox, *fadeInTypesBox, *fadeOutTypesBox, *fftFilterTypesBox,
    *masterFilterTypesBox, *masterFilterModulationTypesBox, *masterFilterLfoTypesBox,
    *tremoloTypesBox, *tremoloStereoModeBox;
    wxListView *factoryListView, *dbListView;
    wxListBox *defaultTypesBox, *warpPatternsBox;
    wxSlider *tremoloDepthSlider, *tremoloPeriodsPerSliceSlider,
    *compressCutoffSlider, *compressRangeSlider,
    *delayMultiTapDistanceSlider, *delayMultiTapFeedbackSlider, *delayMultiTapGainSlider, *delayMultiTapStepNoSlider,
    *delayVariableDistanceSlider, *delayVariableFeedbackSlider, *delayVariableGainSlider,
    *delayVariableLfoDepthSlider, *delayVariableLfoPeriodsPerSliceSlider,
    *delayPhasingDistanceOffsetSlider, *delayPhasingDistanceRangeSlider,
    *delayPhasingGainSlider, *delayPhasingLfoDepthSlider, *delayPhasingLfoPeriodsPerSliceSlider,
    *distortPregainSlider, *distortPostgainSlider, *distortCutoffSlider,
    *distortToneShaping1Slider, *distortToneShaping2Slider, *distortFeedbackSlider,
    *fadeOutRangeSlider, *fadeInRangeSlider, *pitchSemitonesSlider, *normalizeSlider,
    *fftLowPassBandHiSlider, *fftLowPassSlopeHiSlider,
    *fftHighPassBandLoSlider, *fftHighPassSlopeLoSlider,
    *fftBandPassBandLoSlider, *fftBandPassSlopeLoSlider, *fftBandPassBandHiSlider, *fftBandPassSlopeHiSlider,
    *fftBandStopBandLoSlider, *fftBandStopSlopeLoSlider, *fftBandStopBandHiSlider, *fftBandStopSlopeHiSlider,
    *loudnessBandLoSlider,*loudnessSlopeLoSlider, *loudnessBandHiSlider, *loudnessSlopeHiSlider,
    *masterFilterBandLoSlider, *masterFilterSlopeLoSlider, *masterFilterBandHiSlider, *masterFilterSlopeHiSlider,
    *masterFilterModulationDepthSlider, *masterFilterLfoDepthSlider,
    *masterFilterLfoPeriodsPerSliceSlider, *masterFilterResonanceSlider,
    *reverbRoomsizeSlider, *reverbDampSlider, *reverbDryMixSlider,
    *playbackSlider, *retriggerCountSlider, *songLengthSlider,
    *matchBpmBeatsSlider, *matchBpmTargetBpmSlider,
    *eqLowGainSlider, *eqMidGainSlider, *eqHighGainSlider;
    wxButton *delayMultitapPreset1Button, *delayMultitapPreset2Button, *delayMultitapPreset3Button, *delayMultitapPreset4Button, *delayMultitapPreset5Button,
    *delayMultitapPreset6Button, *delayVariablePreset1Button, *delayVariablePreset2Button, *delayVariablePreset3Button, *delayVariablePreset4Button,
    *showFxAllButton, *showFxNoneButton,
    *effectsChainUpButton, *effectsChainDownButton, *effectsChainDeleteButton,
    *effectsChainAddButton, *effectsChainDefaultButton,
    *effectsChainCloneButton, *finishingChainCloneButton,
    *finishingChainUpButton, *finishingChainDownButton, *finishingChainDeleteButton,
    *finishingChainAddButton, *finishingChainDefaultButton,
    *warpGridPreset1Button, *warpGridPreset2Button, *warpGridPreset3Button, *warpGridPreset4Button,
    *warpGridPreset5Button, *warpGridPreset6Button, *warpGridPreset7Button, *warpGridPreset8Button;
    wxCheckBox *compressCheckbox, *distortCheckbox, *gateCheckbox, *normalizeCheckbox, *pitchAdjustCheckbox, *loudnessCheckbox,
    *masterFilterCheckbox, *masterFilterModulationCheckbox, *masterFilterLfoCheckbox, *masterFilterResonanceCheckbox,
    *reverbCheckbox, *auxCheckbox,
    *playbackCheckbox, *songShuffleVersesCheckbox,
    *matchBpmCheckbox, *declickCheckbox, *eqCheckbox, *enhanceCheckbox,
    *delayPhasingCheckbox, *delayVariableCheckbox;
    FxPlotWindow *tremoloPlot, *lowpassPlot, *highpassPlot, *bandpassPlot, *bandstopPlot, *loudnessPlot,
    *masterFilterPlot, *masterFilterModPlot, *masterFilterLfoPlot;
    wxSize fxPlotSize, fxControlSize, fxBoxSize, generalPlotSize, generalControlSize, generalBoxSize;
    int bookWidth, effectsTreeWidthLeft, effectsTreeWidthRight, generalTreeWidthLeft, generalTreeWidthRight, effectsPlotHeight, generalPlotHeight;
    std::basic_string<double> tremoloData, lowpassData, highpassData, bandpassData, bandstopData, loudnessData,
    masterFilterData, masterFilterModData, masterFilterLfoData;
    std::map<wxString, wxString> presetMap;
    std::vector<wxString> finishingVector;
    wxString communityPresets;

    bool tremoloPlotUpdateRequired, lowpassPlotUpdateRequired, highpassPlotUpdateRequired, bandpassPlotUpdateRequired,
    bandstopPlotUpdateRequired, loudnessPlotUpdateRequired, masterFilterPlotUpdateRequired, masterFilterModPlotUpdateRequired,  masterFilterLfoPlotUpdateRequired;
    static int wxCALLBACK compareFunction (
        long item1,
        long item2,
        long sortData );
    void downloadPreset();
    static wxString getTextByColumn ( wxListView *table, long index, int col );
    wxString getRatingString ( size_t i );
    void OnButton( wxCommandEvent& event );
    void OnRadioBoxChanged ( wxCommandEvent& event );
    void OnSliderChanged ( wxScrollEvent& event );
    void OnCheckboxChanged ( wxCommandEvent& event );
    void OnToggleButtonChanged ( wxCommandEvent& event );
    void OnIdle ( wxIdleEvent& event );
    void OnTreeChanged( wxTreebookEvent& event );
    void OnBookChanged( wxAuiNotebookEvent& event );
    void OnColumnClick ( wxListEvent& event );
    void OnSize ( wxSizeEvent& event );
    void OnUpdateChainButton ( wxUpdateUIEvent& event );
    void OnListboxChanged ( wxCommandEvent& event );
    void OnListViewItemActivated ( wxListEvent& event );
    void ratePreset();
    void updateTremoloPlotArea ();
    void updateLowpassPlotArea ();
    void updateHighpassPlotArea ();
    void updateBandpassPlotArea ();
    void updateBandstopPlotArea ();
    void updateLoudnessPlotArea ();
    void updateMasterFilterPlotArea();
    void updateMasterFilterModPlotArea();
    void updateMasterFilterLfoPlotArea();

    DECLARE_EVENT_TABLE()
};

#endif

