#include "sharedialog.h"
#include <wx/stattext.h>
#include <wx/button.h>
#include <wx/textctrl.h>
#include <wx/sizer.h>
#include <wx/msgdlg.h>
#include <wx/url.h>
#include <wx/utils.h>
#include <wx/filename.h>
#include "urlprovider.h"
#include "smasherdb.h"
#include "httpresource.h"

BEGIN_EVENT_TABLE ( ShareDialog, wxDialog )
    EVT_BUTTON ( ID_REGISTER, ShareDialog::OnRegister )
    EVT_BUTTON ( wxID_OK, ShareDialog::OnOk )
END_EVENT_TABLE()

ShareDialog::ShareDialog (
    wxWindow *parent,
    const wxString& title,
    wxString& presetParameter,
    wxString& authorParameter,
    wxString& passcodeParameter,
    wxString& emailParameter,
    const wxString& sampleParameter,
    const wxString& stateParameter
) : wxDialog ( parent, wxID_ANY, title ), sample ( sampleParameter ), state ( stateParameter ), preset ( presetParameter ), author ( authorParameter ), passcode ( passcodeParameter ), email ( emailParameter )
{
    titleLabel = new wxStaticText ( this, wxID_ANY, _T("preset") );
    titleEdit = new wxTextCtrl ( this, wxID_ANY, preset, wxDefaultPosition, wxSize ( 400, -1 ) );
    authorLabel = new wxStaticText ( this, wxID_ANY, _T("user") );
    authorEdit = new wxTextCtrl ( this, wxID_ANY, author, wxDefaultPosition, wxSize ( 400, -1 ) );
    passcodeLabel = new wxStaticText ( this, wxID_ANY, _T("passcode") );
    passcodeEdit = new wxTextCtrl ( this, wxID_ANY, passcode, wxDefaultPosition, wxSize ( 400, -1 ) );
    registerLabel = new wxStaticText ( this, wxID_ANY, _T("email") );
    registerEdit = new wxTextCtrl ( this, wxID_ANY, email, wxDefaultPosition, wxSize ( 240, -1 ) );

    presetPtr = &presetParameter;
    authorPtr = &authorParameter;
    passcodePtr = &passcodeParameter;
    emailPtr = &emailParameter;

    int width, height;
    registerEdit->GetSize ( &width, &height );
    registerButton = new wxButton ( this, ID_REGISTER, _T("request passcode"), wxDefaultPosition, wxSize ( 150, height ) );

    wxBoxSizer *embeddedSizer = new wxBoxSizer ( wxHORIZONTAL );
    embeddedSizer->Add ( registerEdit );
    embeddedSizer->Add ( registerButton );

    wxButton *cancelButton = new wxButton ( this, wxID_CANCEL, _T("&cancel") );
    wxButton *shareButton = new wxButton ( this, wxID_OK, _T("&share preset") );
    wxBoxSizer *buttonSizer = new wxBoxSizer ( wxHORIZONTAL );
    buttonSizer->Add ( cancelButton );
    buttonSizer->Add ( shareButton );
    //wxSizer *buttonSizer = CreateButtonSizer ( wxID_OK | wxID_CANCEL );

    wxStaticBoxSizer *passcodeSizer = new wxStaticBoxSizer ( wxVERTICAL, this, _T("new user") );
    passcodeSizer->Add ( registerLabel );
    passcodeSizer->Add ( embeddedSizer );

    wxBoxSizer *sizer = new wxBoxSizer ( wxVERTICAL );
    sizer->AddSpacer ( 5 );
    sizer->Add ( titleLabel, 0, wxRIGHT|wxLEFT, 5 );
    sizer->Add ( titleEdit, 0, wxRIGHT|wxLEFT, 5 );
    sizer->AddSpacer ( 5 );
    sizer->Add ( authorLabel, 0, wxRIGHT|wxLEFT, 5 );
    sizer->Add ( authorEdit, 0, wxRIGHT|wxLEFT, 5 );
    sizer->AddSpacer ( 5 );
    sizer->Add ( passcodeLabel, 0, wxRIGHT|wxLEFT, 5 );
    sizer->Add ( passcodeEdit, 0, wxRIGHT|wxLEFT, 5 );
    sizer->AddSpacer ( 5 );
    sizer->Add ( passcodeSizer, 0, wxRIGHT|wxLEFT, 5 );
    sizer->AddSpacer ( 5 );

    if ( buttonSizer )
        sizer->Add ( buttonSizer, 0, wxALIGN_RIGHT | wxRIGHT, 0 );

    SetSizer ( sizer );
    sizer->Fit ( this );
    titleEdit->SetFocus();
}

ShareDialog::~ShareDialog()
{ }

void ShareDialog::OnOk ( wxCommandEvent& e )
{
    preset = titleEdit->GetValue();
    author = authorEdit->GetValue();
    passcode = passcodeEdit->GetValue();

    if ( preset.empty() || author.empty() || passcode.empty() )
    {
        wxMessageBox ( _("please fill in the preset, user and passcode fields"),
                       _("can't share preset"),
                       wxOK,
                       GetParent() );
        return;
    }

    if ( preset.size() > 32 || author.size() > 32 || passcode.size() > 32 )
    {
        wxMessageBox (
            _("the preset, user and passcode fields should be no longer than 32 characters"),
            _("can't share preset"),
            wxOK,
            GetParent() );
        return;
    }

    *presetPtr = preset;
    *authorPtr = author;
    *passcodePtr = passcode;

    wxFileName fn ( sample );
    wxString sampleFileName = fn.GetFullName();

    wxString message;
    if ( !SmasherDb::sharePreset (
                preset,
                sampleFileName,
                state,
                author,
                passcode,
                message ) )
    {
        wxMessageBox (
            message,
            _("can't share preset"),
            wxOK,
            GetParent() );
        return;
    }

    e.Skip();
}

void ShareDialog::OnRegister ( wxCommandEvent& e )
{
    author = authorEdit->GetValue();
    email = registerEdit->GetValue();

    if ( author.empty() )
    {
        wxMessageBox (
            _("user name required"),
            _("can't request passcode"),
            wxOK,
            GetParent() );
        return;
    }

    if (
        email.size() < 3 ||
        email.Find ( _("@") ) == wxNOT_FOUND )
    {
        wxMessageBox (
            _("valid email address required"),
            _("can't request passcode"),
            wxOK,
            GetParent() );
        return;
    }


    *emailPtr = email;

    UrlProvider up;

    // check user's access
    std::string baseRegisterUrl = up.getRegisterUrl();
    wxString wideRegisterUrl = wxString ( baseRegisterUrl.c_str(), wxConvUTF8,    baseRegisterUrl.size() );

    wideRegisterUrl += _T("?email=") + email + _T("&user=") + author;

    wxString buffer, message;
    HttpResource hr;
    if ( !hr.load ( wideRegisterUrl, &buffer ) )
    {
        message = hr.getError();
        return;
    }

    if ( buffer.size() > 0 && buffer[0] == L'!' )
    {
        wxMessageBox (
            _("user name already in use"),
            _("can't request passcode"),
            wxOK,
            GetParent() );
        return;
    }

    // trigger email
    std::string baseUrl = up.getPasscodeUrl();
    wxString wideUrl = wxString ( baseUrl.c_str(), wxConvUTF8, baseUrl.size() );

    wideUrl += _T("?email=") + email + _T("&user=") + author;

    wxURL url ( wideUrl );

    if ( url.GetError() != wxURL_NOERR )
    {
        wxMessageBox ( _("can't connect to server"),
                       _T("can't share preset"),
                       wxOK,
                       GetParent()  );

        return;
    }

    wxYield();
    url.GetProtocol().SetTimeout ( 5 );

    wxInputStream *inStream;

    inStream = url.GetInputStream();

    if ( !inStream )
        return;

    wxSleep ( 1 );

}

