#ifndef SLICEAUXTRANSFORMER_H
#define SLICEAUXTRANSFORMER_H

#include <wx/string.h>
#include "sndfile.hh"

#include "slicetransformer.h"
#include "slicegatetransformer.h"
#include "wrapcsound.h"
#include "wraptempfilename.h"

template<class T>
class SliceAuxTransformer : public SliceTransformer<T>
{
public:
    SliceAuxTransformer( const wxString& csdPath, size_t channels );
    void transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel );
private:
    wxString csdPath;
    size_t channels;
};

template<class T>
SliceAuxTransformer<T>::SliceAuxTransformer(
    const wxString& csdPathParameter,
    size_t channelsParameter ) :
        csdPath ( csdPathParameter), channels ( channelsParameter )
{
}

template<class T>
void SliceAuxTransformer<T>::transform (
    std::vector<T> *leftChannel,
    std::vector<T> *rightChannel )
{
    if ( channels != 2 )
        return;

    csdPath.Replace (_T(" "), _T("__SP__") );
    wxString cmd =
#ifndef __WXMSW__
        _T("csound ");
#else
        _T("csound.exe ");
#endif

    cmd += _T("-d -g ") + csdPath; // removed -v [verbose]

    WrapCSound *wcs = new WrapCSound ( cmd );
    if ( !wcs->perform( leftChannel, rightChannel ) )
    {
        std::cout << "!SliceAuxTransformer::transform()" << std::endl;
        wcs->printMessages();
        return;
    }
    delete wcs;
}

#endif
