#ifndef SLICEBITCRUSHTRANSFORMER_H
#define SLICEBITCRUSHTRANSFORMER_H

#define CRUSH_4BIT_MAX 15.0
#define CRUSH_6BIT_MAX 63.0
#define CRUSH_8BIT_MAX 255.0
#define CRUSH_10BIT_MAX 1023.0

#include "enum.h"
#include "samplemath.h"
#include "slicetransformer.h"

template<class T>
class SliceBitCrushTransformer : public SliceTransformer<T>
{
public:
    SliceBitCrushTransformer ( int bitCrushType, size_t channels = 2 );
    void transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel );
    void transformChannel ( std::vector<T> *channel );
private:
    int bitCrushType;
    double bitCrushMax;
    size_t channels;
};

template<class T>
SliceBitCrushTransformer<T>::SliceBitCrushTransformer ( int bitCrushTypeParameter, size_t channelsParameter )
        : bitCrushType ( bitCrushTypeParameter ), channels ( channelsParameter )
{
    switch ( bitCrushType )
    {
    case BITCRUSH_4BIT :
        bitCrushMax = CRUSH_4BIT_MAX;
        break;
    case BITCRUSH_6BIT :
        bitCrushMax = CRUSH_6BIT_MAX;
        break;
    case BITCRUSH_8BIT :
        bitCrushMax = CRUSH_8BIT_MAX;
        break;
    case BITCRUSH_10BIT :
        bitCrushMax = CRUSH_10BIT_MAX;
        break;
    default:
        break;
    }
}

template<class T>
void SliceBitCrushTransformer<T>::transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel )
{
    transformChannel ( leftChannel );
    if ( rightChannel )
        transformChannel ( rightChannel );
}

template<class T>
void SliceBitCrushTransformer<T>::transformChannel ( std::vector<T> *channel )
{
    size_t channelSize = channel->size();
    for ( size_t i = 0; i < channelSize; i++ )
        (*channel) [ i ] = SampleMath::crushBits ( (*channel).at ( i ), SampleMath::shortToDouble ( SHRT_MAX ), bitCrushMax );
}

#endif

