#ifndef SLICEBPMTRANSFORMER_H
#define SLICEBPMTRANSFORMER_H

#include <string>
#include <cmath>
#include <iostream>

#include "functor.h"
#include "samplemath.h"
#include "slicetransformer.h"


template<class T>
class SliceBpmTransformer : public SliceTransformer<T>
{
public:
    SliceBpmTransformer (
        size_t beats,
        size_t targetBpm,
        size_t channels = 2 );
    void transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel );
    void transformChannel ( std::vector<T> *channel );
    void dropSamples ( std::vector<T> *channel, size_t step );
    void addSamples ( std::vector<T> *channel, size_t step );
private:
    size_t beats, targetBpm, channels, channelOutLen;
};

template<class T>
SliceBpmTransformer<T>::SliceBpmTransformer (
    size_t beatsParameter,
    size_t targetBpmParameter,
    size_t channelsParameter ):
        beats ( beatsParameter ),
        targetBpm ( targetBpmParameter ),
        channels ( channelsParameter )
{
    channelOutLen = ( size_t ) ( ( 1.0 * beats ) * ( 60.0 * 44100.0 / ( 1.0 * targetBpm )) );//tbd samplerate
}

template<class T>
void SliceBpmTransformer<T>::transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel )
{
    transformChannel ( leftChannel );
    if ( rightChannel )
        transformChannel ( rightChannel );
    SliceGateTransformer<T> sgt ( channelOutLen );
    sgt.transform ( leftChannel, rightChannel );
}


template<class T>
void SliceBpmTransformer<T>::transformChannel ( std::vector<T> *channel )
{
    size_t channelSize, step;
    channelSize = channel->size();

    if ( channelSize == channelOutLen )
        return;

    else if ( channelSize > channelOutLen )
    {
        step = 1 + channelSize / ( channelSize - channelOutLen );
        step = ( step < 2 ) ? 2 : step;
        dropSamples ( channel, step );
    }
    else if  ( channelOutLen > channelSize )
    {
        step = 1 + channelSize / ( channelOutLen - channelSize );
        step = ( step < 2 ) ? 2 : step;
        addSamples ( channel, step );
    }
}

template<class T>
void SliceBpmTransformer<T>::dropSamples ( std::vector<T> *channel, size_t step )
{
    size_t channelSize = channel->size();
    std::vector<T> result;
    for ( size_t i = 0; i < channelSize; i++ )
    {
        if ( ( i % step ) )
            result.push_back ( ( *channel ) [ i ] );
    }
    *channel = result;
}

template<class T>
void SliceBpmTransformer<T>::addSamples ( std::vector<T> *channel, size_t step )
{
    size_t channelSize = channel->size();
    std::vector<T> result;
    for ( size_t i = 0; i < channelSize - 1; i++ )
    {
        result.push_back ( ( *channel ) [ i ] );
        if ( ! ( i % step ) )
        {
            result.push_back ( 0.5 * ( ( *channel ) [ i ] + ( *channel ) [ i + 1 ]) );
        }
    }
    result.push_back ( (* channel ) [ channelSize - 1 ] );
    *channel = result;
}

#endif
