#ifndef SLICEDECLICKTRANSFORMER_H
#define SLICEDECLICKTRANSFORMER_H

#include <cmath>

#include "dir.h"
#include "slicetransformer.h"
#include "wrapcsound.h"

template<class T>
class SliceDeclickTransformer : public SliceTransformer<T>
{
public:
    SliceDeclickTransformer (
        size_t channels = 2 );
    void transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel );
    void transformChannel ( std::vector<T> *channel );
private:
    size_t channels;
};

template<class T>
SliceDeclickTransformer<T>::SliceDeclickTransformer (
    size_t channelsParameter ) :
        channels ( channelsParameter )
{
}


template<class T>
void SliceDeclickTransformer<T>::transform (
    std::vector<T> *leftChannel,
    std::vector<T> *rightChannel )
{
    if ( channels == 1 )
        return;

    wxString cmd =
#ifndef __WXMSW__
        _T("csound ");
#else
        _T("csound.exe ");
#endif
    wxString dir = Dir::getCsdDir();
    dir.Replace ( _T(" "), _T("__SP__") );
    cmd += _T("-odac -idac -b 4096 -d -g ") + dir; // realtime
    cmd += _T("declick.csd");

    WrapCSound *wcs = new WrapCSound ( cmd );

    if ( !wcs->perform( leftChannel, rightChannel ) )
    {
        std::cout << "!WrapCSound::perform" << std::endl;
        wcs->printMessages();
        return;
    }

    delete wcs;

}

#endif

