#ifndef SLICEDELAYPANTRANSFORMER_H
#define SLICEDELAYPANTRANSFORMER_H

#include <cmath>

#include "dir.h"
#include "slicetransformer.h"
#include "wrapcsound.h"

template<class T>
class SliceDelayPanTransformer : public SliceTransformer<T>
{
public:
    SliceDelayPanTransformer (
        double delayLeft,
        double delayRight,
        size_t channels = 2 );
    void transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel );
    void transformChannel ( std::vector<T> *channel );
private:
    double delayLeft, delayRight;
    size_t channels;
};

template<class T>
SliceDelayPanTransformer<T>::SliceDelayPanTransformer (
    double delayLeftParameter,
    double delayRightParameter,
    size_t channelsParameter ) :
        delayLeft ( delayLeftParameter ),
        delayRight ( delayRightParameter ),
        channels ( channelsParameter )
{
}

template<class T>
void SliceDelayPanTransformer<T>::transform (
    std::vector<T> *leftChannel,
    std::vector<T> *rightChannel )
{
    if ( channels == 1 )
        return;

    wxString cmd =
#ifndef __WXMSW__
        _T("csound ");
#else
        _T("csound.exe ");
#endif
    wxString dir = Dir::getCsdDir();
    dir.Replace ( _T(" "), _T("__SP__") );
    cmd += _T("-odac -idac -b 4096 -d -g ") + dir; // realtime
    cmd += _T("pandelay.csd");

    //std::cout << cmd.mb_str ( wxConvLocal ) << std::endl;

    WrapCSound *wcs = new WrapCSound ( cmd );

    wcs->setDoubleParameter ( "pandelay_dch1", delayLeft );
    wcs->setDoubleParameter ( "pandelay_dch2", delayRight );
    if ( !wcs->perform( leftChannel, rightChannel ) )
    {
        std::cout << "!WrapCSound::perform" << std::endl;
        wcs->printMessages();
        return;
    }

    delete wcs;

}

#endif

