#ifndef SLICEDISTORTTRANSFORMER_H
#define SLICEDISTORTTRANSFORMER_H

#include <cmath>

#include "dir.h"
#include "slicetransformer.h"
#include "wrapcsound.h"

template<class T>
class SliceDistortTransformer : public SliceTransformer<T>
{
public:
    SliceDistortTransformer (
        double pregain = 0.75,
        double postgain = 1.0,
        size_t channels = 2 );
    void transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel );
    void transformChannel ( std::vector<T> *channel );
private:
    double pregain, postgain;
    size_t channels;
};

template<class T>
SliceDistortTransformer<T>::SliceDistortTransformer (
    double pregainParameter,
    double postgainParameter,
    size_t channelsParameter ) :
        pregain ( pregainParameter ),
        postgain ( postgainParameter ),
        channels ( channelsParameter )
{
}

template<class T>
void SliceDistortTransformer<T>::transform (
    std::vector<T> *leftChannel,
    std::vector<T> *rightChannel )
{
    if ( channels == 1 )
        return;

    wxString cmd =
#ifndef __WXMSW__
        _T("csound ");
#else
        _T("csound.exe ");
#endif
    wxString dir = Dir::getCsdDir();
    dir.Replace ( _T(" "), _T("__SP__") );
    cmd += _T("-odac -idac -b 4096 -d -g ") + dir; // realtime
    cmd += _T("distortion_disk.csd");
    //std::cout << cmd.mb_str ( wxConvLocal ) << std::endl;

    WrapCSound *wcs = new WrapCSound ( cmd );

    wcs->setDoubleParameter ( "distort_pregain", pregain );
    wcs->setDoubleParameter ( "distort_postgain", postgain );

    if ( !wcs->perform( leftChannel, rightChannel ) )
    {
        std::cout << "!WrapCSound::perform" << std::endl;
        wcs->printMessages();
        return;
    }

    delete wcs;

}

#endif

