#ifndef SLICEDISTORTTRANSFORMER_H
#define SLICEDISTORTTRANSFORMER_H

#include <cmath>

#include "dir.h"
#include "samplemath.h"
#include "slicetransformer.h"
#include "slicenormalizetransformer.h"
#include "sliceffttransformer.h"

//eal
#include <iostream>


template<class T>
class SliceDistortTransformer : public SliceTransformer<T>
{
public:
    SliceDistortTransformer (
        double pregain = 2.0,
        double postgain = 1.0,
        double cutoff = 5000,
        size_t feedback = 0,
        double toneShaping1 = 1.0,
        double toneShaping2 = 0.0,
        size_t channels = 2 );
    void transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel );
    void transformChannel ( std::vector<T> *channel );
private:
    double pregain, postgain, cutoff;
    size_t feedback;
    double toneShaping1, toneShaping2;
    size_t channels;
};

template<class T>
SliceDistortTransformer<T>::SliceDistortTransformer (
    double pregainParameter,
    double postgainParameter,
    double cutoffParameter,
    size_t feedbackParameter,
    double toneShapingParameter1,
    double toneShapingParameter2,
    size_t channelsParameter ) :
        pregain ( pregainParameter ),
        postgain ( postgainParameter ),
        cutoff ( cutoffParameter ) ,
        feedback ( feedbackParameter ),
        toneShaping1 ( toneShapingParameter1 ),
        toneShaping2 ( toneShapingParameter2 ),
        channels ( channelsParameter )
{
}

template<class T>
void SliceDistortTransformer<T>::transform (
    std::vector<T> *leftChannel,
    std::vector<T> *rightChannel )
{

    for ( size_t i = 0; i < feedback; i++ )
    {
        SliceNormalizeTransformer<T> snt;
        snt.transform ( leftChannel, rightChannel );

        SliceFFTTransformer<T> lpf (
            FFT_LOWPASS,
            0.0,
            0.0,
            toneShaping1,
            toneShaping2
        );
        lpf.transform ( leftChannel, rightChannel );
        transformChannel ( leftChannel );
        if ( rightChannel )
            transformChannel ( rightChannel );
    }
}


template<class T>
void SliceDistortTransformer<T>::transformChannel ( std::vector<T> *channel )
{
    size_t channelSize = channel->size();
    double cPregainSample, cPostgainSample, postgainedCutoff;

    for ( size_t i = 0; i < channelSize; i++ )
    {

        cPregainSample =
            pregain * ( channel->at ( i ) + 3.0 );

        if ( ( cPregainSample > - cutoff ) && ( cPregainSample < cutoff ) )
        {
            cPregainSample *= 1.0 / cutoff;
            cPostgainSample = postgain * cutoff * ( cPregainSample - ( pow ( cPregainSample, 3 ) / 3.0 ) );
            ( *channel ) [ i ] = ( cPostgainSample > DOUBLE_SHRT_MAX ) ?
                                 DOUBLE_SHRT_MAX :
                                 ( ( cPostgainSample < -DOUBLE_SHRT_MAX ) ?
                                   -DOUBLE_SHRT_MAX :
                                   cPostgainSample );
        }
        else
        {
            postgainedCutoff = postgain * cutoff * 2.0 / 3.0;
            if ( cPregainSample >= cutoff )
            {
                ( *channel ) [ i ] = ( postgainedCutoff < DOUBLE_SHRT_MAX ) ? postgainedCutoff : DOUBLE_SHRT_MAX;
            }
            if ( cPregainSample <= - cutoff )
            {
                ( *channel ) [ i ] = ( postgainedCutoff > -DOUBLE_SHRT_MAX ) ? -postgainedCutoff : -DOUBLE_SHRT_MAX;
            }
        }
        //(*channel) [ i ] = currentSample * postgain * tanh ( pregain * currentSample );
    }
}

#endif

