#ifndef SLICEDOWNSAMPLETRANSFORMER_H
#define SLICEDOWNSAMPLETRANSFORMER_H

#include <iostream>

#include "enum.h"
#include "samplemath.h"
#include "slicetransformer.h"


template<class T> class SliceTransformer;

template<class T>
class SliceDownSampleTransformer : public SliceTransformer<T>
{
public:
    SliceDownSampleTransformer ( int downSampleType, size_t channels = 2 );
    void transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel );
    void transformChannel ( std::vector<T>& channel );
private:
    int downSampleType;
    size_t downSampleFactor, channels;
};

template<class T>
SliceDownSampleTransformer<T>::SliceDownSampleTransformer ( int downSampleTypeParameter, size_t channelsParameter )
        : downSampleType ( downSampleTypeParameter ), channels ( channelsParameter)
{
    switch ( downSampleType )
    {
    case DOWNSAMPLE_1 :
        downSampleFactor = 1;
        break;
    case DOWNSAMPLE_2 :
        downSampleFactor = 2;
        break;
    case DOWNSAMPLE_4 :
        downSampleFactor = 4;
        break;
    case DOWNSAMPLE_8 :
        downSampleFactor = 8;
        break;
    case DOWNSAMPLE_16 :
        downSampleFactor = 16;
        break;
    case DOWNSAMPLE_32 :
        downSampleFactor = 32;
        break;
    default:
        break;
    }
}

template<class T>
void SliceDownSampleTransformer<T>::transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel )
{
    transformChannel ( *leftChannel );
    if ( rightChannel )
        transformChannel ( *rightChannel );
}

template<class T>
void SliceDownSampleTransformer<T>::transformChannel ( std::vector<T>& channel )
{
    size_t channelSize = channel.size();
    std::vector<T> result;

    double newValue;
    size_t j;
    for ( size_t i = 0; i < channelSize - downSampleFactor; i += downSampleFactor )
    {
        newValue = 0.0;
        for ( j = 0; j < downSampleFactor ; j++ )
            newValue += channel[ i + j ];

        newValue /= downSampleFactor;

        for ( j = 0; j < downSampleFactor ; j++)
            result.push_back( newValue );
    }

    channel = result;
}

#endif


