#ifndef SLICEENHANCETRANSFORMER_H
#define SLICEENHANCETRANSFORMER_H

#include <cmath>

#include "dir.h"
#include "slicetransformer.h"
#include "wrapcsound.h"

template<class T>
class SliceEnhanceTransformer : public SliceTransformer<T>
{
public:
    SliceEnhanceTransformer (
        size_t channels = 2 );
    void transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel );
    void transformChannel ( std::vector<T> *channel );
private:
    size_t channels;
};

template<class T>
SliceEnhanceTransformer<T>::SliceEnhanceTransformer (
    size_t channelsParameter ) :
        channels ( channelsParameter )
{
}


template<class T>
void SliceEnhanceTransformer<T>::transform (
    std::vector<T> *leftChannel,
    std::vector<T> *rightChannel )
{
    if ( channels == 1 )
        return;

    wxString cmd =
#ifndef __WXMSW__
        _T("csound ");
#else
        _T("csound.exe ");
#endif
    wxString dir = Dir::getCsdDir();
    dir.Replace ( _T(" "), _T("__SP__") );
    cmd += _T("-odac -idac -b 4096 -d -g ") + dir; // realtime
    cmd += _T("enhance.csd");

    WrapCSound *wcs = new WrapCSound ( cmd );

    if ( !wcs->perform( leftChannel, rightChannel ) )
    {
        std::cout << "!WrapCSound::perform" << std::endl;
        wcs->printMessages();
        return;
    }

    delete wcs;

}

#endif
