#ifndef SLICEEQTRANSFORMER_H
#define SLICEEQTRANSFORMER_H

#include <cmath>

#include "dir.h"
#include "slicetransformer.h"
#include "wrapcsound.h"

template<class T>
class SliceEqTransformer : public SliceTransformer<T>
{
public:
    SliceEqTransformer (
        double lowGain = 0.5,
        double midGain = -0.3,
        double highGain = 0.2,
        double cutoffLow = 500,
        double cutoffHigh = 5000,
        size_t channels = 2 );
    void transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel );
    void transformChannel ( std::vector<T> *channel );
private:
    double lowGain, midGain, highGain, cutoffLow, cutoffHigh;
    size_t channels;
};

template<class T>
SliceEqTransformer<T>::SliceEqTransformer (
    double lowGainParameter,
    double midGainParameter,
    double highGainParameter,
    double cutofLowParameter,
    double cutoffHighParameter,
    size_t channelsParameter ) :
        lowGain ( lowGainParameter ),
        midGain ( midGainParameter ),
        highGain ( highGainParameter ),
        cutoffLow ( cutofLowParameter ),
        cutoffHigh ( cutoffHighParameter ),
        channels ( channelsParameter )
{
}


template<class T>
void SliceEqTransformer<T>::transform (
    std::vector<T> *leftChannel,
    std::vector<T> *rightChannel )
{
    if ( channels == 1 )
        return;

    wxString cmd =
#ifndef __WXMSW__
        _T("csound ");
#else
        _T("csound.exe ");
#endif
    wxString dir = Dir::getCsdDir();
    dir.Replace ( _T(" "), _T("__SP__") );
    cmd += _T("-odac -idac -b 4096 -d -g ") + dir; // realtime
    cmd += _T("eq.csd");

    //std::cout << cmd.mb_str ( wxConvLocal ) << std::endl;

    WrapCSound *wcs = new WrapCSound ( cmd );

    wcs->setDoubleParameter ( "eq_lowgain", lowGain );
    wcs->setDoubleParameter ( "eq_highgain", highGain );
    wcs->setDoubleParameter ( "eq_midgain", midGain );
    wcs->setDoubleParameter ( "eq_cutofflow", cutoffLow );
    wcs->setDoubleParameter ( "eq_cutoffhigh", cutoffHigh );

    if ( !wcs->perform( leftChannel, rightChannel ) )
    {
        std::cout << "!WrapCSound::perform" << std::endl;
        wcs->printMessages();
        return;
    }

    delete wcs;

}

#endif

