#ifndef SLICEFADETRANSFORMER_H
#define SLICEFADETRANSFORMER_H

#include <algorithm>
#include <cmath>
#include <string>

#include "enum.h"
#include "samplemath.h"
#include "slicetransformer.h"


template<class T>
class SliceFadeTransformer : public SliceTransformer<T>
{
public:
    SliceFadeTransformer (
        size_t fadeRange,
        size_t channels = 2,
        int fadeDirection = FADE_OUT,
        int fadeType = FADE_LOG5 );
    void transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel );
    void transformChannel ( std::vector<T> *channel );
private:
    size_t fadeRange, channels;
    int fadeDirection, fadeType;
    double logBase;
};

template<class T>
SliceFadeTransformer<T>::SliceFadeTransformer (
    size_t fadeRangeParameter,
    size_t channelsParameter,
    int fadeDirectionParameter,
    int fadeTypeParameter ) :
        fadeRange ( fadeRangeParameter ),
        channels ( channelsParameter ),
        fadeDirection ( fadeDirectionParameter ),
        fadeType ( fadeTypeParameter )
{
    switch (fadeType)
    {
    case FADE_LOG5:
        logBase = 5.0;
        break;
    case FADE_LOG3:
        logBase = 3.0;
        break;
    case FADE_LOG1p5:
        logBase = 1.5;
        break;
    case FADE_LOG10:
    default:
        logBase = 10.0;
        break;
    }
}

template<class T>
void SliceFadeTransformer<T>::transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel )
{
    transformChannel ( leftChannel );
    if ( rightChannel )
        transformChannel ( rightChannel );
}

template<class T>
void SliceFadeTransformer<T>::transformChannel ( std::vector<T> *channel )
{
    size_t channelSize = channel->size();
    double logFadeParameter, logFadeFactor;

    if ( fadeDirection == FADE_OUT )
        std::reverse( channel->begin(), channel->end() );

    size_t limit = ( fadeRange < channelSize ) ? fadeRange : channelSize;
    for ( size_t i = 0; i < limit; i++ )
    {
        logFadeParameter =  i * ( ( logBase - 1 ) / fadeRange );
        logFadeFactor = log10 ( 1 + logFadeParameter ) / log10 ( logBase );
        (*channel) [ i ] = ( logFadeFactor * channel->at ( i ) );
    }
    if ( fadeDirection == FADE_OUT )
        std::reverse( channel->begin(), channel->end() );

}

#endif
