#ifndef SLICEGATETRANSFORMER_H
#define SLICEGATETRANSFORMER_H

#include <string>
#include <cmath>
#include <iostream>

#include "functor.h"
#include "samplemath.h"
#include "slicetransformer.h"


template<class T>
class SliceGateTransformer : public SliceTransformer<T>
{
public:
    SliceGateTransformer (
        size_t dataInLen,
        size_t gateRange = 64,
        size_t channels = 2 );
    void transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel );
    void transformChannel ( std::vector<T> *channel );
private:
    size_t dataInLen, gateRange, channels;
};

template<class T>
SliceGateTransformer<T>::SliceGateTransformer (
    size_t dataInLenParameter,
    size_t gateRangeParameter,
    size_t channelsParameter ):
        dataInLen ( dataInLenParameter ),
        gateRange ( gateRangeParameter ),
        channels ( channelsParameter )
{}

template<class T>
void SliceGateTransformer<T>::transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel )
{
    transformChannel ( leftChannel );
    if ( rightChannel )
        transformChannel ( rightChannel );
}

template<class T>
void SliceGateTransformer<T>::transformChannel ( std::vector<T> *channel )
{
    size_t channelSize = channel->size();
    size_t channelInLen = ( int ) ( dataInLen / channels ) ;
    size_t unchanged;
    int gate = ( int ) ( channelSize - channelInLen );

    if ( gate < 0 )
        return;

    size_t i = 0;

    while ( i < gateRange )
    {
        (*channel)[ i ] = lConcaveSoftClipper ( i, 0, gateRange ) * channel->at ( i );
        i++;
    }

    unchanged = channelInLen - gateRange;
    while ( i < unchanged )
    {
        i++;
    }

    while ( i < channelInLen )
    {
        (*channel) [ i ] = rConvexSoftClipper ( i, unchanged, channelInLen ) * channel->at ( i );
        i++;
    }

}

#endif
