#ifndef SLICENORMALIZETRANSFORMER_H
#define SLICENORMALIZETRANSFORMER_H

#include <string>
#include <cmath>
#include <iostream>

#include "samplemath.h"
#include "slicetransformer.h"

template<class T>
class SliceNormalizeTransformer : public SliceTransformer<T>
{
public:
    SliceNormalizeTransformer ( double normalizeTo = 1.0, size_t channels = 2 );
    void transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel );
    void transformChannel ( std::vector<T> *channel, double peakLevel );
private:
    double normalizeTo;
    size_t channels;
};

template<class T>
SliceNormalizeTransformer<T>::SliceNormalizeTransformer ( double normalizeToParameter, size_t channelsParameter ) :
        normalizeTo ( normalizeToParameter ), channels ( channelsParameter )
{
    if ( normalizeTo > 1.0 ||  normalizeTo < 0.0 )
        throw;
}

template<class T>
void SliceNormalizeTransformer<T>::transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel )
{
    double peakLevel = SampleMath::getPeakLevel ( leftChannel, rightChannel );
    if ( peakLevel == 0.0 )
        return;

    transformChannel ( leftChannel, peakLevel );
    if ( rightChannel )
        transformChannel ( rightChannel, peakLevel );

}

template<class T>
void SliceNormalizeTransformer<T>::transformChannel ( std::vector<T> *channel, double peakLevel )
{
    size_t channelSize = channel->size();

    double normalizeFactor;
    normalizeFactor = normalizeTo * ((double) SHRT_MAX ) / peakLevel;

    for ( size_t i = 0; i < channelSize; i++ )
        (*channel) [ i ] = normalizeFactor * channel->at ( i );

}

#endif
