#ifndef SLICEPITCHTRANSFORMER_H
#define SLICEPITCHTRANSFORMER_H

#include <vector>
#include <algorithm>

#include "slicetransformer.h"

bool transpose12Array[] = { 1, 0 };
bool transpose11Array[] = { 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1 };
bool transpose10Array[] = { 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1 };
bool transpose9Array[]  = { 1, 0, 1, 0, 1, 0, 1 };
bool transpose8Array[]  = { 1, 0, 1, 0, 1 };
bool transpose7Array[]  = { 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0 };
bool transpose6Array[]  = { 1, 1, 0 };
bool transpose5Array[]  = { 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1 };
bool transpose4Array[]  = { 1, 1, 1, 0 };
bool transpose3Array[]  = { 1 ,1, 1, 1, 0 };
bool transpose2Array[]  = { 1, 1, 1, 1, 1, 1, 0 };
bool transpose1Array[]  = { 1, 1, 1, 1 , 1, 1, 1, 1, 1, 1, 1, 1, 0 };
bool transpose0Array[]  = { 1 };

template<class T>
class SlicePitchTransformer : public SliceTransformer<T>
{
public:
    SlicePitchTransformer ( int pitchSemitones = 0, size_t channels = 2 );
    virtual ~SlicePitchTransformer() { }
    void transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel );
    void transformChannel( std::vector<T>& channel );
private:
    std::vector< std::pair<bool *, size_t> > transposeArrayVector;
    int pitchSemitones;
    size_t channels;
};

template<class T>
SlicePitchTransformer<T>::SlicePitchTransformer( int pitchSemitonesParameter, size_t channelsParameter ) :
        pitchSemitones ( pitchSemitonesParameter ), channels ( channelsParameter )
{
    transposeArrayVector.push_back ( std::make_pair ( transpose0Array, sizeof(transpose0Array)/sizeof(bool) ) );
    transposeArrayVector.push_back ( std::make_pair ( transpose1Array, sizeof(transpose1Array)/sizeof(bool) ) );
    transposeArrayVector.push_back ( std::make_pair ( transpose2Array, sizeof(transpose2Array)/sizeof(bool) ) );
    transposeArrayVector.push_back ( std::make_pair ( transpose3Array, sizeof(transpose3Array)/sizeof(bool) ) );
    transposeArrayVector.push_back ( std::make_pair ( transpose4Array, sizeof(transpose4Array)/sizeof(bool) ) );
    transposeArrayVector.push_back ( std::make_pair ( transpose5Array, sizeof(transpose5Array)/sizeof(bool) ) );
    transposeArrayVector.push_back ( std::make_pair ( transpose6Array, sizeof(transpose6Array)/sizeof(bool) ) );
    transposeArrayVector.push_back ( std::make_pair ( transpose7Array, sizeof(transpose7Array)/sizeof(bool) ) );
    transposeArrayVector.push_back ( std::make_pair ( transpose8Array, sizeof(transpose8Array)/sizeof(bool) ) );
    transposeArrayVector.push_back ( std::make_pair ( transpose9Array, sizeof(transpose9Array)/sizeof(bool) ) );
    transposeArrayVector.push_back ( std::make_pair ( transpose10Array, sizeof(transpose10Array)/sizeof(bool) ) );
    transposeArrayVector.push_back ( std::make_pair ( transpose11Array, sizeof(transpose11Array)/sizeof(bool) ) );
    transposeArrayVector.push_back ( std::make_pair ( transpose12Array, sizeof(transpose12Array)/sizeof(bool) ) );
}

template<class T>
void SlicePitchTransformer<T>::transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel )
{
    transformChannel ( *leftChannel );
    if ( rightChannel )
        transformChannel ( *rightChannel );

    /* tbd: mono
        if ( leftChannel )
            std::cerr << "leftChannel->size() = " << leftChannel->size() << std::endl;
        if ( rightChannel )
            std::cerr << "rightChannel->size() = " << rightChannel->size() << std::endl;
    */
}

template<class T>
void SlicePitchTransformer<T>::transformChannel( std::vector<T>& channel )
{
    size_t channelSize = channel.size();

    int guard = ( int ) channelSize - 2;
    if ( guard < 0 )
    {
        std::cerr << "slicepitchtransformer: guard < 0" << std::endl;
        return;
    }

    std::vector<T> result;

    bool *omitArray = transposeArrayVector.at( fabs ( pitchSemitones ) ).first;
    size_t omitArrayLength = transposeArrayVector.at( fabs ( pitchSemitones ) ).second;

    T currentSample, nextSample;

    size_t i = 0;

    for ( ; i < channelSize - 2; i++ )
    {
        currentSample = channel [ i ];
        nextSample = channel [ i + 1 ];

        if ( omitArray[ i % omitArrayLength ] )
            result.push_back ( currentSample );

        else
        {
            if ( pitchSemitones >= 0 )
                continue;
            result.push_back ( currentSample );
            result.push_back ( 0.5 * ( currentSample + nextSample ) );
        }
    }
    result.push_back ( channel [ channelSize - 2 ] );
    result.push_back ( channel [ channelSize - 1 ] );

    channel = result;
}

#endif
