#ifndef SLICERETRIGGERTRANSFORMER_H
#define SLICERETRIGGERTRANSFORMER_H

#include <string>
#include <cmath>
#include <iostream>

#include "enum.h"
#include "functor.h"
#include "samplemath.h"
#include "slicetransformer.h"
#include "slicegatetransformer.h"

template<class T>
class SliceRetriggerTransformer : public SliceTransformer<T>
{
public:
    SliceRetriggerTransformer (
        size_t triggerCount = 1,
        size_t crossFadeRange = 1024,
        size_t channels = 2 );
    void transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel );
    void transformChannel ( std::vector<T>& channel );

private:
    size_t triggerCount, crossFadeRange, channels;
};


template<class T>
SliceRetriggerTransformer<T>::SliceRetriggerTransformer
( size_t triggerCountParameter,
  size_t crossFadeRangeParameter,
  size_t channelsParameter ) :
        triggerCount ( triggerCountParameter ),
        crossFadeRange ( crossFadeRangeParameter ),
        channels ( channelsParameter ) { }

template<class T>
void SliceRetriggerTransformer<T>::transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel )
{
    transformChannel ( *leftChannel );
    if ( rightChannel )
        transformChannel ( *rightChannel );

    SliceGateTransformer<T> sgt (
        ( rightChannel ) ?
        leftChannel->size() + rightChannel->size() :
        leftChannel->size(),
        128,//64,
        channels );
    sgt.transform ( leftChannel, rightChannel );
}

template<class T>
void SliceRetriggerTransformer<T>::transformChannel ( std::vector<T>& channel )
{
    if ( triggerCount < 2 )
        return;
    // data >> sliceVector
    std::vector<std::vector<T> > sliceVector;
    std::vector<T> currentSlice;

    size_t channelSize, sliceSize, limit;
    channelSize = channel.size();
    sliceSize = channelSize / triggerCount;

    if ( ! ( ( 2 * crossFadeRange ) < sliceSize ) )
        return;

    limit = sliceSize + crossFadeRange;
    sliceVector.clear();

    for ( size_t i = 0; i < triggerCount; i ++ )
    {
        for ( size_t j = 0; j < limit; j++)
        {
            currentSlice.push_back ( channel.at ( j ) );
        }

        sliceVector.push_back ( currentSlice );
        currentSlice.clear();
    }

    // sliceVector >> data
    std::vector<T> result;

    size_t i, offset;
    offset = 0;
    size_t resultSize = triggerCount * sliceSize;
    for ( i = 0; i < resultSize; i++ )
        result.push_back ( 0.0 );

    for ( size_t j = 0; j < triggerCount; j++ )
    {
        i = 0;

        while ( i < crossFadeRange )
        {
            result [ offset + i ] +=
                ( j > 0 ) ?
                lConcaveSoftClipper( i, 0, crossFadeRange ) * sliceVector.at ( j ).at( i ) :
                sliceVector.at ( j ).at( i );
            i++;
        }

        while ( ( i >= crossFadeRange ) && ( i < sliceSize  ) )
        {
            result [ offset + i ] += sliceVector.at ( j ).at( i );
            i++;
        }

        if ( j < ( triggerCount - 1 )  )
        {
            while ( i < limit )
            {
                result [ offset + i ] +=
                    rConvexSoftClipper ( i, sliceSize, limit ) * sliceVector.at ( j ).at( i );
                i++;
            }
        }

        offset += sliceSize;
    }

    channel = result;

}

#endif
