#ifndef SLICEREVERBTRANSFORMER_H
#define SLICEREVERBTRANSFORMER_H

#include <cmath>

#include "dir.h"
#include "slicetransformer.h"
#include "wrapcsound.h"

//eal
#include <wx/wx.h>

template<class T>
class SliceReverbTransformer : public SliceTransformer<T>
{
public:
    SliceReverbTransformer (
        double roomsize = 0.5,
        double damp = 0.5,
        double dryMix = 0.05,
        size_t channels = 2 );
    void transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel );
    void transformChannel ( std::vector<T> *channel );
private:
    double roomsize, damp, dryMix;
    size_t channels;
};

template<class T>
SliceReverbTransformer<T>::SliceReverbTransformer (
    double roomsizeParameter,
    double dampParameter,
    double dryMixParameter,
    size_t channelsParameter ) :
        roomsize ( roomsizeParameter ),
        damp ( dampParameter ),
        dryMix ( dryMixParameter ),
        channels ( channelsParameter )
{
}


template<class T>
void SliceReverbTransformer<T>::transform (
    std::vector<T> *leftChannel,
    std::vector<T> *rightChannel )
{
    if ( channels == 1 )
        return;

    wxString cmd =
#ifndef __WXMSW__
        _T("csound ");
#else
        _T("csound.exe ");
#endif
    wxString dir = Dir::getCsdDir();
    dir.Replace ( _T(" "), _T("__SP__") );
    cmd += _T("-odac -idac -b 4096 -d -g ") + dir; // realtime
    cmd += _T("freeverb.csd");

    WrapCSound *wcs = new WrapCSound ( cmd );

    wcs->setDoubleParameter ( "reverb_dry", dryMix );
    wcs->setDoubleParameter ( "reverb_roomsize", roomsize );
    wcs->setDoubleParameter ( "reverb_damp", damp );

    if ( !wcs->perform( leftChannel, rightChannel ) )
    {
        std::cout << "!WrapCSound::perform" << std::endl;
        wcs->printMessages();
        return;
    }

    delete wcs;

}

#endif

