#ifndef SLICE_TAPESTOP_TRANSFORMER_H
#define SLICE_TAPESTOP_TRANSFORMER_H

#include <vector>
#include <algorithm>

#include "slicetransformer.h"


template<class T>
class SliceTapestopTransformer : public SliceTransformer<T>
{
public:
    SliceTapestopTransformer ( size_t channels = 2 );
    virtual ~SliceTapestopTransformer() {}
    void transform ( std::vector<T> *leftChannel, std::vector<T> *rightChannel );
    void transformChannel ( std::vector<T>& channel );
private:
    size_t channels;
};

template<class T>
SliceTapestopTransformer<T>::SliceTapestopTransformer ( size_t channelsParameter ) :
        channels ( channelsParameter )
{}

template<class T>
void SliceTapestopTransformer<T>::transform( std::vector<T> *leftChannel, std::vector<T> *rightChannel )
{
    transformChannel ( *leftChannel );
    if ( rightChannel )
        transformChannel ( *rightChannel );
}

template<class T>
void SliceTapestopTransformer<T>::transformChannel( std::vector<T>& channel )
{
    T currentSample, nextSample;
    std::vector<T> result;

    size_t channelSize = channel.size();
    size_t insertAt = 28;
    size_t insertCount = 0;
    size_t window = ( size_t ) channelSize / insertAt;
    size_t windowCount = 0;

    size_t sampleCount = 0;

    for ( size_t j = 0; j < channelSize - 2; j +=2, windowCount+=2, insertCount+= 2, sampleCount +=2 )
    {
        if ( sampleCount >= channelSize )
            break;
        currentSample = channel.at ( j );
        nextSample = channel.at ( j + 1 );

        result.push_back ( currentSample );


        if ( windowCount >= window )
        {
            if ( insertAt > 1 )
            {
                insertAt--;
                windowCount = 0;
            }
        }

        if ( insertCount >= insertAt )
        {
            result.push_back ( 0.5 * ( currentSample + nextSample ) );
            sampleCount ++;
            insertCount = 0;
        }

        result.push_back ( nextSample );
    }

    channel = result;
}

#endif

