#ifndef SLICEVECTORTRANSFORMER_H
#define SLICEVECTORTRANSFORMER_H

#include <vector>

#include "slicetransformer.h"

template<class T>
class SliceVectorTransformer
{
public:
    SliceVectorTransformer<T> (
        const std::vector<size_t>& positionVectorParameter,
        SliceTransformer<T> *transformerParameter );
    void vectorTransform ( std::vector<std::vector<T> > *sliceVectorL, std::vector<std::vector<T> > *sliceVectorR );
private:
    std::vector<size_t> positionVector;
    SliceTransformer<T> *transformer;
};

template<class T>
SliceVectorTransformer<T>::SliceVectorTransformer (
    const std::vector<size_t>& positionVectorParameter,
    SliceTransformer<T> *transformerParameter) :
        positionVector ( positionVectorParameter ),
        transformer ( transformerParameter ) { }

template<class T>
void SliceVectorTransformer<T>::vectorTransform ( std::vector<std::vector<T> > *sliceVectorL, std::vector<std::vector<T> > *sliceVectorR )
{
    std::vector<size_t>::iterator it;

    for ( it = positionVector.begin(); it != positionVector.end(); it++ )
    {
        if ( *it < sliceVectorL->size() )
        {
            transformer->transform ( & (sliceVectorL->at ( *it ) ),
                                     ( sliceVectorR ) ? & (sliceVectorR->at ( *it ) ) : NULL );
        }
    }
}

#endif

