#include <wx/string.h>
#include <wx/tokenzr.h>
#include <wx/url.h>
#include <wx/msgdlg.h>
#include <wx/filename.h>
#include <iostream>
#include "smasherdb.h"
#include "binaryfile.h"
#include "dir.h"
#include "urlprovider.h"
#include "httpresource.h"

bool SmasherDb::getTitles ( wxString& s )
{
    s.clear();
    UrlProvider up;
    std::string stdBaseUrl = up.getRefreshUrl();
    wxString address = wxString ( stdBaseUrl.c_str(), wxConvUTF8, stdBaseUrl.size() );
    HttpResource hr;
    if ( !hr.load ( address, &s ) )
        return false;
    return true;
}

bool SmasherDb::getPreset ( wxString& title, wxString& author, wxString& preset )
{
    title.Replace ( _T(" "), _T("%20") );
    author.Replace ( _T(" "), _T("%20") );

    UrlProvider up;
    std::string stdAddress = up.getDownloadUrl();
    wxString address = wxString ( stdAddress.c_str(), wxConvUTF8, stdAddress.size() );
    address += _T("?title=") +
               title +
               _T("&author=") +
               author;

    wxString buffer, message;
    HttpResource hr;
    if ( !hr.load ( address, &buffer ) )
    {
        message = buffer;
        message.Remove ( 0, 1 );
        return false;
    }

    wxStringTokenizer tokenizer( buffer, _T(",") );
    std::vector<wxString> tokenVector;

    while ( tokenizer.HasMoreTokens() )
    {
        tokenVector.push_back ( tokenizer.GetNextToken() );
    }

    if ( tokenVector.size() < 2 )
    {
        return false;
    }

    wxString sample, state;
    sample = Dir::getAppDir() + _T("samples") +
             wxFileName::GetPathSeparator() +
             tokenVector[0];
    state = tokenVector[1];

    preset = _T( "fileIn=" ) + sample + _T("|") + state;
    return true;
}

bool SmasherDb::sharePreset (
    const wxString& title,
    const wxString& sample,
    const wxString& state,
    const wxString& author,
    const wxString& passcode,
    wxString& message)
{
    // remove path from sample
    wxFileName fn ( sample );
    wxString shortSample = fn.GetFullName();

    UrlProvider up;
    std::string stdAddress = up.getShareUrl();
    wxString address = wxString ( stdAddress.c_str(), wxConvUTF8, stdAddress.size() );
    address += _T("?");

    address += _T("title=") + title + _T("&");
    address += _T("sample=") + shortSample + _T("&");
    address += _T("author=") + author + _T("&");
    address += _T("passcode=") + passcode + _T("&");

    //strip 1st state segment
    wxString myState = state;

    int offset = myState.Find ( _T("|") );
    if ( offset ==  wxNOT_FOUND )
        return false;
    myState = myState.Mid ( (size_t) offset );

    myState.Replace ( _T("|"), _T("_P_") );
    myState.Replace ( _T("="), _T("_E_") );
    myState.Replace ( _T("#"), _T("_H_") );

    size_t stateLength, chunkLength;
    stateLength = myState.length();
    chunkLength = myState.length() / 36;

    if ( chunkLength > 96 )
        return false;

    size_t chunkCounter, chunkIterator;
    chunkCounter = 1;
    chunkIterator = 0;
    wxString chunkLabel;

    chunkLabel.Printf ( _T("state%i"), chunkCounter++ );
    address << chunkLabel << _T("=");
    for ( size_t i = 0; i < stateLength; i++ )
    {
        address += myState[i];
        chunkIterator++;
        if ( chunkIterator >= chunkLength && chunkCounter <= 36)
        {
            chunkLabel.Printf ( _T("&state%i"), chunkCounter++ );
            address << chunkLabel << _T("=");
            chunkIterator = 0;
        }
    }

    address.Replace ( _T(" "), _T("%20") );

    wxString buffer;
    HttpResource hr;
    if ( !hr.load ( address, &buffer ) )
    {
        message = hr.getError();
        message.Remove ( 0, 1 );
        return false;
    }

    if ( buffer.size() > 0 && buffer[0] == L'!' )
    {
        message = buffer;
        message.Remove ( 0, 1 );
        return false;
    }

    return true;
}


bool SmasherDb::ratePreset (
    const int rating,
    const wxString& title,
    const wxString& user,
    const wxString& passcode,
    wxString& message )
{
    UrlProvider up;
    std::string stdAddress = up.getRateUrl();
    wxString address = wxString ( stdAddress.c_str(), wxConvUTF8, stdAddress.size() );
    address += _T("?");

    address += _T("rating=");
    address << rating;
    address += _T("&preset=") + title;
    address += _T("&user=") + user;
    address += _T("&passcode=") + passcode;
    address.Replace ( _T(" "), _T("%20") );

    wxString buffer;
    HttpResource hr;
    if ( !hr.load ( address, &buffer ))
    {
        message = hr.getError();
        message.Remove ( 0, 1 );
        return false;
    }

    if ( buffer.size() > 0 && buffer[0] == L'!' )
    {
        //tbd: delete !
        message = buffer;
        message.Remove ( 0, 1 );
        return false;
    }

    return true;
}
