#include "undohistory.h"
#include <wx/msgdlg.h>

UndoHistory::UndoHistory() {}

void UndoHistory::initialize ( const wxString& state )
{
    history.clear();
    history.push_back ( state );
    index = 0;
}

void UndoHistory::pushState ( const wxString& state )
{
    if ( index + 1 < history.size() )
    {
        size_t toPop = history.size() - 1 - index;
        while ( toPop > 0 )
        {
            history.pop_back();
            toPop--;
        }
    }
    history.push_back ( state );
    index++;
}

wxString UndoHistory::undo()
{
    if ( !canUndo() )
        return wxEmptyString;
    return history.at ( --index );
}

wxString UndoHistory::redo()
{
    if ( !canRedo() )
        return wxEmptyString;
    return history.at ( ++index );
}

bool UndoHistory::canUndo()
{
    return ( index > 0 );
}

bool UndoHistory::canRedo()
{
    return ( index < ( history.size() - 1 ) );
}
