#include "wx/pen.h"
#include <wx/window.h>

#include "samplemath.h"
#include "wavewindow.h"

#include <iostream>

BEGIN_EVENT_TABLE( WaveWindow, /*wxWindow*/ wxScrolledWindow )
    EVT_PAINT( WaveWindow::OnPaint )
    EVT_SIZE ( WaveWindow::OnSize )
    EVT_IDLE ( WaveWindow::OnIdle )
END_EVENT_TABLE()

WaveWindow::WaveWindow ( wxWindow *parent, int id, const wxPoint& pos, const wxSize& size ) :
        /*wxWindow*/ wxScrolledWindow ( parent, id, pos, size, wxHSCROLL )
{
    data = NULL;
    SetBackgroundColour ( *wxWHITE );
    idleCounter = 0;

    //EnableScrolling( true, false );//tbd
    //SetScrollbars( 20, 0, 100, 0 );
}

void WaveWindow::prepareDC ( wxClientDC& dc )
{
    dc.SetPen( *wxBLACK_PEN );
    dc.SetAxisOrientation( true, true );
    dc.SetDeviceOrigin( 0, height );
}

void WaveWindow::set( std::basic_string<double> *dataParameter )
{
    if ( !dataParameter )
    {
        //std::cout << "no plot data" << std::endl;
        return;
    }

    data = dataParameter;

    wxClientDC dc ( this );
    prepareDC ( dc );

    if ( !points.empty() )
    {
        wxPen pen ( ( const wxColour ) GetBackgroundColour() );
        dc.SetPen ( pen );
        int x1, y1, y2;
        for ( size_t i = 0; i < ( points.size()/ 2 ) + 1 ; i++ )
        {
            x1 = points.at ( i ).x;
            y1 = points.at ( i ).y;
            y2 = height/2 - ( y1 + 1 - height/2 );
            dc.DrawLine( x1, y1, x1, y2);
        }

        dc.DrawLines ( ( int ) points.size(), ( wxPoint* ) points.data() );
    }

    DoPaint( dc );

}

void WaveWindow::DoPaint( wxClientDC& dc )
{
    updatePoints( dc );

    dc.SetPen ( *wxBLACK_PEN );
    int x1, y1, y2;
    for ( size_t i = 0; i < ( points.size() )/ 2 + 1; i++ )
    {
        x1 = points.at ( i ).x;
        y1 = points.at ( i ).y;
        y2 = height/2 - ( y1 + 1 - height/2 );
        dc.DrawLine( x1, y1, x1, y2);
    }

    wxPen pen ( wxColour ( 222, 222, 222, wxALPHA_TRANSPARENT ));
    dc.SetPen ( pen );
    dc.DrawLines ( ( int ) points.size(), ( wxPoint* ) points.data() );
}

void WaveWindow::OnIdle ( wxIdleEvent& event )
{
    /*if ( ++idleCounter % 16 )
        return;*/

    GetClientSize ( &width, &height );

    if ( ( height <= 0 ) || ( width <= 0 ) )
    {
        //std::cout << "wavewindow size out of bounds" << std::endl;
        return;
    }

    if ( ( height != heightMemory ) || ( width != widthMemory ) )
        set( data );

    heightMemory = height;
    widthMemory = width;
}

void WaveWindow::OnPaint( wxPaintEvent& event )
{
    if ( !points.empty() )
    {
        wxClientDC dc ( this );
        prepareDC ( dc );
        DoPaint( dc );
    }
    event.Skip();
}

void WaveWindow::OnSize( wxSizeEvent& event )
{
    if ( !points.empty() )
        set ( data );

    event.Skip();
}

void WaveWindow::updatePoints( wxClientDC& dc )
{
    GetClientSize ( &width, &height );
    dc.SetDeviceOrigin( 0, height );

    if ( height <= 0 )
    {
        //std::cout << "wavewindow height out of bounds" << std::endl;
        return;
    }

    points.clear();

    size_t xOffset =
#ifdef __WXMSW__
        85;//75;
#else
        95;
#endif

    if ( !data )
        return;

    for ( size_t i = 0 + xOffset; i < data->size() + xOffset; i ++ )
        points.push_back( wxPoint ( i, ( int ) ( 0.5 * ( double ) height * ( 1.0 + data->at( i - xOffset ) ) ) ));

    for ( size_t i = data->size() - 1 + xOffset; i >= 0 + xOffset; i -- )
        points.push_back( wxPoint ( i, ( int ) ( 0.5 * ( double )height * ( 1.0 - data->at( i - xOffset ) ) ) ));

    points.push_back( wxPoint ( xOffset, ( int ) ( 0.5 * ( double )height * ( 1.0 + data->at( ( 0 ) ) ) )));
}

void WaveWindow::drawCrossHair( const wxCoord& x, const wxCoord& y )
{
    wxClientDC dc ( this );
    prepareDC ( dc );

    size_t xOffset =
#ifdef __WXMSW__
        85;//75;
#else
        95;
#endif
    dc.SetPen (*wxBLACK_PEN);
    dc.CrossHair ( xOffset + x, y );
}

void WaveWindow::deleteCrossHair( const wxCoord& x, const wxCoord& y )
{
    wxClientDC dc ( this );
    prepareDC ( dc );

    size_t xOffset =
#ifdef __WXMSW__
        85;//75;
#else
        95;
#endif
    dc.SetPen (*wxWHITE_PEN);
    dc.CrossHair ( xOffset + x, y );
    DoPaint( dc );
}
