#include "wrapcsound.h"
//#include <csound/csound.h>
#include <csound/csound.hpp>
#include <wx/tokenzr.h>
#include <wx/string.h>
#include <wx/utils.h>
#include <wx/wx.h>//eal
#include <wx/filename.h>
#include <iostream>
#include "samplemath.h"
#include "sndfile.hh"
#include "dir.h"

WrapCSound::WrapCSound( const wxString& cmd, const wxString& tempFile ) : success ( false )
{
    wxStringTokenizer tokenizer ( cmd, _T(" ") );
    std::vector<std::string> tokenV;
    wxString wideToken;
    std::string narrowToken;
    while ( tokenizer.HasMoreTokens() )
    {
        wideToken = tokenizer.GetNextToken();

        wideToken.Replace ( _T("__SP__"), _T(" ") );

        narrowToken = wideToken.mb_str ( wxConvUTF8 );
        tokenV.push_back ( narrowToken );
    }

    int myArgc = tokenV.size();

    if ( myArgc > 31 )
    {
        return;
    }

    char *myArgv[myArgc + 1];
    for ( size_t i = 0; i < tokenV.size(); i++ )
        myArgv[i] = ( char *) ( ( tokenV.at ( i )).c_str());
    myArgv[myArgc] = NULL;

    csound = csoundCreate ( NULL );

    csoundEnableMessageBuffer ( csound, 0 );

    if ( csoundPreCompile ( csound ) != CSOUND_SUCCESS )
    {
        std::cout << "!csoundPreCompile" << std::endl;
        return;
    }

    csoundSetHostImplementedAudioIO ( csound, 1, 4096 );	//16384 );//8192 );//512 );//16384 );//512 );

    if ( csoundCompile ( csound, myArgc, myArgv ) )
    {
        std::cout << "!WrapCSound::csoundCompile" << std::endl;
        printMessages();
        return;
    }
    doubleParameterVector.clear();

    success = true;
    return;
}

WrapCSound::~WrapCSound()
{
    if ( !success || !csound )
        return;

    csoundDestroyMessageBuffer ( csound );
    //csoundCleanup ( csound );
    //csoundReset ( csound );
    //csoundDestroy ( csound );
}

bool WrapCSound::perform( std::vector<double> *leftChannel, std::vector<double> *rightChannel )
{
    if ( !success || !leftChannel || !rightChannel )
        return false;

    std::vector<double> *inSlice = new std::vector<double>;

    SampleMath::joinChannels (
        leftChannel,
        rightChannel,
        inSlice );

    /*
        size_t parameterVectorSize = doubleParameterVector.size();
        double parameterValue;
        std::string parameterName;
        for ( size_t i = 0; i < parameterVectorSize; i++ )
        {
            parameterName = doubleParameterVector.at ( i ).first;
            parameterValue = doubleParameterVector.at ( i ).second;

            double *doublePtr;
            if ( csoundGetChannelPtr ( csound, &doublePtr, parameterName.c_str(), CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL ) == 0 )
                *doublePtr = parameterValue;
        }
    */

    long inBufferLimit, outBufferLimit, inBufferIndex, outBufferIndex, offset, sliceSize, i;
    double *inBuffer, *outBuffer;

    std::vector<double> *outSlice = new std::vector<double>;


    inBuffer = outBuffer = NULL;
    inBufferLimit = outBufferLimit = inBufferIndex = outBufferIndex = offset = i = 0;

    inBufferLimit = csoundGetInputBufferSize ( csound );
    outBufferLimit = csoundGetOutputBufferSize ( csound );
    inBuffer = csoundGetInputBuffer( csound );
    sliceSize = ( long ) inSlice->size();

    for ( i = 0; i < inBufferLimit && i < sliceSize; i++, inBufferIndex++ )
        * ( inBuffer + i ) = inSlice->at ( inBufferIndex );

    //begin eal
    size_t parameterVectorSize = doubleParameterVector.size();
    double parameterValue;
    std::string parameterName;
    double *doublePtr;
    //end eal

    while ( !csoundPerformBuffer ( csound ) )
    {
        //begin eal
        for ( i = 0; i < parameterVectorSize; i++ )
        {
            parameterName = doubleParameterVector.at ( i ).first;
            parameterValue = doubleParameterVector.at ( i ).second;
            if ( csoundGetChannelPtr ( csound, &doublePtr, parameterName.c_str(), CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL ) == 0 )
                *doublePtr = parameterValue;
        }
        //end eal

        outBuffer = csoundGetOutputBuffer( csound );

        if ( outBufferIndex > ( sliceSize - 1 ) )
            break;

        for (
            i = 0;
            i < outBufferLimit && outBufferIndex < sliceSize;
            i++, outBufferIndex++ )
        {
            outSlice->push_back ( * ( outBuffer + i ) );
        }

        if ( inBufferIndex > ( sliceSize - 1 ) )
            break;

        for (
            i = 0;
            i < inBufferLimit && inBufferIndex < sliceSize;
            i++, inBufferIndex++ )
        {
            * ( inBuffer + i ) = inSlice->at ( inBufferIndex );
        }
    }

    SampleMath::splitChannels ( outSlice, leftChannel, rightChannel );

    //tbd: delete slice vectors
    delete inSlice;
    delete outSlice;

    return true;
}

void WrapCSound::setDoubleParameter ( const std::string& name, double value )
{
    doubleParameterVector.push_back ( std::make_pair ( name, value ) );
}

void WrapCSound::printMessages()
{
    if ( !csoundGetMessageCnt( csound ) )
    {
        std::cout << "[no messages]" << std::endl;
        return;
    }

    while ( csoundGetMessageCnt( csound ) > 0 )
    {
        std::cout << csoundGetFirstMessage ( csound );// << std::endl;
        csoundPopFirstMessage( csound );
    }
}
