#include <stdexcept>
#include <wx/file.h>
#include "wraptempfilename.h"

WrapTempFileName::WrapTempFileName ( wxString hint, wxString extension )
{
    keepFiles = false;
    wxFile file;
    myTempFileName = wxFileName::CreateTempFileName ( hint, &file );
    file.Close();

    if ( !extension.empty() )
    {
        myTempFileNameOriginal = myTempFileName;
#ifdef __WXMSW__
        myTempFileName.Replace ( _T ( ".tmp" ), extension );
#else
        myTempFileName += extension;
#endif
    }
}

WrapTempFileName::~WrapTempFileName()
{
    if ( keepFiles )
        return;
    wxRemoveFile ( myTempFileName );

    if ( !myTempFileNameOriginal.empty() )
        wxRemoveFile ( myTempFileNameOriginal );
}

wxString WrapTempFileName::wideName()
{
    return myTempFileName;
}

std::string WrapTempFileName::name()
{
    std::string tmp ( ( const char * ) myTempFileName.mb_str ( wxConvLocal ) );
    return tmp;
}

wxString WrapTempFileName::originalWideName()
{
    return myTempFileNameOriginal;
}

std::string WrapTempFileName::originalName()
{
    std::string tmp ( ( const char* ) myTempFileNameOriginal.mb_str ( wxConvLocal ) );
    return tmp;
}

void WrapTempFileName::setKeepFiles ( bool b )
{
    keepFiles = b;
}
